﻿---------------------------------------------------------

if exists ( select * From sysobjects where id= object_id ( 'dbo.A_RegistrarSession' ))
	Drop table dbo.A_RegistrarSession
GO

Create table dbo.A_RegistrarSession (
	Id int identity( 1, 1 ) Primary key,
	CallId nvarchar(50),
	Account nvarchar(100),
	RecvAddr nvarchar(50),
	VIA nvarchar(200),
	Nonce nvarchar(50),
	NC nvarchar(50),
	StartTimeX nvarchar(50),
	LocalTimeCreate datetime,
	LocalTimeUpdate datetime
)
GO

---------------------------------------------------------

if exists ( select * from sysobjects where id= object_id ( 'dbo.A_RegistrarContact' ))
	Drop table dbo.A_RegistrarContact
GO

Create table dbo.A_RegistrarContact (
	Id int identity(1,1) primary key,
	SessionId int,
	URI nvarchar(100),
	Type nvarchar(50),
	Expires int,
	LocalTimeExpire datetime
)
GO

---------------------------------------------------------

if exists (select * from dbo.sysindexes where name = N'idx_RegistrarSession_CallId' and id = object_id(N'[dbo].[A_RegistrarSession]'))
	Drop index [dbo].[A_RegistrarSession].[idx_RegistrarSession_CallId]
GO

Create Nonclustered Index [idx_RegistrarSession_CallId] 
On [dbo].[A_RegistrarSession]([CallId]) 
With FillFactor = 80 ON [PRIMARY]
GO

---------------------------------------------------------

if exists (select * from dbo.sysindexes where name = N'idx_A_RegistrarContact_SessionId' and id = object_id(N'[dbo].[A_RegistrarContact]'))
	Drop index [dbo].[A_RegistrarContact].[idx_A_RegistrarContact_SessionId]
GO

Create Nonclustered Index [idx_A_RegistrarContact_SessionId] 
On [dbo].[A_RegistrarContact]([SessionId]) 
With FillFactor = 80 ON [PRIMARY]
GO

---------------------------------------------------------

if exists (select * from dbo.sysindexes where name = N'idx_A_RegistrarContact_TimeFilter' and id = object_id(N'[dbo].[A_RegistrarContact]'))
	Drop index [dbo].[A_RegistrarContact].[idx_A_RegistrarContact_TimeFilter]
GO

Create Nonclustered Index [idx_A_RegistrarContact_TimeFilter] 
On [dbo].[A_RegistrarContact]([LocalTimeExpire]) 
With FillFactor = 80 ON [PRIMARY]
GO

---------------------------------------------------------
---------------------------------------------------------
---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_FilterSessions'))
	Drop procedure dbo.A_Registrar_FilterSessions
GO

Create procedure dbo.A_Registrar_FilterSessions
as
begin
	Declare @dtnow datetime
	Set @dtnow = GetDate()

	Delete
	From A_RegistrarContact
	Where LocalTimeExpire < @dtnow

	Delete
	From A_RegistrarSession
	Where not exists ( Select * From A_RegistrarContact c Where c.SessionId = A_RegistrarSession.Id )
end
GO

---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_DropSession'))
	Drop procedure dbo.A_Registrar_DropSession
GO

Create procedure dbo.A_Registrar_DropSession
	@sessionid int = -1,
	@account nvarchar(100) = '',
	@callid nvarchar(100) = ''	
as
begin
	
	Declare @id int		
	Set @id = @sessionid

	if ( @id <= 0 ) begin	
	    Select @id = Id 
	    From A_RegistrarSession
	    Where ( CallId like @callid )
			and ( Account like @account )
	end

	Delete 
	From A_RegistrarSession
	Where Id = @sessionid

	Delete 
	From A_RegistrarContact
	Where SessionId = @sessionid

end
GO

---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_UpdateSession'))
	Drop procedure dbo.A_Registrar_UpdateSession
GO

Create procedure dbo.A_Registrar_UpdateSession
	@sessionid int = -1,
	@callid nvarchar(50),
	@account nvarchar(100),
	@recvaddr nvarchar(50),
	@via nvarchar(200),
	@nonce nvarchar(50),
	@nc nvarchar(50),
	@starttimex nvarchar(50)
as
begin
	Declare @dtnow Datetime
	Set @dtnow = GetDate()
	
	if ( @sessionid > 0 ) begin
	    Update A_RegistrarSession
	    Set Nonce = @nonce,
			NC = @nc,
			StartTimeX = @starttimex,
			LocalTimeUpdate = @dtnow	
	    Where Id = @sessionid
	end

	if (( @@rowcount > 0 ) and ( @sessionid > 0 ))

		Select @sessionid

	else begin

		Insert into A_RegistrarSession ( CallId, Account, RecvAddr, VIA, Nonce, NC, StartTimeX, LocalTimeCreate, LocalTimeUpdate )
		Values ( @callid, @account, @recvaddr, @via, @nonce, @nc, @starttimex, @dtnow, @dtnow )

		Select @@identity
	end
end
GO

---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_UpdateContact'))
	Drop procedure dbo.A_Registrar_UpdateContact
GO

Create procedure dbo.A_Registrar_UpdateContact
	@sessionid int,
	@uri nvarchar(100),
	@type nvarchar(50),
	@expires int	
as
begin
	
	Declare @dtnow datetime, @localtimeexpire datetime
	Set @dtnow = GetDate()
	Set @localtimeexpire = DateAdd ( ss, @expires, @dtnow )

	Update A_RegistrarContact
	Set 	Expires = @expires,		
		LocalTimeExpire = @localtimeexpire
	Where SessionId = @sessionid
		And URI like @uri

	if ( @@rowcount = 0 ) begin

		Insert into A_RegistrarContact ( SessionId, URI, Type, Expires, LocalTimeExpire )
		Values ( @sessionid, @uri, @type, @expires, @localtimeexpire )
	end
end
GO

---------------------------------------------------------

if exists ( select * FROM sysobjects where id=object_id ( 'dbo.A_Registrar_GetData'))
	Drop procedure dbo.A_Registrar_GetData
GO

Create procedure dbo.A_Registrar_GetData
as
begin
	Select CallId, Account, RecvAddr, VIA, Nonce, NC, StartTimeX,
		URI, Type, Expires
	From A_RegistrarSession s
		Inner join A_RegistrarContact c on c.SessionId = s.Id
	Where c.LocalTimeExpire > GetDate()
	Order by Account, CallId
end
GO

---------------------------------------------------------
---------------------------------------------------------
---------------------------------------------------------

--exec A_Registrar_UpdateSession @sessionid = -1, @callid = 'callid', @account = 'account', @recvaddr = 'recvaddr', @via = 'via', @nonce = 'nonce', @nc = 'nc', @starttimex = 'starttimex'
--exec A_Registrar_UpdateContact @sessionid = 10, @uri = 'uri', @type = 'type', @expires = 30
--exec A_Registrar_UpdateSession @sessionid = 9, @callid = 'callid', @account = 'account', @recvaddr = 'recvaddr', @via = 'via', @nonce = 'nonce', @nc = 'nc', @starttimex = 'starttimex'
--exec A_Registrar_UpdateContact @sessionid = 9, @uri = 'uri', @type = 'type', @expires = 30
--exec A_Registrar_FilterSessions
--exec A_Registrar_GetData
--Select * From A_RegistrarSession s Inner join A_RegistrarContact c on c.SessionId = s.Id
--Select * FROM A_RegistrarSession
--Select * FROM A_RegistrarContact

