﻿-------------------------------------------------------------------------

if not exists ( select * from sysobjects where id=object_id ( 'dbo.A_UserMessageData' ))	
Create Table dbo.A_UserMessageData
(
	[Id] int identity ( 1, 1 ) primary key, 
	[IdUserA] uniqueidentifier,
	[SentDate] DateTime,
	[Message] Text COLLATE database_default,
	[IsConf] int default 0
)
GO


if not exists ( select * from sysobjects where id=object_id ( 'dbo.A_UserMessageLink' ))	
Create Table dbo.A_UserMessageLink
(
	[Id] uniqueidentifier primary key,
	[IdMessageData] int,
	[IdUserB] uniqueidentifier,
	[Viewed] bit,
	[ViewedDate] datetime			
)
GO

if not exists ( select * from sysobjects where id=object_id ( 'dbo.A_UserMessageFile' ))	
Create Table dbo.A_UserMessageFile
(
	[id] uniqueidentifier primary key,
	[sentdate] datetime not null,
	[idmessagedata] int not null,
	[filepath] nvarchar(4000),
	[filesize] int
)
GO

--------------------------------------------------------------------------
--------------------------------------------------------------------------
-------------------------------------------------------------------------

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserMessages' ))
    If not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_UserMessages' 
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME like 'idx' )
	Alter Table dbo.A_UserMessages
	Add [idx] int default 0
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserMessages' ))
begin

    -- Сначала дубликатные строки удаляем, по одной на идентификатор

	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#t2' ))
		Drop table #t2
	
	Select * into #t2
	From A_UserMessages 
	Where Id in ( Select id FROM A_UserMessages 
		Group by id
		Having COUNT(*) > 1 )
		
	Delete 
	From A_UserMessages
	Where Id in ( Select Id from #t2 )
	
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#t3' ))
		Drop table #t3
				
	Create table #t3 
	(
		Id Uniqueidentifier,
		IdUserA Uniqueidentifier, 
		IdUserB Uniqueidentifier, 
		SentDate datetime,
		Message ntext,
		Viewed bit,
		ViewedDate DateTime,
		IsConf int,
		idx int,
		idxfilter int identity ( 1, 1 )
	)
	
	Insert into #t3 ( Id, IdUserA, IdUserB, SentDate, Message, Viewed, ViewedDate, IsConf )
		Select Id, IdUserA, IdUserB, SentDate, Message, Viewed, ViewedDate, IsConf
		From #t2 
		Order by SentDate
		
	Insert into A_UserMessages	
		Select t3.Id, IdUserA, IdUserB, SentDate, Message, Viewed, ViewedDate, IsConf, 4
		From #t3 t3 
			Inner join ( Select Id, MIN (idxfilter) as idxfilter
						 From #t3
						 Group by Id ) t on t3.idxfilter = t.idxfilter

    -----------------------------------------
    -- проставляем индексы будущие в соответствии с датами

	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#t' ))
		Drop table #t

	Create table #t
	(
		idx int identity ( 1, 1 ),
		Id uniqueidentifier	
	)

	Insert into #t ( id )
		Select id from A_UserMessages
		Order by sentdate

	Update um
	Set um.idx = t.idx
	From A_UserMessages um, #t t
	Where um.Id = t.id

end
GO

---------------------------------------------------------------------
If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserMessages' ))
begin

	Truncate table dbo.A_UserMessageData

	Set Identity_Insert dbo.A_UserMessageData On 
	
	Insert into dbo.A_UserMessageData ( Id, IdUserA, SentDate, Message, IsConf )
		Select idx, IdUserA, SentDate, Message, IsConf
		From A_UserMessages
		Order by idx

	Set Identity_Insert dbo.A_UserMessageData Off


	Truncate table dbo.A_UserMessageLink

	Insert into dbo.A_UserMessageLink ( [Id], [IdMessageData], [IdUserB], [Viewed], [ViewedDate] )
		Select /*id*/NewID(), idx, IdUserB, Viewed, ViewedDate
		From A_UserMessages
		Order by idx


	If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserMessageFiles' )) begin

		Truncate table dbo.A_UserMessageFile

		Insert into dbo.A_UserMessageFile ( id, sentdate, idmessagedata, filepath, filesize )
			Select umf.id, umf.sentdate, um.idx, umf.filepath, umf.filesize
			From A_UserMessageFiles umf
				Inner join A_UserMessages um on um.Id = umf.IdMessage
	end

	Drop table dbo.A_UserMessages	
end
GO

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_UserMessageFiles' ))
	Drop table dbo.A_UserMessageFiles
GO
--------------------------------------------------------------------------
--------------------------------------------------------------------------
--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessageFiles_Add' ))
	Drop procedure dbo.[A_UserMessageFiles_Add]
GO

Create procedure [dbo].[A_UserMessageFiles_Add]
	@id uniqueidentifier,
	@sentdate datetime,
	@idmessagedata int,
	@filepath nvarchar(4000),
	@filesize int
as	
begin
	Insert into A_UserMessageFile (id, sentdate, idmessagedata, filepath, filesize )
	Values ( @id, @sentdate, @idmessagedata, @filepath, @filesize )
end
GO


--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessageFiles_Remove' ))
	Drop procedure dbo.[A_UserMessageFiles_Remove]
GO

Create procedure [dbo].[A_UserMessageFiles_Remove]
	@id uniqueidentifier
as	
begin
	Delete 
	From A_UserMessageFile
	Where id = @id
end
GO

---------------------------------------------------------------------------
--------------------------------------------------------------------------
--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessages_AddData' ))
	Drop procedure dbo.A_UserMessages_AddData
GO	

Create Procedure dbo.A_UserMessages_AddData
	@idusera	uniqueidentifier,
	@sentdate	datetime,
	@message	text,
	@isconf		int = 0
as
begin

	Insert into A_UserMessageData ( IdUserA, SentDate, [Message], IsConf )
	Values ( @idusera, @sentdate, @message, @isconf )

	Select @@identity
	
end
GO

--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessages_AddLink' ))
	Drop procedure dbo.A_UserMessages_AddLink
GO	

Create Procedure dbo.A_UserMessages_AddLink
	@id uniqueidentifier,
	@iduserb uniqueidentifier,
	@idmessagedata int,
	@isconf int = 0,
	@sentdate datetime
as
begin

	Declare @isviewed bit, @vieweddate datetime
	if ( @isconf > 0 )
 	  Select @isviewed = 1, @vieweddate = @sentdate
	else
	  Select @isviewed = 0, @vieweddate = null

	Insert into A_UserMessageLink ( Id, IdUserB, IdMessageData, Viewed, ViewedDate )
	Values ( @id, @iduserb, @idmessagedata, @isviewed, @vieweddate )

end
GO

--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessages_Add' ))
	Drop procedure dbo.A_UserMessages_Add
GO	

--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessages_GetHistory' ))
	Drop procedure dbo.[A_UserMessages_GetHistory]
GO	

Create procedure [dbo].[A_UserMessages_GetHistory]
	@dtstart DateTime,
	@dtstop  DateTime,
	@searchtext nvarchar ( 100 ),
	@iduser uniqueidentifier,
	@idopponent uniqueidentifier
as
begin
	declare @ids table ( Id uniqueidentifier )

	if (( @idopponent != '00000000-0000-0000-0000-000000000000' ) and ( @searchtext != '' ))

		Insert into @ids
			Select uml.Id
			From A_UserMessageData umd
				Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and ((IdUserA = @iduser and IdUserB = @idopponent) 
					or (IdUserB = @iduser and IdUserA = @idopponent))
--					or (IsConf = 1 and IdUserB = @idopponent
				and [Message] like '%' + @searchtext + '%'

	else if ( @idopponent != '00000000-0000-0000-0000-000000000000' )

		Insert into @ids
			Select uml.Id
			From A_UserMessageData umd
				Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and ((IdUserA = @iduser and IdUserB = @idopponent) 
					or (IdUserB = @iduser and IdUserA = @idopponent))

	else if ( @searchtext != '' )

		Insert into @ids
			Select uml.Id
			From A_UserMessageData umd
				Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and (IdUserA = @iduser or IdUserB = @iduser)
				and [Message] like '%' + @searchtext + '%'

	else

		Insert into @ids
			Select uml.Id
			From A_UserMessageData umd
				Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
			Where SentDate > @dtstart 
				and SentDate < @dtstop 
				and (IdUserA = @iduser or IdUserB = @iduser)			
	


	-- Первая выборка - сами сообщения
	Select uml.Id, IdUserA, IsNull ( u1.Name, IsNull ( c1.Name, '???' )) as UserAName, IdUserB, IsNull ( u2.Name, IsNull ( c2.Name, '???' )) as UserBName, SentDate, [Message], Viewed, ViewedDate, IsConf
	From A_UserMessageData umd
		Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Inner join @ids ids on uml.Id = ids.Id
		Left join A_Users u1 on umd.IdUserA = u1.Id
		Left join A_Users u2 on uml.IdUserB = u2.Id
		Left join A_Stat_Conference c1 on ( umd.IsConf = 1 and umd.IdUserA = c1.Id )
		Left join A_Stat_Conference c2 on ( umd.IsConf = 1 and uml.IdUserB = c2.Id )
	Order by SentDate


	-- Вторая выборка - файлы к сообщениям
	Select umf.Id, umf.SentDate, uml.Id, umf.FilePath, umf.FileSize
	From A_UserMessageData umd
		Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Inner join @ids ids on uml.Id = ids.Id
		Inner join A_UserMessageFile umf on umf.IdMessageData = umd.Id and umf.SentDate = umd.SentDate
	Order by umf.SentDate
end
GO


--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessages_GetLastForUser' ))
	Drop procedure dbo.[A_UserMessages_GetLastForUser]
GO	

Create Procedure [dbo].[A_UserMessages_GetLastForUser]
	@idusera	uniqueidentifier,
	@iduserb	uniqueidentifier,
	@count		int
as
begin
	Declare @tmp table ( 
		Inc	int identity ( 1, 1 ),
		Id	uniqueidentifier,
		IdMessageData int,
		IdUserA uniqueidentifier,
		IdUserB	uniqueidentifier,
		SentDate DateTime,
		[Message] text collate database_default,
		Viewed bit,
		ViewedDate DateTime,
		IsConf int )

	Insert into @tmp ( Id, IdMessageData, IdUserA, IdUserB, SentDate, Message, Viewed, ViewedDate, IsConf )
		Select Top 100 uml.Id, umd.Id, umd.IdUserA, uml.IdUserB, umd.SentDate, umd.Message, uml.Viewed, uml.ViewedDate, umd.IsConf
		From A_UserMessageData umd
			Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Where ( IdUserA = @idusera And IdUserB = @iduserb )
			or ( IdUserA = @iduserb And IdUserB = @idusera )
		Order By SentDate Desc


	-- Первая выборка - сами сообщения
	Select tmp.Id, IdUserA, IsNull ( u1.Name, IsNull ( c1.Name, '???' )) as UserAName, IdUserB, IsNull ( u2.Name, IsNull ( c2.Name, '???' )) as UserBName, SentDate, Message, Viewed, ViewedDate, IsConf
	FROM @tmp tmp
		Left join A_Users u1 on IdUserA = u1.Id
		Left join A_Users u2 on IdUserB = u2.Id
		Left join A_Stat_Conference c1 on IdUserA = c1.Id
		Left join A_Stat_Conference c2 on IdUserB = c2.Id
	Where Inc <= @count
	Order by SentDate


	-- Вторая выборка - файлы к сообщениям
	Select umf.Id, umf.SentDate, msg.Id, umf.FilePath, umf.FileSize
	From @tmp msg
		Inner join A_UserMessageFile umf on umf.IdMessageData = msg.IdMessageData and umf.SentDate = msg.SentDate
	Order by umf.SentDate
end
GO


--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessages_GetNew' ))
	Drop procedure dbo.[A_UserMessages_GetNew]
GO	

Create Procedure [dbo].[A_UserMessages_GetNew]
	@iduser uniqueidentifier
as
begin
	Declare @allusers bit
	Set @allusers = 0
	if ( @iduser = '00000000-0000-0000-0000-000000000000' )
	   Set @allusers = 1	

	-- Не нужны сообщения из чатов конференций, поскольку они сразу считаются прочитанными

	Declare @t table (Id uniqueidentifier, IdMessageData int, SentDate datetime)
	Insert into @t (Id, IdMessageData, SentDate)
		Select uml.Id, umd.Id, SentDate
		From A_UserMessageData umd
			Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Where (( IdUserB = @iduser ) or ( @allusers = 1 ))
			and ( Viewed = 0 ) and ( IsConf = 0 )

	-- Первая выборка - сами сообщения
	Select uml.Id, IdUserA, IsNull ( u1.Name, '???' ) as UserAName, IdUserB, IsNull ( u2.Name, '???' ) as UserBName, umd.SentDate, Message, Viewed, ViewedDate, IsConf 
	From A_UserMessageData umd
		Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Inner join @t t on uml.Id = t.Id
		Left join A_Users u1 on IdUserA = u1.Id
		Left join A_Users u2 on IdUserB = u2.Id
	Order by umd.SentDate

	-- Вторая выборка - файлы к сообщениям
	Select umf.Id, umf.SentDate, msg.Id, umf.FilePath, umf.FileSize
	From @t msg
		Inner join A_UserMessageFile umf on umf.IdMessageData = msg.IdMessageData and umf.SentDate = msg.SentDate
	Order by umf.SentDate

end
GO


--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessages_GetNewForActive' ))
	Drop procedure dbo.[A_UserMessages_GetNewForActive]
GO	

Create Procedure [dbo].[A_UserMessages_GetNewForActive]
	@makeviewed bit = 1
as
begin	
	-- Запрос для обеспечения периодического опроса БД на предмет появления внешним образом новых сообщений

	-- Достаем активных подключенных
	Declare @active table (id uniqueidentifier)

	-- По A_UserStateHistory. Тут дефолтные пользователи тоже попадают
	/*
	Insert into @active (id)
		Select id--, name
		From ( Select u.id, --u.Name, 
				IsNull (( Select Top 1 h.state as state
					From dbo.A_UserStateHistory h 
					Where h.UserId = u.id
					Order By h.TimeChange Desc ), 0) as state
			From dbo.A_Users u
			Group by u.id--, u.Name
			) a
		Where state > 0
		--Order by name
	*/

	-- По A_CallCenter_UserStateHistory. Тут без дефолтных пользователей и запрос проще
	Insert into @active (id)
		Select OperatorId
		From A_CallCenter_UserStateHistory h
			Inner join A_Users u on u.id = h.operatorid
		Where TimeLeave is null
		Group by OperatorId


	-- Не нужны сообщения из чатов конференций, поскольку они сразу считаются прочитанными

	Declare @t table (Id uniqueidentifier, IdMessageData int, SentDate datetime)
	Insert into @t (Id, IdMessageData, SentDate)
		Select uml.Id, umd.Id, SentDate
		From A_UserMessageData umd
			Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Where (( Viewed = 0 ) and ( IsConf = 0 )
			and (IdUserB in ( Select id from @active )))


	-- Первая выборка - сами сообщения
	Select uml.Id, IdUserA, IsNull ( u1.Name, '???' ) as UserAName, IdUserB, IsNull ( u2.Name, '???' ) as UserBName, umd.SentDate, Message, Viewed, ViewedDate, IsConf
	From A_UserMessageData umd
			Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Inner join @t t on uml.Id = t.Id
		Left join A_Users u1 on IdUserA = u1.Id
		Left join A_Users u2 on IdUserB = u2.Id
	Order by umd.SentDate


	-- Вторая выборка - файлы к сообщениям
	Select umf.Id, umf.SentDate, msg.Id, umf.FilePath, umf.FileSize
	From @t msg
		Inner join A_UserMessageFile umf on umf.IdMessageData = msg.IdMessageData and umf.SentDate = msg.SentDate
	Order by umf.SentDate


	-- Считаем их просмотренными ?
	if ( @makeviewed = 1 ) begin

		Update ml
		Set ml.Viewed = 1,
			ml.ViewedDate = GetDate()
		From A_UserMessageLink ml, @active u
		Where ml.IdUserB = u.id and ml.Viewed = 0	
	end
end
GO


--------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.A_UserMessages_SetViewed' ))
	Drop procedure dbo.[A_UserMessages_SetViewed]
GO	

Create procedure [dbo].[A_UserMessages_SetViewed]
	@idmessage uniqueidentifier
as
begin
	declare @ida uniqueidentifier
	declare @idb uniqueidentifier
	declare @dt datetime

	Select @ida = IdUserA, @idb = IdUserB, @dt = SentDate
	From A_UserMessageData umd
		Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id	
	Where uml.Id = @idmessage

	if ( @@rowcount = 0 )
		return


	declare @rowcount int
	
	Update uml
	Set Viewed = 1, ViewedDate = GetDate ( )
	From A_UserMessageLink uml, A_UserMessageData umd
	Where umd.Id = uml.IdMessageData 
		and IdUserA = @ida 
		and IdUserB = @idb 
		and SentDate <= @dt 
		and Viewed = 0
	
	select @rowcount = @@rowcount
	
	if (( Select count(*) 
		  From A_UserMessageData umd
			Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		  Where IdUserA = @ida and IdUserB = @idb and Viewed = 0) = 0 )
	begin
		Update W_Messages_Dialogs 
		Set hasnew = 0 
		Where IdUserB = @ida 
			and IdUserA = @idb 
			and hasnew != 0
	end
	
	Select 120318, @rowcount as updatedcount
end
GO

---------------------------------------------------------------------
---------------------------------------------------------------------

if exists ( SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[W_UpdateDialogTrigger]'))
	DROP TRIGGER [dbo].[W_UpdateDialogTrigger]
GO

CREATE TRIGGER [dbo].[W_UpdateDialogTrigger] 
   ON [dbo].[A_UserMessageLink]
   AFTER INSERT
AS 
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;
	
	Declare @message nvarchar(4000), @count int
	
	-- NEW (by peter)
	Declare @tableinserted table 
	( 
		[Id] [uniqueidentifier] NOT NULL,
		[IdUserA] [uniqueidentifier] NULL,
		[IdUserB] [uniqueidentifier] NULL,
		[SentDate] [datetime] NOT NULL,
		[Message] [text] COLLATE database_default,
		[Viewed] [bit] NULL,
		[ViewedDate] [datetime] NULL,
		[IsConf] [int] NULL DEFAULT (0),
		[IdMessageData] int
    )

	Insert into @tableinserted ( id, idusera, iduserb, sentdate, message, viewed, vieweddate, isconf, idmessagedata )
		Select i.Id, umd.IdUserA, i.IdUserB, umd.SentDate, umd.Message, i.Viewed, i.ViewedDate, umd.IsConf, umd.Id
		From inserted i
			Inner join A_UserMessageData umd on umd.Id = i.IdMessageData

	-- Change from inserted to @tableinserted

	--------
	
	Select count ( * ) 
	From @tableinserted
	Where IsConf = 0

	if ( @@rowcount = 0 )
  	  return;

		
	Select @message = substring (message, 0, 255)
	From @tableinserted
	
	Update W_Messages_Dialogs 
	Set sentdate = i.sentdate, 
		lastmessage = @message,
		messagecount = md.messagecount+1,
		hasnew = 0,
		fromA = 1
	From @tableinserted i,
		W_Messages_Dialogs md
	Where md.iduserA = i.iduserA 
		and md.iduserB = i.iduserB
	
	
	
	if (( Select count(*) 
		  From @tableinserted i, A_UserMessageData umd, A_UserMessageLink uml
		  Where ( umd.Id = uml.IdMessageData ) 
				and umd.iduserA = i.iduserB 
				and uml.iduserB = i.iduserA 
				and uml.Viewed = 0) > 0)
	begin
		Update W_Messages_Dialogs 
		Set hasnew = 1 
		From @tableinserted i, W_Messages_Dialogs md 
		Where md.iduserA = i.iduserA 
			and md.iduserB = i.iduserB 
			and hasnew = 0
	end
	
	
	Update W_Messages_Dialogs 
	Set sentdate = i.sentdate, 
		lastmessage = @message,
		messagecount = md.messagecount + 1,
		hasnew = 1,
		fromA = 0
	From @tableinserted i, 
		W_Messages_Dialogs md
	Where md.iduserB = i.iduserA 
		and md.iduserA = i.iduserB
	
	if ( @@rowcount = 0 ) begin
	
		Select @count = count (*)
		From @tableinserted i, A_UserMessageData umd, A_UserMessageLink uml
		Where ( umd.Id = uml.IdMessageData ) 
			and ((umd.iduserA = i.iduserA and uml.iduserB = i.iduserB) 
				or (uml.iduserB = i.iduserA and umd.iduserA = i.iduserB))
		
		Declare @conftimestart datetime
		Declare @confname nvarchar(250)
		Select @conftimestart = TimeStart, @confname = Name 
		From A_Stat_Conference sc, @tableinserted i 
		Where sc.Id = i.iduserA 
			or sc.Id = i.iduserB
		
		Insert into W_Messages_Dialogs (iduserA, iduserB, sentdate, messagecount, lastmessage, hasnew, conftimestart, confname)--
		  Select iduserA,
			iduserB,
			sentdate,
			@count,
			@message,
			0,
			@conftimestart,
			@confname
		  From @tableinserted
		
		Insert into W_Messages_Dialogs (iduserB, iduserA, sentdate, messagecount, lastmessage, hasnew, conftimestart, confname)--
		  Select iduserA,
			iduserB,
			sentdate,
			@count,
			@message,
			1,
			@conftimestart,
			@confname
		  From @tableinserted
	end

END
GO


---------------------------------------------------------------------------
---------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.W_Messages_Search' ))
	Drop procedure dbo.[W_Messages_Search]
GO

Create Procedure [dbo].[W_Messages_Search]
	@idusera	uniqueidentifier,
	@iduserb	uniqueidentifier,
	@text	nvarchar(100),
	@startdate	datetime,
	@count int = null
as
begin
	Create table #temp  ( 
		Inc	int identity (1,1),
		Id	uniqueidentifier,
		IdUserA uniqueidentifier,
		IdUserB	uniqueidentifier,
		SentDate DateTime,
		Message text collate database_default,
		Viewed bit--,ViewedDate DateTime 
		)

		Insert into #temp
		Select top 100 uml.Id, IdUserA, IdUserB, SentDate, Message, Viewed 
		From A_UserMessageData umd
			Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Where (( IdUserA = @idusera And IdUserB = @iduserb ) or ( IdUserA = @iduserb And IdUserB = @idusera ))
			and ([Message] like @text + '%' or [Message] like '%[ "#@№(-:;.,«'']' + @text + '%')
		and SentDate < @startdate
		--Order By SentDate Desc
		
		select * 
		from #temp
		where Inc <= @count
		Order by SentDate desc
		drop table #temp
end
GO

---------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.W_Messages_GetLastNByDate' ))
	Drop procedure dbo.[W_Messages_GetLastNByDate]
GO

Create Procedure [dbo].[W_Messages_GetLastNByDate]
	@authorizeduserid	uniqueidentifier,
	@iduserb	uniqueidentifier,
	@startdate	datetime,
	@count		int
as
begin
	Create table #temp  ( 
		Inc	int identity (1,1),
		Id	uniqueidentifier,
		IdUserA uniqueidentifier,
		IdUserB	uniqueidentifier,
		SentDate DateTime,
		Message text collate database_default,
		Viewed bit--,ViewedDate DateTime 
		)

		Insert into #temp
		Select top 100 uml.Id, IdUserA, IdUserB, SentDate, Message, Viewed --Top 100
		From A_UserMessageData umd
			Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Where ( ( isConf = 0 and (( IdUserA = @authorizeduserid And IdUserB = @iduserb ) or ( IdUserA = @iduserb And IdUserB = @authorizeduserid )) ) or
			( IsConf = 1 and IdUserB = @iduserb ) )
			and SentDate < @startdate
		--Order By SentDate Desc
		
		select * 
		from #temp
		where Inc <= @count
		Order by SentDate desc
		

		Select umf.id, umf.sentdate, uml.Id as idmessage, umf.filepath, umf.filesize
		From [dbo].[A_UserMessageFile] umf
			Inner join A_UserMessageData umd on umd.Id = umf.IdMessageData
			Inner join A_UserMessageLink uml on uml.IdMessageData = umd.Id
		Where uml.Id in ( Select Id
							From #temp
							Where Inc <= @count )
		
		drop table #temp
end
GO

---------------------------------------------------------------------------

if exists ( select * From sysobjects where id=object_id ( 'dbo.W_Messages_GetNewCount' ))
	Drop procedure dbo.[W_Messages_GetNewCount]
GO


Create Procedure [dbo].[W_Messages_GetNewCount]
	@iduser uniqueidentifier

as
begin
	Select count (*) as [count]
	From A_UserMessageLink uml
	Where IdUserB = @iduser 
		and Viewed = 0
end
GO