﻿-----------------------------------
-----     Создание таблиц     -----
-----------------------------------

------------------------
-- A_Chat_Dialog --
------------------------

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'dbo.[A_Chat_Dialog]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE dbo.[A_Chat_Dialog]
GO

CREATE TABLE dbo.[A_Chat_Dialog](
	[IdDialog] [uniqueidentifier] NOT NULL,
	[Type] [int] NULL,
	[Name] [nvarchar](300) Collate database_default NULL,
	[LastIdMessage] [uniqueidentifier] NULL,
	[LastDateUTC] [datetime] NULL,
	[IdLinkedObject] [uniqueidentifier] NULL,
	[God] [uniqueidentifier] NOT NULL,
 CONSTRAINT [PK_A_Chat_Dialog] PRIMARY KEY CLUSTERED 
 (
	[IdDialog] ASC
 ) ON [PRIMARY]
) ON [PRIMARY]
GO

--Индекс по LastDateUTC

IF  EXISTS (SELECT * FROM dbo.sysindexes WHERE id = OBJECT_ID(N'dbo.[A_Chat_Dialog]') AND name = N'LASTDATE')
DROP INDEX dbo.[A_Chat_Dialog].[LASTDATE]
GO

CREATE NONCLUSTERED INDEX [LASTDATE] ON dbo.[A_Chat_Dialog] 
(
	[LastDateUTC] ASC
) ON [PRIMARY]
GO

-----------------------------
-- A_Chat_DialogTypes --
-----------------------------

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'dbo.[A_Chat_DialogTypes]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE dbo.[A_Chat_DialogTypes]
GO

CREATE TABLE dbo.[A_Chat_DialogTypes](
	[Id] [int] IDENTITY(1,1) NOT NULL,
	[Type] [nvarchar](50) Collate database_default NULL,
 CONSTRAINT [PK_A_Chat_DialogTypes] PRIMARY KEY CLUSTERED 
 (
	[Id] ASC
 ) ON [PRIMARY]
) ON [PRIMARY]
GO


Set identity_insert dbo.[A_Chat_DialogTypes] on
insert into dbo.[A_Chat_DialogTypes] ( Id, Type) Values ( 1, 'private' )
insert into dbo.[A_Chat_DialogTypes] ( Id, Type) Values ( 2, 'dialog' )
insert into dbo.[A_Chat_DialogTypes] ( Id, Type) Values ( 3, 'conference' )
Set identity_insert dbo.[A_Chat_DialogTypes] off
GO


-------------------------
-- A_Chat_Members --
-------------------------

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'dbo.[A_Chat_Members]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE dbo.[A_Chat_Members]
GO

CREATE TABLE dbo.[A_Chat_Members](
	[IdDialog] [uniqueidentifier] NOT NULL,
	[IdUser] [uniqueidentifier] NOT NULL,
 CONSTRAINT [PK_A_Chat_Members] PRIMARY KEY CLUSTERED 
 (
	[IdDialog] ASC,
	[IdUser] ASC
 ) ON [PRIMARY]
) ON [PRIMARY]

GO


-----------------------------
-- A_Chat_MessageFile --
-----------------------------

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'dbo.[A_Chat_MessageFile]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE dbo.[A_Chat_MessageFile]
GO

CREATE TABLE dbo.[A_Chat_MessageFile](
	[Id] [uniqueidentifier] NOT NULL,
	[IdMessage] [uniqueidentifier] NULL,
	[FilePath] [nvarchar](4000) Collate database_default NULL,
	[FileSize] [int] NULL,
) ON [PRIMARY]
GO

--Индекс по (IdMessage)
--Для поиска по привязке к сообщениям. 
--Собственные Id никому не упали

IF  EXISTS (SELECT * FROM dbo.sysindexes WHERE id = OBJECT_ID(N'dbo.[A_Chat_MessageFile]') AND name = N'idx_A_Chat_MessageFile_IdMessage')
DROP INDEX dbo.[A_Chat_MessageFile].[idx_A_Chat_MessageFile_IdMessage]
GO

CREATE NONCLUSTERED INDEX idx_A_Chat_MessageFile_IdMessage ON dbo.[A_Chat_MessageFile] 
(
	[IdMessage] ASC
) ON [PRIMARY]
GO

--------------------------
-- A_Chat_Messages --
--------------------------

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'dbo.[A_Chat_Messages]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE dbo.[A_Chat_Messages]
GO

CREATE TABLE dbo.[A_Chat_Messages](
	[IdMessage] [uniqueidentifier] NOT NULL,
	[IdDialog] [uniqueidentifier] NULL,
	[IdUser] [uniqueidentifier] NULL,
	[Text] [ntext] Collate database_default NULL,
	[DateUTC] [datetime] NULL,
) ON [PRIMARY]
GO

--Индекс по (IdDialog, DateUTC)
--Для поиска и возврата последних сообщений в диалоге
--по IdMessage вообще как будто искать их никто не будет

IF  EXISTS (SELECT * FROM dbo.sysindexes WHERE id = OBJECT_ID(N'dbo.[A_Chat_Messages]') AND name = N'idx_A_Chat_Messages_IdDialog_DateUTC')
DROP INDEX dbo.[A_Chat_Messages].[idx_A_Chat_Messages_IdDialog_DateUTC]
GO

CREATE NONCLUSTERED INDEX [idx_A_Chat_Messages_IdDialog_DateUTC] ON dbo.[A_Chat_Messages] 
(
	[IdDialog] ASC,
	[DateUTC] ASC
) ON [PRIMARY]
GO

-----------------------------
-- A_Chat_NewMessages --
-----------------------------

IF  EXISTS (SELECT * FROM dbo.sysobjects WHERE id = OBJECT_ID(N'dbo.[A_Chat_NewMessages]') AND OBJECTPROPERTY(id, N'IsUserTable') = 1)
DROP TABLE dbo.[A_Chat_NewMessages]
GO

CREATE TABLE dbo.[A_Chat_NewMessages](
	[IdMessage] [uniqueidentifier] NOT NULL,
	[IdDialog] [uniqueidentifier] NOT NULL,
	[IdUser] [uniqueidentifier] NOT NULL,
	[DateUTC] [datetime] NOT NULL,
 CONSTRAINT [PK_A_Chat_NewMessages] PRIMARY KEY CLUSTERED 
 (
	[IdMessage] ASC, [IdDialog] ASC, [IdUser] ASC
 ) ON [PRIMARY] 
) ON [PRIMARY]
GO

--Индекс по (IdDialog, IdUser, DateUTC)
--Для поиска и снятия признака просмотренности

IF  EXISTS (SELECT * FROM dbo.sysindexes WHERE id = OBJECT_ID(N'dbo.[A_Chat_NewMessages]') AND name = N'idx_A_Chat_NewMessages_IdMessage_IdDlg_Date')
DROP INDEX dbo.[idx_A_Chat_NewMessages_IdMessage].[idx_A_Chat_NewMessages_IdMessage_IdDlg_Date]
GO

CREATE NONCLUSTERED INDEX [idx_A_Chat_NewMessages_IdMessage_IdDlg_Date] ON dbo.[A_Chat_NewMessages] 
(
	[IdDialog],
	[IdUser],
	[DateUTC] ASC
) ON [PRIMARY]
GO


-------------------------------------
-----     Создание процедур     -----
-------------------------------------

-------------------------------
-- A_Chat_Dialog_Change --
-------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_Change]' ))
	drop procedure dbo.[A_Chat_Dialog_Change]
GO

create procedure dbo.[A_Chat_Dialog_Change]
	@dialogid	uniqueidentifier,
	@newdialogname	nvarchar(300)
as
begin
		-- Изменяет имя чату

		update	dbo.[A_Chat_Dialog]
		set	Name = @newdialogname
		where	IdDialog = @dialogid
end

GO


-------------------------------
-- A_Chat_Dialog_Create --
-------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_Create]' ))
	drop procedure dbo.[A_Chat_Dialog_Create]
GO

create procedure dbo.[A_Chat_Dialog_Create]
	@authorizeduserid	uniqueidentifier,
	@dialogid uniqueidentifier,
	@dialogtype int,
	@dialogname	nvarchar(300),
	@linkedobjectid	uniqueidentifier = NULL,
	@dateutc datetime = NULL
as
begin
	
	-- Размещает данные по новому чату (первая часть - сами свойства), потом отдельно еще вызывается сохранение участников

	if ( @dateutc is NULL )
		Set @dateutc = GetUTCDate()

	Insert into dbo.[A_Chat_Dialog] ( [IdDialog], [Type], [Name], [LastIdMessage], [LastDateUTC], [IdLinkedObject], [God] )
		Select	@dialogid, @dialogtype, @dialogname, 
			null, @dateutc, 
			@linkedobjectid,
			@authorizeduserid
end
GO


-----------------------------
-- A_Chat_Members_Add --
-----------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Members_Add]' ))
	drop procedure dbo.[A_Chat_Members_Add]
GO

create procedure dbo.[A_Chat_Members_Add]
	@dialogid	uniqueidentifier,
	@member	uniqueidentifier
as
begin
		-- Добавляет указанного участника в чат

		Insert into dbo.[A_Chat_Members] ( IdDialog, IdUser )
		Values ( @dialogid, @member )		

end
GO


-----------------------------
-- A_Chat_Members_Get --
-----------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Members_Get]' ))
	drop procedure dbo.[A_Chat_Members_Get]
GO

create procedure dbo.[A_Chat_Members_Get]
	@dialogid	uniqueidentifier
as
begin
		-- Возвращает список участников чата		

		select	members.IdUser
		from	dbo.[A_Chat_Members] members
		where	members.IdDialog = @dialogid
	
end
GO


--------------------------------
-- A_Chat_Members_Remove --
--------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Members_Remove]' ))
	drop procedure dbo.[A_Chat_Members_Remove]
GO

create procedure dbo.[A_Chat_Members_Remove]
	@dialogid	uniqueidentifier,
	@member	uniqueidentifier
as
begin
		-- Удаляет указанного участника из чата

		Delete
		From	dbo.[A_Chat_Members]
		Where	IdDialog = @dialogid
			and IdUser = @member

		-- Из непросмотренных сообщений
		Delete
		From	dbo.[A_Chat_NewMessages]
		Where	IdDialog = @dialogid
			and IdUser = @member
end
GO


-----------------------------
-- A_Chat_Message_Add --
-----------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Message_Add]' ))
	drop procedure dbo.[A_Chat_Message_Add]
GO

create procedure dbo.[A_Chat_Message_Add]
	@authorizeduserid uniqueidentifier,
	@dialogid uniqueidentifier,
	@messageid uniqueidentifier,
	@messagetext ntext,
	@nowutcdate datetime = NULL
as
begin
	
		-- Добавляет сообщение в чат (после другой процедурой добавляет файлы)
	
		If ( @nowutcdate is NULL )
			Set @nowutcdate = getutcdate()
				
		-- В таблицу сообщений
		Insert into dbo.[A_Chat_Messages]
			Select	@messageid,
					@dialogid,
					@authorizeduserid,
					@messagetext,
					@nowutcdate
				
		-- Обновляем свойства чата на предмет последнего сообщения
		Update	dbo.[A_Chat_Dialog]
		Set	LastIdMessage = @messageid,
			LastDateUTC = ( Select DateUTC 
							From dbo.[A_Chat_Messages] 
							Where IdMessage = @messageid )
		Where	IdDialog = @dialogid
		
		-- Вставляем в непрочитанные
		Insert into dbo.[A_Chat_NewMessages] (IdMessage, IdDialog, IdUser, DateUTC )
			Select	@messageid,
					@dialogid,
					members.IdUser,
					@nowutcdate
			From dbo.[A_Chat_Members] members
			Where IdDialog = @dialogid 
				and IdUser != @authorizeduserid		
end
GO

-----------------------------
-- A_Chat_Message_AddFile --
-----------------------------

if exists ( select * From sysobjects where id= object_id ( 'dbo.A_Chat_Message_AddFile' ))
	Drop procedure dbo.A_Chat_Message_AddFile
GO

Create procedure dbo.A_Chat_Message_AddFile
	@dialogid	uniqueidentifier,
	@messageid	uniqueidentifier,
	@fileid		uniqueidentifier,
	@filepath	nvarchar(4000),
	@filesize	int = NULL
as
begin

	-- Добавляет файл к сообщению	

	Insert into dbo.[A_Chat_MessageFile] ( Id, IdMessage, FilePath, FileSize )
	Values ( @fileid, @messageid, @filepath, @filesize )

end
GO


------------------------------------
-- A_Chat_Message_MarkViewed --
------------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Message_MarkViewed]' ))
	drop procedure dbo.[A_Chat_Message_MarkViewed]
GO

create procedure dbo.[A_Chat_Message_MarkViewed]
	@authorizeduserid uniqueidentifier,
	@dialogid uniqueidentifier,
	@messageid	uniqueidentifier
as
begin
		-- Отмечает сообщение как прочитанное

		-- Был ли просмотрен кем либо уже ранее?
		if
		(
			(
				select	count(*)
				from	dbo.[A_Chat_NewMessages]
				where	IdMessage = @messageid
			)
				>=
			(
				select	count(*) - 1
				from	dbo.[A_Chat_Members]
				where	IdDialog = @dialogid
			)
		)
		begin

			select	IdUser as AuthorMessage
			from	dbo.[A_Chat_Messages]
			where	IdMessage = @messageid		

		end else begin

			select	null as AuthorMessage

		end

		
		-- Удаление всех предыдущих нужно по диалогу
		Declare @dt datetime

		Select @dt = [DateUTC]
		From dbo.[A_Chat_NewMessages]
		Where	IdUser = @authorizeduserid
				and IdMessage = @messageid				
		
		---- Помечаем все более ранние по диалогу
		Delete
		from dbo.[A_Chat_NewMessages]
		where	IdUser = @authorizeduserid
				and IdDialog = @dialogid
				and DateUTC < @dt
end
GO


------------------------------------------
-- A_Chat_Message_MarkViewedByOldMapper --
------------------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Message_MarkViewedByOldMapper]' ))
	drop procedure dbo.[A_Chat_Message_MarkViewedByOldMapper]
GO

create procedure dbo.[A_Chat_Message_MarkViewedByOldMapper]
	@authorizeduserid uniqueidentifier,
	@messageid	uniqueidentifier
as
begin
		-- Отмечает сообщение как прочитанное (для поддержки старой схемы в толстом клиенте с маппером из предыдущего недиалогового режима)

		-- Удаление всех предыдущих нужно и по диалогу, и по юзеру
		Declare @dt datetime, @dialogid uniqueidentifier, @authorid uniqueidentifier, @markedcount int
		
		Select @dt = m.[DateUTC], @dialogid = m.[IdDialog], @authorid = m.[IdUser]
		From dbo.A_Chat_NewMessages nm
			Inner join dbo.A_Chat_Messages m on m.IdMessage = nm.IdMessage
		Where	nm.IdUser = @authorizeduserid
				and m.IdMessage = @messageid

		---- Помечаем все ранние по диалогу (Не так идет рассылка маппером)
		--Delete
		--from dbo.[A_Chat_NewMessages]
		--where	IdUser = @authorizeduserid
		--		and IdDialog = @dialogid
		--		and DateUTC <= @dt

		-- Помечаем по юзеру-отправителю (Именно так идет рассылка маппером)
		---- Из приватной беседы с отправителем
		--Declare @privatedialogid uniqueidentifier
		--
		--/* в приватных всегда два участника */
		--Select @privatedialogid = d1.IdDialog
		--From dbo.A_Chat_Dialog d1
		--	Inner join dbo.A_Chat_Members m1 on m1.IdDialog = d1.IdDialog and m1.IdUser = @authorizeduserid
		--	Inner join dbo.A_Chat_Members m2 on m2.IdDialog = d1.IdDialog and m2.IdUser = @authorid
		--Where d1.Type = 1
		--
		--Delete
		--From dbo.[A_Chat_NewMessages]
		--Where	IdUser = @authorizeduserid
		--		and IdDialog = @privatedialogid
		--		and DateUTC <= @dt

		-- Из всех диалогов, но только те сообщения, что отправитель писал
		--Select * 
		--From dbo.A_Chat_NewMessages nm
		--	Inner join dbo.A_Chat_Messages m on m.IdMessage = nm.IdMessage
		--Where nm.IdUser = @authorizeduserid 
		--	and m.DateUTC <= @dt
		--	and m.IdUser = @authorid
			
		Delete
		From dbo.A_Chat_NewMessages
		Where IdUser = @authorizeduserid 
			and DateUTC <= @dt
			and IdMessage in ( Select IdMessage 
								From dbo.A_Chat_Messages m
								Where m.IdUser = @authorid )
								
		Set @markedcount = @@rowcount

		-- Возвращаем отправителя и ид чата
		Select @authorid, @dialogid, @markedcount
end
GO

----------------------------
-- A_Chat_Dialog_Get --
----------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_Get]' ))
	drop procedure dbo.[A_Chat_Dialog_Get]
GO

Create procedure dbo.[A_Chat_Dialog_Get]
	@dialogid	uniqueidentifier
as
begin
		-- Возвращает серверную инфу по конкретному чату
	
		-- Выборка 1 (данные чата)
		Select	dlg.IdDialog,
				dlg.Type,
				dlg.Name,
				dlg.LastIdMessage,
				dlg.LastDateUTC,
				dlg.IdLinkedObject,
				dlg.God
		From dbo.[A_Chat_Dialog] dlg
		Where dlg.IdDialog = @dialogid

		-- Выборка 2 (участники)
		Select m.IdDialog, m.IdUser 
		From dbo.A_Chat_Members m 
		Where m.IdDialog = @dialogid
end
GO


--------------------------------
-- A_Chat_Dialog_GetTop --
--------------------------------

-- параметр - количество секунд, для поиска горячих диалогов по LastUTCDT

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_GetTop]' ))
	drop procedure dbo.[A_Chat_Dialog_GetTop]
GO

create procedure dbo.[A_Chat_Dialog_GetTop]
	@countsec int,
	@dateutc datetime = null
as
begin
		-- Возвращает список горячих чатов для сервера по дате последнего сообщения, находящейся в некотором интервале.
		
		-- Определяем интервал и чаты попадающие в него

		if (@dateutc is null) 
			set @dateutc = getutcdate()

		Declare @datefrom datetime
		Set @datefrom = DateAdd ( second, -@countsec, @dateutc )

		Declare @t table ( id uniqueidentifier )
		
		Insert into @t
			Select	dlg.IdDialog
			From dbo.[A_Chat_Dialog] dlg
			Where dlg.LastDateUTC >= @datefrom


		-- Выборка 1 (чаты)		
		
		Select	dlg.IdDialog,
				dlg.Type,
				dlg.Name,
				dlg.LastIdMessage,
				dlg.LastDateUTC,
				dlg.IdLinkedObject,
				dlg.God
		from dbo.[A_Chat_Dialog] dlg
				inner join @t t on t.id = dlg.IdDialog

		-- Выборка 2 (участники)

		Select m.IdDialog, m.IdUser 
		From dbo.A_Chat_Members m 
			inner join @t t on t.Id = m.IdDialog
		Order by m.IdDialog
		
end
GO

----------------------------------
--  A_Chat_Dialog_GetIdByUser  --
----------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_GetIdByUser]' ))
	drop procedure dbo.[A_Chat_Dialog_GetIdByUser]
GO

create procedure dbo.[A_Chat_Dialog_GetIdByUser]
	@authorizeduserid	uniqueidentifier,
	@userid	uniqueidentifier
as
begin
	-- Возвращает ид диалога по пользователю, с которым хочет общаться приватно

	select	IdDialog
	from	dbo.[A_Chat_Dialog]
	where	Type = 1 and
			IdDialog in
			(
				Select	IdDialog
				From	dbo.[A_Chat_Members]
				Where	IdUser in (@authorizeduserid, @userid)
				Group by IdDialog
				Having	count(*) = 2
			)	

--	Select d.IdDialog
--	From dbo.A_Chat_Dialog d
--		Inner join dbo.A_Chat_Members m1 on m1.IdDialog = d.IdDialog and m1.IdUser = @authorizeduserid
--		Inner join dbo.A_Chat_Members m2 on m2.IdDialog = d.IdDialog and m2.IdUser = @userid
--	Where d1.Type = 1

end
GO

----------------------------
-- A_Chat_Dialog_GetX --
----------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_GetX]' ))
	drop procedure dbo.[A_Chat_Dialog_GetX]
GO

create procedure dbo.[A_Chat_Dialog_GetX]
	@authorizeduserid	uniqueidentifier,
	@dialogid	uniqueidentifier
as
begin
		-- Возвращает информацию о чате для пользователя, снабжая ее инфой о количестве непрочитанных им сообщений
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#DialogCountMembers' ))
			Drop table #DialogCountMembers
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#Dialog' ))
			Drop table #Dialog

		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#PrivateDialogOpponent' ))
			Drop table #PrivateDialogOpponent
		

		-- Набор участников по диалогу
		Select	members.IdDialog as IdDialog,
			count(*) as CountMembers
		Into #DialogCountMembers
		From dbo.[A_Chat_Members] members
		Where members.IdDialog = @dialogid
		Group by members.IdDialog
		

		-- Набор данных по диалогу
		Select	dlg.IdDialog,
				dlg.Type,
				dlg.Name,
				dlg.LastIdMessage,
				dlg.LastDateUTC,
				dlg.IdLinkedObject,
				dlg.God
		Into #Dialog
		From dbo.[A_Chat_Dialog] dlg
		Where dlg.IdDialog = @dialogid
		

		-- Оппоненты приватных диалогов
		Select	members.IdDialog as IdDialog,
				members.IdUser as PrivateIdUser
		Into #PrivateDialogOpponent
		From dbo.[A_Chat_Members] members
		Where	members.IdDialog = @dialogid
				and members.IdDialog in ( select IdDialog from #Dialog where Type = 1 )
				and members.IdUser != @authorizeduserid
		

		-- Итоговая выборка
		Select	td.IdDialog,
				td.Type,
				td.Name,
				td.God,
				td.LastDateUTC,
				td.IdLinkedObject,
				m.IdMessage as LastMessageId,
				m.IdUser as LastUserId,
				u.Login as LastUserLogin,
				u.Name as LastUserName,
				m.DateUTC as LastMessageDTUTC,
				m.Text as LastMessageText,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
						case when (
									(
										select	count(*)
										from	dbo.[A_Chat_NewMessages]
										where	IdMessage in (select LastIdMessage from #Dialog) 
													and IdDialog = @dialogid
									)
										>=
									(
										select	count(*)
										from	dbo.[A_Chat_Members]
										where	IdDialog = @dialogid
												and IdUser != @authorizeduserid
									)
								)
								then 1 else 0 
						end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
						case when (	select count(*) 
									from dbo.[A_Chat_NewMessages] 
									where IdMessage in (select LastIdMessage from #Dialog) 
										and IdDialog = @dialogid 
										and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew,
				dcm.CountMembers,
				pdo.PrivateIdUser

		From #Dialog td 
				inner join #DialogCountMembers dcm on dcm.IdDialog = td.IdDialog
				left join dbo.[A_Chat_Messages] m on m.IdMessage = td.LastIdMessage
				left join #PrivateDialogOpponent pdo on pdo.IdDialog = td.IdDialog
				left join dbo.[A_Users] u on u.Id = m.IdUser
		Where td.IdDialog = @dialogid
			
	
		drop table #DialogCountMembers
		drop table #Dialog
		drop table #PrivateDialogOpponent	
end
GO


--------------------------------
-- A_Chat_Dialog_GetTopX --
--------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_GetTopX]' ))
	drop procedure dbo.[A_Chat_Dialog_GetTopX]
GO

create procedure dbo.[A_Chat_Dialog_GetTopX]
	@authorizeduserid uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = ''
as
begin
		-- Достает диалоги по дате последнего сообщения с конца указанную пачку (по индексу, количество)
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#TopDialog' ))
			Drop table #TopDialog

		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#DialogCountMembers' ))
			Drop table #DialogCountMembers
				
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#PrivateDialogOpponent' ))
			Drop table #PrivateDialogOpponent
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#FoundDialogs' ))
			Drop table #FoundDialogs

		Create table #TopDialog (IdDialog uniqueidentifier, Type int)
		Create table #DialogCountMembers (IdDialog uniqueidentifier, CountMembers int)
		Create table #PrivateDialogOpponent (IdDialog uniqueidentifier, PrivateIdUser uniqueidentifier)
		Create table #FoundDialogs (IdDialog uniqueidentifier)
		
		Declare @sql nvarchar(4000)
		
		if (( @search1 = '' ) and ( @search2 = '' ))
		begin

			-- Нарезаем нужный интервал из сортированного по дате
			Set @sql = 'Insert into #TopDialog ( IdDialog, Type )
				Select Top ' + Cast ( @count as nvarchar(10)) + ' IdDialog, Type
				From (
						Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdDialog, Type, LastDateUTC
						From dbo.[A_Chat_Dialog] dlg
						Where dlg.IdDialog in (	Select IdDialog 
												From dbo.[A_Chat_Members] 
												Where IdUser = @usr )
						Order by LastDateUTC desc ) a
				Order by LastDateUTC asc'

			Exec sp_executesql @sql, N'@usr uniqueidentifier', @usr = @authorizeduserid
			
		end else begin
			
			if (( @search1 = '' ) or ( @search2 = '' )) begin

				Declare @search nvarchar(4000)
				if ( @search1 != '' )
					Set @search = @search1
				else
					Set @search = @search2

				-- Ищем диалоги с участием пользователя с фильтром по названию
				Insert into #FoundDialogs
					Select idDialog
					From 
					(
						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid )
								and dlg.Type != 1
								and ( dlg.Name like '%'+@search1+'%' )
						
						Union all

						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg 
									inner join dbo.[A_Chat_Members] m on m.IdDialog = dlg.IdDialog
									inner join dbo.A_Users users on m.IdUser = users.ID
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid )
								and dlg.Type = 1
								and m.IdUser != @authorizeduserid
								and ( users.Name like '%'+@search+'%' )
					) a

			end else begin
			
				-- Ищем диалоги с участием пользователя с фильтром по названию
				Insert Into #FoundDialogs	
					Select IdDialog
					From 
					(
						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid )
								and dlg.Type != 1
								and (( dlg.Name like '%'+@search1+'%' ) or ( dlg.Name like '%'+@search2+'%' ))

						Union all

						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg 
									inner join dbo.[A_Chat_Members] m on m.IdDialog = dlg.IdDialog
									inner join dbo.A_Users users on users.ID = m.IdUser
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid)
								and dlg.Type = 1
								and m.IdUser != @authorizeduserid
								and (( users.Name like '%'+@search1+'%' ) or ( users.Name like '%'+@search2+'%' ))
					) a				
				
			end

			-- Нарезаем нужный интервал из сортированного по дате (уже по фильтру)
			Set @sql = 'Insert into #TopDialog ( IdDialog, Type )
				Select Top ' + Cast ( @count as nvarchar(10)) + ' IdDialog, Type
				From (
						Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdDialog, Type, LastDateUTC
						From dbo.[A_Chat_Dialog] dlg
							Inner join #FoundDialogs f on f.IdDialog = dlg.IdDialog
						Order by LastDateUTC desc ) a
				Order by LastDateUTC asc'

			Exec sp_executesql @sql

		end

		
		-- Сохраняем количество участников
		Insert Into #DialogCountMembers ( IdDialog, CountMembers )
			Select	m.IdDialog,
				count(m.IdUser) as CountMembers
			From dbo.[A_Chat_Members] as m
			Where m.IdDialog in ( Select IdDialog From #TopDialog )
			Group by m.IdDialog
		

		-- Сохраняем оппонента приватных бесед
		Insert Into #PrivateDialogOpponent ( IdDialog, PrivateIdUser )
			Select	m.IdDialog,
					m.IdUser
			From dbo.[A_Chat_Members] as m
			Where m.IdDialog in ( Select IdDialog From #TopDialog Where [Type] = 1 )
				and m.IdUser != @authorizeduserid


		-- Итоговая выборка
		Select	dlg.IdDialog,
				dlg.Type,
				dlg.Name,
				dlg.God,
				dlg.LastDateUTC,					
				dlg.IdLinkedObject,
				m.IdMessage as LastMessageId,
				m.IdUser as LastUserId,
				u.Login as LastUserLogin,
				u.Name as LastUserName,
				m.DateUTC as LastMessageDTUTC,
				m.Text as LastMessageText,
				Case When ( m.IdUser = @authorizeduserid ) then	--'from me'
							Case When ((
											select	count(*)
											from	dbo.[A_Chat_NewMessages]
											where	IdMessage in (select LastIdMessage from #TopDialog) 
													and IdDialog = dlg.IdDialog
										)
											>=
										(
											select	count(*)
											from	dbo.[A_Chat_Members]
											where	IdDialog = dlg.IdDialog
													and IdUser != @authorizeduserid
										))
									Then 1 Else 0 
						End
					Else null
					End as FromMeIsNew,
				Case When ( m.IdUser != @authorizeduserid ) then --'for me'
							Case When ( Select count(*) 
										From dbo.[A_Chat_NewMessages] 
										Where IdMessage in ( Select LastIdMessage From #TopDialog ) 
												and IdDialog = dlg.IdDialog 
												and IdUser = @authorizeduserid ) = 0 
									Then 0 Else 1 
						End
					Else null
					End ForMeIsNew,
				dcm.CountMembers,
				pdo.PrivateIdUser
		From #TopDialog td 
				Inner join dbo.[A_Chat_Dialog] dlg on dlg.IdDialog = td.IdDialog
				Inner join #DialogCountMembers dcm on dcm.IdDialog = td.IdDialog
				Left join dbo.[A_Chat_Messages] m on m.IdMessage = dlg.LastIdMessage
				Left join #PrivateDialogOpponent pdo on pdo.IdDialog = td.IdDialog
				Left join dbo.A_Users u on u.Id = m.IdUser
			

		
		drop table #DialogCountMembers
		drop table #TopDialog
		drop table #PrivateDialogOpponent
		drop table #FoundDialogs
		
end
GO


---------------------------------
-- A_Chat_Messages_GetTopX --
---------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Messages_GetTopX]' ))
	drop procedure dbo.[A_Chat_Messages_GetTopX]
GO

create procedure dbo.[A_Chat_Messages_GetTopX]
	@authorizeduserid	uniqueidentifier,
	@dialogid	uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = '',
	@dtstartutc datetime = null,
	@dtstoputc datetime = null
as
begin
		-- Возвращает список нескольких последних сообщений по диалогу для пользователя

		-- Нарезаем нужный интервал из сортированного по дате
		if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#interval' ))
			Drop table #interval

		Create table #interval ( IdMessage uniqueidentifier )
				
		-- Фильтр для даты
		Declare @filterdate nvarchar(4000)
		Set @filterdate = ''
		if ( @dtstartutc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC >= @dt1 )'
		if ( @dtstoputc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC <= @dt2 )'

		Declare @sql nvarchar(4000)

		if (( @search1 = '' ) and ( @search2 = '' )) begin

			Set @sql = 'Declare @dtstart datetime, @dtstop datetime
				Select @dtstart = @dt1, @dtstop = @dt2
			
				Insert into #interval ( IdMessage )				
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdMessage, DateUTC
							From dbo.[A_Chat_Messages]
							Where (IdDialog = @dlg)' + @filterdate + '
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		end else if (( @search1 = '' ) or ( @search2 = '' )) begin

			Declare @search nvarchar(4000)
			if ( @search1 != '' )
				Set @search = @search1
			else
				Set @search = @search2

			Set @sql = 'Declare @dtstart datetime, @dtstop datetime
				Select @dtstart = @dt1, @dtstop = @dt2

				Insert into #interval ( IdMessage )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdMessage, DateUTC
							From dbo.[A_Chat_Messages]
							Where (IdDialog = @dlg)' + @filterdate + '
								and (Text like ''%' + @search + '%'')
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc
			
		end else begin

			Set @sql = 'Declare @dtstart datetime, @dtstop datetime
				Select @dtstart = @dt1, @dtstop = @dt2

				Insert into #interval ( IdMessage )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdMessage, DateUTC
							From dbo.[A_Chat_Messages]
							Where (IdDialog = @dlg)' + @filterdate + '
								and (( Text like ''%' + @search1 + '%'' ) or ( Text like ''%' + @search2 + '%'' ))
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc
	
		end
	
		
		-- Собственно возвращаем сообщения
		Select	m.IdMessage,
				m.IdUser as AuthorId,
				u.Login as AuthorLogin,
				u.Name as AuthorName,
				m.DateUTC,
				m.Text,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
							case when (
										(
											select count(*)
											from dbo.[A_Chat_NewMessages]
											where IdMessage = i.IdMessage 
												and IdDialog = @dialogid
										)
											>=
										(
											select count(*)
											from dbo.[A_Chat_Members]
											where IdDialog = @dialogid
												and IdUser != @authorizeduserid
										)
									)
									then 1 else 0 
							end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
							case when ( select count(*) 
										from dbo.[A_Chat_NewMessages] 
										where IdMessage = i.IdMessage 
											and IdDialog = @dialogid 
											and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew
		From #interval as i
			Inner join dbo.[A_Chat_Messages] as m on m.IdMessage = i.IdMessage
			Inner join dbo.[A_Users] u on u.Id = m.IdUser
		
		-- Файлы привязанные
		Select	IdMessage,
				Id,
				FilePath,
				FileSize
		From dbo.[A_Chat_MessageFile]
		Where IdMessage in (select IdMessage from #interval)
		Order by IdMessage
		

		drop table #interval

end
GO

---------------------------------
-- A_Chat_Messages_GetTopXByOldMapper --
---------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Messages_GetTopXByOldMapper]' ))
	drop procedure dbo.[A_Chat_Messages_GetTopXByOldMapper]
GO

create procedure dbo.[A_Chat_Messages_GetTopXByOldMapper]
	@authorizeduserid	uniqueidentifier,
	@opponentid uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = '',
	@dtstartutc datetime = null,
	@dtstoputc datetime = null
as
begin
		-- Возвращает список нескольких последних сообщений по диалогу для пользователя

		-- Нарезаем нужный интервал из сортированного по дате
		if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#interval' ))
			Drop table #interval

		Create table #interval ( IdMessage uniqueidentifier, IdDialog uniqueidentifier )
				
		-- Фильтр для даты
		Declare @filterdate nvarchar(4000)
		Set @filterdate = ''
		if ( @dtstartutc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC >= @dt1 )'
		if ( @dtstoputc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC <= @dt2 )'

		Declare @sql nvarchar(4000)

		if (( @search1 = '' ) and ( @search2 = '' )) begin

			Set @sql = 'Insert into #interval ( IdMessage, IdDialog )				
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage, IdDialog
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' msg.IdMessage, msg.IdDialog, msg.DateUTC
							From dbo.[A_Chat_Messages] msg
								Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) and (mem.IdUser = @me)
							Where (msg.IdUser = @author)' + @filterdate + '
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		end else if (( @search1 = '' ) or ( @search2 = '' )) begin

			Declare @search nvarchar(4000)
			if ( @search1 != '' )
				Set @search = @search1
			else
				Set @search = @search2

			Set @sql = 'Insert into #interval ( IdMessage, IdDialog )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage, IdDialog
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' msg.IdMessage, msg.IdDialog, msg.DateUTC
							From dbo.[A_Chat_Messages] msg
								Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) and (mem.IdUser = @me)
							Where (msg.IdUser = @author)' + @filterdate + '
								and (Text like ''%' + @search + '%'')
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc
			
		end else begin

			Set @sql = 'Insert into #interval ( IdMessage, IdDialog )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage, IdDialog
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' msg.IdMessage, msg.IdDialog msg.DateUTC
							From dbo.[A_Chat_Messages] msg
								Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) and (mem.IdUser = @me)
							Where (msg.IdUser = @author)' + @filterdate + '
								and (( Text like ''%' + @search1 + '%'' ) or ( Text like ''%' + @search2 + '%'' ))
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc
	
		end
	
		
		-- Собственно возвращаем сообщения
		Select	m.IdMessage,
				m.IdUser as AuthorId,
				u.Login as AuthorLogin,
				u.Name as AuthorName,
				m.DateUTC,
				m.Text,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
							case when (
										(
											select count(*)
											from dbo.[A_Chat_NewMessages] nm
											where nm.IdMessage = i.IdMessage 
												and nm.IdDialog = i.IdDialog
										)
											>=
										(
											select count(*)
											from dbo.[A_Chat_Members] mem
											where mem.IdDialog = i.IdDialog
												and mem.IdUser != @authorizeduserid
										)
									)
									then 1 else 0 
							end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
							case when ( select count(*) 
										from dbo.[A_Chat_NewMessages] 
										where IdMessage = i.IdMessage 
											and IdDialog = i.IdDialog
											and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew
		From #interval as i
			Inner join dbo.[A_Chat_Messages] as m on m.IdMessage = i.IdMessage
			Inner join dbo.[A_Users] u on u.Id = m.IdUser
		
		-- Файлы привязанные
		Select	IdMessage,
				Id,
				FilePath,
				FileSize
		From dbo.[A_Chat_MessageFile]
		Where IdMessage in (select IdMessage from #interval)
		Order by IdMessage
		

		drop table #interval

end
GO

--------------------------------------
-- A_Chat_Messages_GetTotalNewCountX --
--------------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Messages_GetTotalNewCountX]' ))
	drop procedure dbo.[A_Chat_Messages_GetTotalNewCountX]
GO

Create procedure dbo.[A_Chat_Messages_GetTotalNewCountX]
	@authorizeduserid	uniqueidentifier
as
begin
	-- Возвращает количествно непрочитанных пользователем сообщений всего
	
	Select Count(*)
	From dbo.A_Chat_NewMessages
	Where IdUser = @authorizeduserid

end
GO


--------------------------------------
-- A_Chat_Messages_GetAllNewX --
--------------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Messages_GetAllNewX]' ))
	drop procedure dbo.[A_Chat_Messages_GetAllNewX]
GO

Create procedure dbo.[A_Chat_Messages_GetAllNewX]
	@authorizeduserid	uniqueidentifier
as
begin
	-- Возвращает количествно непрочитанных пользователем сообщений всего
	
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#msg' ))
		Drop table #msg

	Create table #msg ( IdMessage uniqueidentifier )

	-- Фильтр
	Insert into #msg ( IdMessage )
		Select Top 1000 
			m.IdMessage
		From dbo.A_Chat_NewMessages nm
			Inner join dbo.A_Chat_Messages m on m.IdMessage = nm.IdMessage
		Where nm.IdUser = @authorizeduserid
		Order by m.DateUTC Desc


	-- Сообщения
	Select Top 1000 
		m.IdDialog,  
		m.IdMessage, 
		m.IdUser as AuthorId, 
		u.Login as AuthorLogin, 
		u.Name as AuthorName, 
		m.DateUTC,
		m.Text
	From #msg filter
		Inner join dbo.A_Chat_Messages m on m.IdMessage = filter.IdMessage
		Left join dbo.A_Users u on u.Id = m.IdUser	
	Order by m.DateUTC Desc


	-- Файлы привязанные
	Select	IdMessage,
			Id,
			FilePath,
			FileSize
	From dbo.[A_Chat_MessageFile]
	Where IdMessage in (select IdMessage from #msg)
	Order by IdMessage

end
GO

