﻿------------------------------------
-- A_Chat_Message_MarkViewed --
------------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Message_MarkViewed]' ))
	drop procedure dbo.[A_Chat_Message_MarkViewed]
GO

create procedure dbo.[A_Chat_Message_MarkViewed]
	@authorizeduserid uniqueidentifier,
	@dialogid uniqueidentifier,
	@messageid	uniqueidentifier
as
begin
		-- Отмечает сообщение как прочитанное

		Declare @authorid uniqueidentifier, @markedcount int

		-- Был ли просмотрен кем либо уже ранее?
		if
		(
			(
				select	count(*)
				from	dbo.[A_Chat_NewMessages]
				where	IdMessage = @messageid
			)
				>=
			(
				select	count(*) - 1
				from	dbo.[A_Chat_Members]
				where	IdDialog = @dialogid
			)
		)
		begin

			select	@authorid = IdUser
			from	dbo.[A_Chat_Messages]
			where	IdMessage = @messageid		

		end

		
		-- Удаление всех предыдущих нужно по диалогу
		Declare @dt datetime

		Select @dt = [DateUTC]
		From dbo.[A_Chat_NewMessages]
		Where	IdUser = @authorizeduserid
				and IdMessage = @messageid				
		
		---- Помечаем все более ранние по диалогу
		Delete
		from dbo.[A_Chat_NewMessages]
		where	IdUser = @authorizeduserid
				and IdDialog = @dialogid
				and DateUTC <= @dt
				
		-- Сколько прочитано итого
		Set @markedcount = @@rowcount
		
		Select @authorid, @markedcount
end
GO

----------------------------
-- A_Chat_Dialog_GetX --
----------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_GetX]' ))
	drop procedure dbo.[A_Chat_Dialog_GetX]
GO

create procedure dbo.[A_Chat_Dialog_GetX]
	@authorizeduserid	uniqueidentifier,
	@dialogid	uniqueidentifier
as
begin
		-- Возвращает информацию о чате для пользователя, снабжая ее инфой о количестве непрочитанных им сообщений
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#DialogCountMembers' ))
			Drop table #DialogCountMembers
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#Dialog' ))
			Drop table #Dialog

		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#PrivateDialogOpponent' ))
			Drop table #PrivateDialogOpponent
		

		-- Набор участников по диалогу
		Select	members.IdDialog as IdDialog,
			count(*) as CountMembers
		Into #DialogCountMembers
		From dbo.[A_Chat_Members] members
		Where members.IdDialog = @dialogid
		Group by members.IdDialog
		

		-- Набор данных по диалогу
		Select	dlg.IdDialog,
				dlg.Type,
				dlg.Name,
				dlg.LastIdMessage,
				dlg.LastDateUTC,
				dlg.IdLinkedObject,
				dlg.God
		Into #Dialog
		From dbo.[A_Chat_Dialog] dlg
		Where dlg.IdDialog = @dialogid
		

		-- Оппоненты приватных диалогов
		Select	members.IdDialog as IdDialog,
				members.IdUser as PrivateIdUser
		Into #PrivateDialogOpponent
		From dbo.[A_Chat_Members] members
		Where	members.IdDialog = @dialogid
				and members.IdDialog in ( select IdDialog from #Dialog where Type = 1 )
				and members.IdUser != @authorizeduserid
		

		-- Итоговая выборка
		Select	td.IdDialog,
				td.Type,
				td.Name,
				td.God,
				td.LastDateUTC,
				td.IdLinkedObject,
				m.IdMessage as LastMessageId,
				m.IdUser as LastUserId,
				u.Login as LastUserLogin,
				u.Name as LastUserName,
				m.DateUTC as LastMessageDTUTC,
				m.Text as LastMessageText,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
						case when (
									(
										select	count(*)
										from	dbo.[A_Chat_NewMessages]
										where	IdMessage in (select LastIdMessage from #Dialog) 
													and IdDialog = @dialogid
									)
										>=
									(
										select	count(*)
										from	dbo.[A_Chat_Members]
										where	IdDialog = @dialogid
												and IdUser != @authorizeduserid
									)
								)
								then 1 else 0 
						end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
						case when (	select count(*) 
									from dbo.[A_Chat_NewMessages] 
									where IdMessage in (select LastIdMessage from #Dialog) 
										and IdDialog = @dialogid 
										and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew,
				dcm.CountMembers,
				pdo.PrivateIdUser

		From #Dialog td 
				inner join #DialogCountMembers dcm on dcm.IdDialog = td.IdDialog
				left join dbo.[A_Chat_Messages] m on m.IdMessage = td.LastIdMessage
				left join #PrivateDialogOpponent pdo on pdo.IdDialog = td.IdDialog
				left join dbo.[A_Users] u on u.Id = m.IdUser
		Where td.IdDialog = @dialogid
			
	
		drop table #DialogCountMembers
		drop table #Dialog
		drop table #PrivateDialogOpponent	
end
GO


--------------------------------
-- A_Chat_Dialog_GetTopX --
--------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Dialog_GetTopX]' ))
	drop procedure dbo.[A_Chat_Dialog_GetTopX]
GO

create procedure dbo.[A_Chat_Dialog_GetTopX]
	@authorizeduserid uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = ''
as
begin
		-- Достает диалоги по дате последнего сообщения с конца указанную пачку (по индексу, количество)
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#TopDialog' ))
			Drop table #TopDialog

		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#DialogCountMembers' ))
			Drop table #DialogCountMembers
				
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#PrivateDialogOpponent' ))
			Drop table #PrivateDialogOpponent
		
		if exists ( Select * FROM tempdb..sysobjects where id= object_id ( 'tempdb..#FoundDialogs' ))
			Drop table #FoundDialogs

		Create table #TopDialog (IdDialog uniqueidentifier, Type int)
		Create table #DialogCountMembers (IdDialog uniqueidentifier, CountMembers int)
		Create table #PrivateDialogOpponent (IdDialog uniqueidentifier, PrivateIdUser uniqueidentifier)
		Create table #FoundDialogs (IdDialog uniqueidentifier)
		
		Declare @sql nvarchar(4000)
		
		if (( @search1 = '' ) and ( @search2 = '' ))
		begin

			-- Нарезаем нужный интервал из сортированного по дате
			Set @sql = 'Insert into #TopDialog ( IdDialog, Type )
				Select Top ' + Cast ( @count as nvarchar(10)) + ' IdDialog, Type
				From (
						Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' dlg.IdDialog, dlg.Type, dlg.LastDateUTC
						From dbo.[A_Chat_Dialog] dlg
						Where dlg.IdDialog in (	Select IdDialog 
												From dbo.[A_Chat_Members] 
												Where IdUser = @usr )
						Order by LastDateUTC desc ) a
				Order by LastDateUTC asc'

			Exec sp_executesql @sql, N'@usr uniqueidentifier', @usr = @authorizeduserid
			
		end else begin
			
			if (( @search1 = '' ) or ( @search2 = '' )) begin

				Declare @search nvarchar(4000)
				if ( @search1 != '' )
					Set @search = @search1
				else
					Set @search = @search2

				-- Ищем диалоги с участием пользователя с фильтром по названию
				Insert into #FoundDialogs
					Select idDialog
					From 
					(
						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid )
								and dlg.Type != 1
								and ( dlg.Name like '%'+@search1+'%' )
						
						Union all

						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg 
									inner join dbo.[A_Chat_Members] m on m.IdDialog = dlg.IdDialog
									inner join dbo.A_Users users on m.IdUser = users.ID
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid )
								and dlg.Type = 1
								and m.IdUser != @authorizeduserid
								and ( users.Name like '%'+@search+'%' )
					) a

			end else begin
			
				-- Ищем диалоги с участием пользователя с фильтром по названию
				Insert Into #FoundDialogs	
					Select IdDialog
					From 
					(
						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid )
								and dlg.Type != 1
								and (( dlg.Name like '%'+@search1+'%' ) or ( dlg.Name like '%'+@search2+'%' ))

						Union all

						Select	dlg.IdDialog
						From	dbo.[A_Chat_Dialog] dlg 
									inner join dbo.[A_Chat_Members] m on m.IdDialog = dlg.IdDialog
									inner join dbo.A_Users users on users.ID = m.IdUser
						Where	dlg.IdDialog in ( Select IdDialog 
												  From dbo.[A_Chat_Members] 
												  Where IdUser = @authorizeduserid)
								and dlg.Type = 1
								and m.IdUser != @authorizeduserid
								and (( users.Name like '%'+@search1+'%' ) or ( users.Name like '%'+@search2+'%' ))
					) a				
				
			end

			-- Нарезаем нужный интервал из сортированного по дате (уже по фильтру)
			Set @sql = 'Insert into #TopDialog ( IdDialog, Type )
				Select Top ' + Cast ( @count as nvarchar(10)) + ' IdDialog, Type
				From (
						Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' dlg.IdDialog, dlg.Type, dlg.LastDateUTC
						From dbo.[A_Chat_Dialog] dlg
							Inner join #FoundDialogs f on f.IdDialog = dlg.IdDialog
						Order by LastDateUTC desc ) a
				Order by LastDateUTC asc'

			Exec sp_executesql @sql

		end

		
		-- Сохраняем количество участников
		Insert Into #DialogCountMembers ( IdDialog, CountMembers )
			Select	m.IdDialog,
				count(m.IdUser) as CountMembers
			From dbo.[A_Chat_Members] as m
			Where m.IdDialog in ( Select IdDialog From #TopDialog )
			Group by m.IdDialog
		

		-- Сохраняем оппонента приватных бесед
		Insert Into #PrivateDialogOpponent ( IdDialog, PrivateIdUser )
			Select	m.IdDialog,
					m.IdUser
			From dbo.[A_Chat_Members] as m
			Where m.IdDialog in ( Select IdDialog From #TopDialog Where [Type] = 1 )
				and m.IdUser != @authorizeduserid


		-- Итоговая выборка
		Select	dlg.IdDialog,
				dlg.Type,
				dlg.Name,
				dlg.God,
				dlg.LastDateUTC,					
				dlg.IdLinkedObject,
				m.IdMessage as LastMessageId,
				m.IdUser as LastUserId,
				u.Login as LastUserLogin,
				u.Name as LastUserName,
				m.DateUTC as LastMessageDTUTC,
				m.Text as LastMessageText,
				Case When ( m.IdUser = @authorizeduserid ) then	--'from me'
							Case When ((
											select	count(*)
											from	dbo.[A_Chat_NewMessages]
											where	IdMessage in (select LastIdMessage from #TopDialog) 
													and IdDialog = dlg.IdDialog
										)
											>=
										(
											select	count(*)
											from	dbo.[A_Chat_Members]
											where	IdDialog = dlg.IdDialog
													and IdUser != @authorizeduserid
										))
									Then 1 Else 0 
						End
					Else null
					End as FromMeIsNew,
				Case When ( m.IdUser != @authorizeduserid ) then --'for me'
							Case When ( Select count(*) 
										From dbo.[A_Chat_NewMessages] 
										Where IdMessage in ( Select LastIdMessage From #TopDialog ) 
												and IdDialog = dlg.IdDialog 
												and IdUser = @authorizeduserid ) = 0 
									Then 0 Else 1 
						End
					Else null
					End ForMeIsNew,
				dcm.CountMembers,
				pdo.PrivateIdUser
		From #TopDialog td 
				Inner join dbo.[A_Chat_Dialog] dlg on dlg.IdDialog = td.IdDialog
				Inner join #DialogCountMembers dcm on dcm.IdDialog = td.IdDialog
				Left join dbo.[A_Chat_Messages] m on m.IdMessage = dlg.LastIdMessage
				Left join #PrivateDialogOpponent pdo on pdo.IdDialog = td.IdDialog
				Left join dbo.A_Users u on u.Id = m.IdUser
		Order by dlg.LastDateUTC Desc

		
		drop table #DialogCountMembers
		drop table #TopDialog
		drop table #PrivateDialogOpponent
		drop table #FoundDialogs
		
end
GO



---------------------------------
-- A_Chat_Messages_GetTopX --
---------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Messages_GetTopX]' ))
	drop procedure dbo.[A_Chat_Messages_GetTopX]
GO

create procedure dbo.[A_Chat_Messages_GetTopX]
	@authorizeduserid	uniqueidentifier,
	@dialogid	uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = '',
	@dtstartutc datetime = null,
	@dtstoputc datetime = null
as
begin
		-- Возвращает список нескольких последних сообщений по диалогу для пользователя

		-- Является ли участником чата
		Declare @ismember int
		
		Select @ismember = Count(*)
		From dbo.A_Chat_Members
		Where IdDialog = @dialogid
			And IdUser = @authorizeduserid

		if ( @ismember = 0 )
			return

		-- Нарезаем нужный интервал из сортированного по дате
		if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#interval' ))
			Drop table #interval

		Create table #interval ( IdMessage uniqueidentifier )
				
		-- Фильтр для даты
		Declare @filterdate nvarchar(4000)
		Set @filterdate = ''
		if ( @dtstartutc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC >= @dt1 )'
		if ( @dtstoputc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC <= @dt2 )'

		Declare @sql nvarchar(4000)

		if (( @search1 = '' ) and ( @search2 = '' )) begin

			Set @sql = 'Declare @dtstart datetime, @dtstop datetime
				Select @dtstart = @dt1, @dtstop = @dt2
			
				Insert into #interval ( IdMessage )				
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdMessage, DateUTC
							From dbo.[A_Chat_Messages]
							Where (IdDialog = @dlg)' + @filterdate + '
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		end else if (( @search1 = '' ) or ( @search2 = '' )) begin

			Declare @search nvarchar(4000)
			if ( @search1 != '' )
				Set @search = @search1
			else
				Set @search = @search2

			Set @sql = 'Declare @dtstart datetime, @dtstop datetime
				Select @dtstart = @dt1, @dtstop = @dt2

				Insert into #interval ( IdMessage )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdMessage, DateUTC
							From dbo.[A_Chat_Messages]
							Where (IdDialog = @dlg)' + @filterdate + '
								and (Text like ''%' + @search + '%'')
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc
			
		end else begin

			Set @sql = 'Declare @dtstart datetime, @dtstop datetime
				Select @dtstart = @dt1, @dtstop = @dt2

				Insert into #interval ( IdMessage )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' IdMessage, DateUTC
							From dbo.[A_Chat_Messages]
							Where (IdDialog = @dlg)' + @filterdate + '
								and (( Text like ''%' + @search1 + '%'' ) or ( Text like ''%' + @search2 + '%'' ))
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@dlg uniqueidentifier, @dt1 datetime, @dt2 datetime', @dlg = @dialogid, @dt1 = @dtstartutc, @dt2 = @dtstoputc
	
		end
	
		
		-- Собственно возвращаем сообщения
		Select	m.IdMessage,
				m.IdUser as AuthorId,
				u.Login as AuthorLogin,
				u.Name as AuthorName,
				m.DateUTC,
				m.Text,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
							case when (
										(
											select count(*)
											from dbo.[A_Chat_NewMessages]
											where IdMessage = i.IdMessage 
												and IdDialog = @dialogid
										)
											>=
										(
											select count(*)
											from dbo.[A_Chat_Members]
											where IdDialog = @dialogid
												and IdUser != @authorizeduserid
										)
									)
									then 1 else 0 
							end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
							case when ( select count(*) 
										from dbo.[A_Chat_NewMessages] 
										where IdMessage = i.IdMessage 
											and IdDialog = @dialogid 
											and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew
		From #interval as i
			Inner join dbo.[A_Chat_Messages] as m on m.IdMessage = i.IdMessage
			Inner join dbo.[A_Users] u on u.Id = m.IdUser
		Order by m.DateUTC Desc
		
		-- Файлы привязанные
		Select	IdMessage,
				Id,
				FilePath,
				FileSize
		From dbo.[A_Chat_MessageFile]
		Where IdMessage in (select IdMessage from #interval)
		Order by IdMessage
		

		drop table #interval

end
GO



----------------------------------------
-- A_Chat_Messages_GetTopXByOldMapper --
----------------------------------------

if exists ( select * from sysobjects where id = object_id ( 'dbo.[A_Chat_Messages_GetTopXByOldMapper]' ))
	drop procedure dbo.[A_Chat_Messages_GetTopXByOldMapper]
GO

create procedure dbo.[A_Chat_Messages_GetTopXByOldMapper]
	@authorizeduserid	uniqueidentifier,
	@opponentid uniqueidentifier,
	@start int,
	@count int,
	@search1 nvarchar(1000) = '',
	@search2 nvarchar(1000) = '',
	@dtstartutc datetime = null,
	@dtstoputc datetime = null
as
begin
		-- Возвращает список нескольких последних сообщений по диалогу для пользователя
	

		-- Нарезаем нужный интервал из сортированного по дате
		if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#interval' ))
			Drop table #interval

		Create table #interval ( IdMessage uniqueidentifier, IdDialog uniqueidentifier )
				
		-- Фильтр для даты
		Declare @filterdate nvarchar(4000)
		Set @filterdate = ''
		if ( @dtstartutc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC >= @dt1 )'
		if ( @dtstoputc is not null )
			Set @filterdate = @filterdate + ' and ( DateUTC <= @dt2 )'

		Declare @sql nvarchar(4000)

		if (( @search1 = '' ) and ( @search2 = '' )) begin

			Set @sql = 'Insert into #interval ( IdMessage, IdDialog )				
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage, IdDialog
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' msg.IdMessage, msg.IdDialog, msg.DateUTC
							From dbo.[A_Chat_Messages] msg
								Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) and (mem.IdUser = @me)
							Where (msg.IdUser = @author)' + @filterdate + '
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc

		end else if (( @search1 = '' ) or ( @search2 = '' )) begin

			Declare @search nvarchar(4000)
			if ( @search1 != '' )
				Set @search = @search1
			else
				Set @search = @search2

			Set @sql = 'Insert into #interval ( IdMessage, IdDialog )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage, IdDialog
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' msg.IdMessage, msg.IdDialog, msg.DateUTC
							From dbo.[A_Chat_Messages] msg
								Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) and (mem.IdUser = @me)
							Where (msg.IdUser = @author)' + @filterdate + '
								and (Text like ''%' + @search + '%'')
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc
			
		end else begin

			Set @sql = 'Insert into #interval ( IdMessage, IdDialog )
					Select Top ' + Cast ( @count as nvarchar(10)) + ' IdMessage, IdDialog
					From ( 
							Select Top ' + Cast ( @start + @count as nvarchar(10) ) + ' msg.IdMessage, msg.IdDialog msg.DateUTC
							From dbo.[A_Chat_Messages] msg
								Inner join dbo.A_Chat_Members mem on (mem.IdDialog = msg.IdDialog) and (mem.IdUser = @me)
							Where (msg.IdUser = @author)' + @filterdate + '
								and (( Text like ''%' + @search1 + '%'' ) or ( Text like ''%' + @search2 + '%'' ))
							Order by DateUTC desc ) a
					Order by DateUTC asc'

			exec sp_executesql @sql, N'@author uniqueidentifier, @me uniqueidentifier, @dt1 datetime, @dt2 datetime', 
						@author = @opponentid, @me = @authorizeduserid, @dt1 = @dtstartutc, @dt2 = @dtstoputc
	
		end
	
		
		-- Собственно возвращаем сообщения
		Select	m.IdMessage,
				m.IdUser as AuthorId,
				u.Login as AuthorLogin,
				u.Name as AuthorName,
				m.DateUTC,
				m.Text,
				case when ( m.IdUser = @authorizeduserid ) then	--'from me'
							case when (
										(
											select count(*)
											from dbo.[A_Chat_NewMessages] nm
											where nm.IdMessage = i.IdMessage 
												and nm.IdDialog = i.IdDialog
										)
											>=
										(
											select count(*)
											from dbo.[A_Chat_Members] mem
											where mem.IdDialog = i.IdDialog
												and mem.IdUser != @authorizeduserid
										)
									)
									then 1 else 0 
							end
					else null
					end FromMeIsNew,
				case when ( m.IdUser != @authorizeduserid ) then	--'for me'
							case when ( select count(*) 
										from dbo.[A_Chat_NewMessages] 
										where IdMessage = i.IdMessage 
											and IdDialog = i.IdDialog
											and IdUser = @authorizeduserid ) = 0 
									then 0 else 1 
						end
					else null
					end ForMeIsNew
		From #interval as i
			Inner join dbo.[A_Chat_Messages] as m on m.IdMessage = i.IdMessage
			Inner join dbo.[A_Users] u on u.Id = m.IdUser
		Order by m.DateUTC Asc
		
		-- Файлы привязанные
		Select	IdMessage,
				Id,
				FilePath,
				FileSize
		From dbo.[A_Chat_MessageFile]
		Where IdMessage in (select IdMessage from #interval)
		Order by IdMessage
		

		drop table #interval

end
GO