﻿-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_SessionStorage' ))
	Drop table dbo.A_SessionStorage
GO

Create table dbo.A_SessionStorage
(
	storageid int not null,
	storeid nvarchar(100) collate database_default not null,
	storeinterval int not null,
	storedt datetime not null,
	userid uniqueidentifier,
	remoteip nvarchar(50)
  CONSTRAINT [PK_A_SessionStorage] PRIMARY KEY CLUSTERED 
  (
	storageid,
	storeid
  ) ON [PRIMARY]
)
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_SessionStorage_GetAll' ))
	Drop procedure dbo.A_SessionStorage_GetAll
GO

Create procedure dbo.A_SessionStorage_GetAll
	@dbid int,
	@dtnow datetime = NULL
as
begin
	
	if ( @dtnow is NULL )
		Set @dtnow = GetDate()
	
	Select storeid, storeinterval, storedt, userid, remoteip
	From A_SessionStorage
	Where (storageid = @dbid)
		And (DateAdd ( ms, storeinterval, storedt ) > @dtnow)
		
end
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_SessionStorage_Save' ))
	Drop procedure dbo.A_SessionStorage_Save
GO

Create procedure dbo.A_SessionStorage_Save
	@dbid int,
	@storeid nvarchar(100),
	@storeinterval int,
	@storedt datetime,
	@userid uniqueidentifier,
	@remoteip nvarchar(50) = NULL
as
begin

	Update A_SessionStorage
	Set storeinterval = @storeinterval,
		storedt = @storedt,
		userid = @userid,
		remoteip = @remoteip
	Where (storageid = @dbid)
		And (storeid = @storeid)
	
	if ( @@rowcount = 0 ) begin
	
		Insert into A_SessionStorage ( storageid, storeid, storeinterval, storedt, userid, remoteip )
		Values ( @dbid, @storeid, @storeinterval, @storedt, @userid, @remoteip )
		
	end
	
end
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_SessionStorage_Remove' ))
	Drop procedure dbo.A_SessionStorage_Remove
GO

Create procedure dbo.A_SessionStorage_Remove
	@dbid int,
	@storeid nvarchar(100)	
as
begin
	
	Delete 
	From A_Storage
	Where (storageid = @dbid)
		And (storeid = @storeid)
		
end
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_SessionStorage_FilterSessions' ))
	Drop procedure dbo.A_SessionStorage_FilterSessions
GO

Create procedure dbo.A_SessionStorage_FilterSessions
	@dtnow datetime = null
as
begin
	
	if ( @dtnow is NULL )
		Set @dtnow = GetDate()
	
	Delete From A_SessionStorage
	Where (DateAdd ( ms, storeinterval, storedt ) < @dtnow)
	
end
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Timer_UserDefinedProc' ))
	Drop procedure dbo.A_Timer_UserDefinedProc
GO

Create Procedure dbo.A_Timer_UserDefinedProc
as
begin
	-- Произвольная процедура, исполняемая периодически (раз в минуту по умолчанию).
	-- Не претерпевает изменений при обновлениях.
	-- Здесь можно любой необходимый код исполнять.
	-- Нельзя держать пустой и допускать ошибки.
	
	Declare @a int
	
end
GO

-----------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Timer_MinuteProc' ))
	Drop procedure dbo.[A_Timer_MinuteProc]
GO


Create Procedure [dbo].[A_Timer_MinuteProc]
	@dtnow datetime = null
as
begin
	if ( @dtnow is null )	
		set @dtnow = GetDate()
--------------------------------------------------------------------------------------------
-- Обеспечивается однократность исполнения (при нескольких серверах к одной базе) 
--------------------------------------------------------------------------------------------
   	Begin tran DbProcActive

		Declare @dtactive datetime
		Select @dtactive = valuedatetime from A_Settings where name like 'DB_procactive'
		if ( not @dtactive is null )
 		  if ( DateDiff ( mi, @dtactive, @dtnow ) < 10 ) begin
		    Rollback tran DbProcActive
		    return
		  end

		Update A_Settings Set valuedatetime = @dtnow where name like 'DB_procactive'
		if ( @@rowcount = 0 )
		  Insert into A_Settings (name, valuedatetime) Values ( 'DB_procactive', @dtnow )

   	Commit tran DbProcActive
--------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------

	-- Очистка неактуальных регистрационных сессий
	exec dbo.A_Registrar_FilterSessions

	--Время по которое производить расчет намеренно назад на 30 секунд отводим
	-- чтобы исключить возможность попадания в интервалы, когда отложенные сохранения в БД идут
	-- и когда одна таблица уже заполнена, а другая еще нет данными до текущего момента.
	-- ну и еще случаи частичного заполнения строки, когда данные вставляются в момент выборки
	Declare @tilldt DateTime
	Set @tilldt = DateAdd ( ss, -30, @dtnow )

	exec A_CallCenter_Temp_UpdateOperatorStates @tilldt
	
	-- Очистка неактуальных пользовательских сессий
	exec dbo.A_SessionStorage_FilterSessions @dtnow
	
	-- Пользовательская процедура, которой можно пользоваться и изменять
	exec dbo.A_Timer_UserDefinedProc

--------------------------------------------------------------------------------------------
-- Обеспечивается однократность исполнения (при нескольких серверах к одной базе) 
--------------------------------------------------------------------------------------------
	Delete From A_Settings where name like 'DB_procactive'
--------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------
end
GO