﻿
--------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Policy_SubjectGroups_GetList' ))
	Drop procedure dbo.A_Policy_SubjectGroups_GetList
GO

Create procedure dbo.A_Policy_SubjectGroups_GetList
	@objecttype int = 0
as 
begin

	-- Возвращает список всех именованных субъектов, доступных для назначения прав

	if exists ( Select * From tempdb..sysobjects where id=object_id('tempdb..#subjects'))
		Drop table #subjects
		
	Create table #subjects ( [type] int, [name] nvarchar(500), [id] uniqueidentifier, [extraId] uniqueidentifier, [equalMode] int )


	-- Пользователи
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 1, [Name], [Id], null, 2
		From A_Users u
		Where u.IsDeleted = 0
	
	-- Отделы
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 2, [Name], [Id], null, 2
		From A_Groups g	
	
	-- Роли
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 3, r.[Name], r.[Id], null, 2
		From A_UserRoles_Roles r
		Where not exists ( Select Id from A_Users u Where u.Id = r.Id )
			and Domination = 0
	
	
	-- Все операторы	
	Declare @alloperators nvarchar(200)
	Select @alloperators = langvalue 
	From A_LangTexts
	Where langkey like 'alloperators'
	if ( @alloperators is NULL )
		Set @alloperators = 'Все операторы'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
		Select 4, @alloperators, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 4
		
		
	if ( @objecttype in ( 0, 101, 102, 105, 106, 107 )) begin
	
		-- Операторы текущего проекта
		Declare @currentprojectoperators nvarchar(200)
		Select @currentprojectoperators = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectoperators'
		if ( @currentprojectoperators is NULL )
			Set @currentprojectoperators = 'Операторы текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 12, @currentprojectoperators, st.[groupId], null, 1
			From A_Policy_SubjectTypes st
			Where st.[id] = 12
	end
	
	
	---- Операторы задачи
	--Declare @taskoperators nvarchar(200)
	--Select @taskoperators = langvalue 
	--From A_LangTexts
	--Where langkey like 'taskoperators'
	--if ( @taskoperators is NULL )
	--	Set @taskoperators = 'Операторы задачи'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 5, @taskoperators + ' «' + t.[Name] + '»', t.[Id], null, 2
	--	From A_TaskManager_Tasks t
		
		
	---- Операторы проекта
	--Declare @projectoperators nvarchar(200)
	--Select @projectoperators = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectoperators'
	--if ( @projectoperators is NULL )
	--	Set @projectoperators = 'Операторы проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 5, @projectoperators + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p
	
	
	-- Все супервизоры
	Declare @allsupervisors nvarchar(200)
	Select @allsupervisors = langvalue 
	From A_LangTexts
	Where langkey like 'allsupervisors'
	if ( @allsupervisors is NULL )
		Set @allsupervisors = 'Все супервизоры'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )		
		Select 7, @allsupervisors, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 7
	
	
	if ( @objecttype in ( 0, 101, 102, 105, 106, 107 )) begin
	
		-- Супервизоры текущего проекта	
		Declare @currentprojectsupervisors nvarchar(200)
		Select @currentprojectsupervisors = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectsupervisors'
		if ( @currentprojectsupervisors is NULL )
			Set @currentprojectsupervisors = 'Супервизоры текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 13, @currentprojectsupervisors, st.[groupId], null, 1
			From A_Policy_SubjectTypes st
			Where st.[id] = 13	
	end
	
	---- Супервизоры задачи
	--Declare @tasksupervisors nvarchar(200)
	--Select @tasksupervisors = langvalue 
	--From A_LangTexts
	--Where langkey like 'tasksupervisors'
	--if ( @tasksupervisors is NULL )
	--	Set @tasksupervisors = 'Супервизоры задачи'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 8, @tasksupervisors + ' «' + t.[Name] + '»', t.[Id], null, 2
	--	From A_TaskManager_Tasks t
		
		
	---- Супервизоры проекта
	--Declare @projectsupervisors nvarchar(200)
	--Select @projectsupervisors = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectsupervisors'
	--if ( @projectsupervisors is NULL )
	--	Set @projectsupervisors = 'Супервизоры проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 9, @projectsupervisors + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p
	

	---- Владельцы проекта
	--Declare @projectowners nvarchar(200)
	--Select @projectowners = langvalue 
	--From A_LangTexts
	--Where langkey like 'projectowners'
	--if ( @projectowners is NULL )
	--	Set @projectowners = 'Владельцы проекта'
		
	--Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
	--	Select 10, @projectowners + ' «' + p.[Name] + '»', p.[Id], null, 2
	--	From A_TaskManager_Projects p


	-- По другим привилегиями других объектов - ниже
	
	
	-- Все пользователи
	Declare @allusers nvarchar(200)
	Select @allusers = langvalue 
	From A_LangTexts
	Where langkey like 'allusers'
	if ( @allusers is NULL )
		Set @allusers = 'Все пользователи'
		
	Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )		
		Select 14, @allusers, st.[groupId], null, 1
		From A_Policy_SubjectTypes st
		Where st.[id] = 14


	-- По привилегиям всех объектов - слишком много попадет в список. Пропустим пока.
	-- Только на проекты сошлемся
	
	if ( @objecttype in ( 0, 102, 105, 106, 107 )) begin
		
		Declare @idempty uniqueidentifier
		Set @idempty = '00000000-0000-0000-0000-000000000000'
		
		
		-- Читатели текущего проекта
		Declare @currentprojectviewers nvarchar(200)
		Select @currentprojectviewers = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectviewers'
		if ( @currentprojectviewers is NULL )
			Set @currentprojectviewers = 'Пользователи с правом просмотра текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 101, @currentprojectviewers, @idempty, 'F876E650-7972-475A-AE96-BCCF831AF818', 4
					
						
		-- Редакторы текущего проекта
		Declare @currentprojectredactors nvarchar(200)
		Select @currentprojectredactors = langvalue 
		From A_LangTexts
		Where langkey like 'currentprojectredactors'
		if ( @currentprojectredactors is NULL )
			Set @currentprojectredactors = 'Пользователи с правом изменения текущего проекта'
			
		Insert into #subjects ( [type], [name], [id], [extraId], [equalMode] )
			Select 101, @currentprojectredactors, @idempty, 'C892CC9E-4660-4ABC-8AD1-BAECDBFEAAC3', 4
						
	end
	
	
	-- Итоговый вывод
	Select s.[type], tp.[priority], s.[name], s.[id], s.[extraId], s.[equalMode]
	From #subjects s
		Inner join dbo.A_Policy_SubjectTypePriority tp on tp.[idType] = s.[type]
	Order By tp.[priority], s.[name]
	
end
GO

-----------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id('dbo.A_DB_RemoveTails' ))
	Drop procedure dbo.[A_DB_RemoveTails]
GO

Create Procedure [dbo].[A_DB_RemoveTails] 
as
begin

-- Отрубание глюков после изменений в БД. Лишние ссылки, пустые элементы и т.д.

		-----------------------------------------------------
		-- преобразования на стыке версий -------------------
		-----------------------------------------------------

		Update dbo.A_ServerExtLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerGSMLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerIntLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerSipLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerUSBLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		Update dbo.A_ServerVirtualLines
		Set SystemNumStr = Cast ( SystemNum as nvarchar(10 ))
		Where SystemNumStr is NULL

		-----------------------------------------------------
		-- общие возможные глюки ----------------------------
		-----------------------------------------------------
		Delete From A_WorkPlaces where (type in (2,3,4,6)) and not (parentId in (Select [ID] From A_WorkPlaces))
		Delete From A_WorkPlaces where (type in (2,3,4,6)) and not ( LineId in ( Select [ID] From A_View_ServerIntLinesAll ))
		Delete From A_ServerExtLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerIntLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerGSMLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerUSBLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerSipLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		Delete From A_ServerUSBLines where not [id] in (Select LineID From A_WORKPLACES)
		Delete From A_ServerVirtualLines where not [ServerID] in (Select [id] From A_WORKPLACES)
		
		Delete From A_LinesReactors where not [WPID] in ( Select [id] From A_WORKPLACES )
		Delete From A_LinesReactors where not LineID in ( Select [ID] From A_View_ServerLinesAll )
		Delete From A_ServerNumbers where not [id] in ( Select [ID] From A_WorkPlaces Where type=5 )
		Delete From A_ServerNumbers where [id] in ( Select [ID] From A_WorkPlaces where hasdatabase>0 )

		Delete From A_CallRoutes_Lines where not [idroute] in ( Select [ID] From A_CallRoutes )

		Declare @t table(id_ uniqueidentifier)
		insert into @t
			Select [id] From A_WORKPLACES Union 
			Select [id] From A_Users union
			Select [id] From A_ExtNumbers union
			Select [id] From A_View_ServerLinesAll			

		
		Delete From A_RuleRecords where not RuleID in (Select ID From A_Rules)

		--записи правил, где пользователь или линия несуществующие
		Delete From A_RuleRecords where (InnerAddressType in (0,1)) and ( not ReactID in (Select id_ From @t ))
		--записи правил, где поток несуществующий
		Delete From A_RuleRecords where (InnerAddressType in (3)) and ( not ReactID in (Select [Id] From A_ServerStreams ))

		Delete From A_NumberPlanAction where numtype=0 and extraid not in (Select ID From A_Rules)
		Delete From A_NumberPlan where id not in (Select numid From A_NumberPlanAction)
		Delete From A_RuleRecords where InnerAddressType=2 and not ReactID in (Select ID From A_NumberPlan)
		Delete From A_Rules	Where not id in ( Select ruleid From A_RuleRecords )		

		Delete From A_TaskManager_TaskExtLines where not ExtLineId in ( Select id From A_ServerExtLines )
		Delete From A_TaskManager_Operators where not OperatorId in ( Select id From A_Users u inner join A_UserParams up on u.Id = up.IdUser Where up.IsOperator = 1 Union Select Id From C_Groups )
		Delete From A_TaskManager_TaskProperties where not IdTask in ( Select id From A_TaskManager_Tasks )
		Delete From A_TaskManager_Lists Where (ProjectId = '00000000-0000-0000-0000-000000000000') and not exists ( Select Id From A_TaskManager_Tasks t where ListId = A_TaskManager_Lists.Id )

		-----------------------------------------------------
		-- сетевые адреса рабочих мест = адресу сервера - заменяем брехней 
		-----------------------------------------------------
		Declare @svraddr nvarchar ( 100 )
		Select @svraddr = Address 
		From A_WorkPlaces
		Where HasFastEngine = 1
		Update A_WorkPlaces Set Address = 'Ошибка в адресе' 
		Where Address like @svraddr and HasFastEngine = 0

		-----------------------------------------------------
		-- Null в 0 в ID юзеров по умолчанию ----------------
		-----------------------------------------------------
		Update A_ServerIntLines
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		Update A_ServerSipLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		Update A_ServerVirtualLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where DefaultUserId is null

		-----------------------------------------------------
		-- в 0 несуществующих юзеров по умолчанию -----------
		-----------------------------------------------------
		Update A_ServerIntLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id From A_Users)

		Update A_ServerSipLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id From A_Users)

		Update A_ServerVirtualLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		Where not DefaultUserId in (Select id From A_Users)

		-----------------------------------------------------
		-- двоение юзеров по умолчанию в ссылках по линиям --
		-----------------------------------------------------
		Declare @useddefusers table (id uniqueidentifier)
		Declare @idtozero table (id uniqueidentifier)
		Declare @lid uniqueidentifier
		Declare @uid uniqueidentifier
		Declare @tmp bit 		

		Declare defusercur Cursor For
			Select Id, DefaultUserId 
			From A_View_ServerIntLinesDefaultUsers
			Where DefaultUserId != '00000000-0000-0000-0000-000000000000'

		Open defusercur
		Fetch Next From defusercur Into @lid, @uid
		While @@FETCH_STATUS = 0
		Begin		
			Select @tmp = 1 From @useddefusers where id = @uid
			if ( @tmp > 0 )
				Insert Into @idtozero (id) values (@lid)
			else
				Insert Into @useddefusers (id) values (@uid)
			Fetch Next From defusercur  Into @lid, @uid
		End
		Close defusercur
		Deallocate defusercur
		
		Update A_ServerIntLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerIntLines si, @idtozero iz
		Where si.Id = iz.Id

		Update A_ServerSipLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerSipLines si, @idtozero iz
		Where si.Id = iz.Id

		Update A_ServerVirtualLines 
		Set DefaultUserId = '00000000-0000-0000-0000-000000000000'
		From A_ServerVirtualLines si, @idtozero iz
		Where si.Id = iz.Id
		
		-----------------------------------------------------
		-- удаляем несуществующие значения ------------------
		-----------------------------------------------------
		Begin Tran UIT

		Delete From A_TaskManager_UserInTask with ( TabLock, HoldLock )
		Where not exists ( Select Id From A_Users Where A_Users.Id = UserId )

		Delete From A_TaskManager_UserInTask --with ( TabLock )
		Where not exists ( Select Id From A_TaskManager_Tasks Where A_TaskManager_Tasks.Id = TaskId )
		
		Commit tran UIT


		Delete From A_TaskManager_Operators
		Where not exists ( Select Id From A_TaskManager_Tasks where A_TaskManager_Tasks.Id = TaskId )

		Delete From A_TaskManager_TaskExtLines
		Where not exists ( Select Id From A_TaskManager_Tasks where A_TaskManager_Tasks.Id = TaskId )

		Delete From A_TaskManager_Operators
		Where IsGroup = 1 and not exists ( Select Id From C_Groups where C_Groups.Id = OperatorId )

		Delete From A_TaskManager_Operators
		Where IsGroup = 1 and not Exists ( Select * From A_Users where A_Users.Id = OperatorId )
		

		Declare @uidt table ( id uniqueidentifier )
		Insert into @uidt
			Select Id From A_Users
		Delete From A_UserControls
		Where not UserA in (Select id From @uidt)
			or not UserB in (Select id From @uidt)
		
		-----------------------------------------------------
		-- Роли на удаленных юзеров -------------------------
		-----------------------------------------------------
		
		Delete From A_UserRoles_Roles
		Where (Domination = 1)
			and ( not Id in (Select id From A_Users))
			
		Delete
		From A_UserRoles_RolesOfUsers
		Where not IdUser in (Select Id From A_Users)
	
		Delete
		From A_UserRoles_RolesOfUsers
		Where not IdRole in (Select Id From A_UserRoles_Roles)
		
		Delete
		From A_UserRoles_RightsOfRoles
		Where not IdRole in (Select Id From A_UserRoles_Roles)

		-----------------------------------------------------
		-- Редиректы на удаленных юзеров --------------------
		-----------------------------------------------------
		Delete 
		From A_Redirect
		Where IdUser not in ( Select Id From A_Users )

		-----------------------------------------------------
		-- Пустые группы, номера, несвязанные правила -------
		-----------------------------------------------------
		exec A_Admin_UserGroup_Group_SiftEmpty		

		exec A_Admin_Number_SiftByActions
		
		exec A_Admin_Rule_SiftByNumbers
		
		-----------------------------------------------------
		-- неиспользуемые плагин-модули ---------------------
		-----------------------------------------------------
		Delete From A_PluginMenu_User
		Where not exists ( Select Id From A_Users u Where u.Id = A_PluginMenu_User.IdUser )

		-----------------------------------------------------
		-- канонические адреса ------------------------------
		-----------------------------------------------------	
		Delete from A_WorkPlace_Address
		
		-----------------------------------------------------	
		-- Очистка пустых ссылок в таблицах политики прав доступа к объектам
		-----------------------------------------------------	
		exec dbo.A_Policy_CheckFilter
end
GO
