﻿

--------------------------------------
-- WO_Dashboards_Dashboards_GetList --
--------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboards_GetListPrivileges]'))
DROP procedure [dbo].[WO_Dashboards_Dashboards_GetListPrivileges]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboards_GetListPrivileges]
	@authorizeduserid	uniqueidentifier
as
begin

	-- Возвращает информацию о привилегиях пользователя по всем дашбордам
	
	-- Привилегии доступные пользователю по дашбордам
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#privileges' ))
		Drop table #privileges
	Create table #privileges ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200))
	Insert into #privileges ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 105
			--@idparentproject = NULL,
			--@idprivilege = '495DEB4D-D5FF-4C52-A219-541C3AEC6CAC'


	-- Набор привилегий для каждого из доступных дашбордов
	Select [idObject], [idPrivilege], [code]
	From #privileges
	Order by [idObject]
	
end
GO


---------------------------------
-- WO_Dashboards_Dashboard_Get --
---------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Get]'))
DROP procedure [dbo].[WO_Dashboards_Dashboard_Get]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Get]
	@authorizeduserid	uniqueidentifier,
	@dashboardid    uniqueidentifier
as
begin
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#pp' ))
		Drop table #pp
	Create table #pp ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200), [type] int)
	Insert into #pp ( [idObject], [idPrivilege], [code], [type] )
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @authorizeduserid,
			@idobject = @projectid,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'
	
	/* Привилегия чтения по дашборду есть?
	*/
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#dp' ))
		Drop table #dp
	Create table #dp ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200), [type] int)
	Insert into #dp ( [idObject], [idPrivilege], [code], [type] )
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @authorizeduserid,
			@idobject = @dashboardid,
			@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863'			
			
	
	-- Достаем из пересечения
	Select	d.Id,
			d.IdProject,
			d.Name,
			d.Creator,
			d.CreationDate
	From	[dbo].[WO_Dashboards_Dashboards] d
		Inner join #dp dp on dp.IdObject = d.Id
	Where (d.Id = @dashboardid)	
		and ((d.IdProject is NULL ) 
			or (exists (Select idObject From #pp pp Where pp.idObject = d.IdProject)))
	
end
GO
