﻿-------------------------------------
-----     Создание процедур     -----
-------------------------------------

IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Create]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Dashboard_Create]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Create]
	@authorizeduserid	uniqueidentifier,
	@dashboardname	nvarchar(100),
	@projectid	nvarchar(50)
as
begin
	
	declare	@en bit
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode = 'DashboardCreate', @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	if ((select @en) = 1)
		begin
			
			declare	@dashboardid uniqueidentifier
			set	@dashboardid = newid()
			
			if ( @projectid like '' )
				Set @projectid = NULL
			
			-- Добавление нового дашборда
			insert into [dbo].[WO_Dashboards_Dashboards] (Id, IdProject, Name, Creator, CreationDate)
				select	@dashboardid,
						@projectid,
						@dashboardname,
						@authorizeduserid,
						getutcdate()
			
			select	@dashboardid dashboardid
			
			/* Default privileges (Non IVR Lists) 
			*/
			exec A_Policy_Objects_Save
				@idObject = @dashboardid,
				@type = 105,
				@name = @dashboardname,
				@idParentProject = @projectid,
				@actionUserId = @authorizeduserid
			
		end
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Get]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Dashboard_Get]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Get]
	@authorizeduserid	uniqueidentifier,
	@dashboardid    uniqueidentifier
as
begin
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#pp' ))
		Drop table #pp
	Create table #pp ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default, [type] int)
	Insert into #pp ( [idObject], [idPrivilege], [code], [type] )
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @authorizeduserid,
			@idobject = @projectid,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'
	
	/* Привилегия чтения по дашборду есть?
	*/
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#dp' ))
		Drop table #dp
	Create table #dp ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default, [type] int)
	Insert into #dp ( [idObject], [idPrivilege], [code], [type] )
		exec A_Policy_SubjectPrivileges_GetForUserByObject
			@iduser = @authorizeduserid,
			@idobject = @dashboardid,
			@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863'			
			
	
	-- Достаем из пересечения
	Select	d.Id,
			d.IdProject,
			d.Name,
			d.Creator,
			d.CreationDate
	From	[dbo].[WO_Dashboards_Dashboards] d
		Inner join #dp dp on dp.IdObject = d.Id
	Where (d.Id = @dashboardid)	
		and ((d.IdProject is NULL ) 
			or (exists (Select idObject From #pp pp Where pp.idObject = d.IdProject)))
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Remove]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Dashboard_Remove]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Remove]
	@authorizeduserid	uniqueidentifier,
	@dashboardid	uniqueidentifier
as
begin
	
	declare	@can bit
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			-- Сами дашборды
			delete
			from	[dbo].[WO_Dashboards_Dashboards]
			where	Id = @dashboardid
			
			-- Привязанные поля (для таблицы и запроса)
			delete
			from	[dbo].[WO_Dashboards_Widgets_Fields]
			where	IdFields in
								(
									select	IdFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection in
																				(
																					select	Id
																					from	[dbo].[WO_Dashboards_Sections]
																					where	IdDashboard = @dashboardid
																				)
													)
								)
			
			-- Привязанные поля (для конструктора)
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
			where	IdConstructorFields in
								(
									select	IdConstructorFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection in
																				(
																					select	Id
																					from	[dbo].[WO_Dashboards_Sections]
																					where	IdDashboard = @dashboardid
																				)
													)
								)
			
			-- Фильтры (для таблицы и запроса)
			delete
			from	[dbo].[WO_Dashboards_Widgets_Filters]
			where	IdFilters in
								(
									select	IdFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection in
																				(
																					select	Id
																					from	[dbo].[WO_Dashboards_Sections]
																					where	IdDashboard = @dashboardid
																				)
													)
								)
			
			-- Фильтры (для конструктора)
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
			where	IdConstructorFilters in
								(
									select	IdConstructorFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection in
																				(
																					select	Id
																					from	[dbo].[WO_Dashboards_Sections]
																					where	IdDashboard = @dashboardid
																				)
													)
								)
			
			-- Виджеты
			delete
			from	[dbo].[WO_Dashboards_Widgets]
			where	IdSection in
								(
									select	Id
									from	[dbo].[WO_Dashboards_Sections]
									where	IdDashboard = @dashboardid
								)
			
			-- Борды
			delete
			from	[dbo].[WO_Dashboards_Sections]
			where	IdDashboard = @dashboardid
			
			
			-- Из таблиц политики прав доступа к объектам
			exec A_Policy_Objects_Remove @dashboardid
			
		end
		
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboard_Rename]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Dashboard_Rename]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboard_Rename]
	@authorizeduserid	uniqueidentifier,
	@dashboardid	uniqueidentifier,
	@dashboardnewname	nvarchar(100)
as
begin
	
	declare	@can bit
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			-- Обновляет название
			Update [dbo].[WO_Dashboards_Dashboards]
			Set	Name = @dashboardnewname
			Where Id = @dashboardid
			
			/* Default privileges
			*/
			exec A_Policy_Objects_Save
				@idObject = @dashboardid,
				@type = 105,
				@name = @dashboardnewname,
				@idParentProject = NULL, --во время переименования не волнует
				@actionUserId = @authorizeduserid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboards_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Dashboards_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboards_GetList]
	@authorizeduserid	uniqueidentifier
as
begin

	-- Доступные пользователю проекты по привилегии ЧТЕНИЕ
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#projects' ))
		Drop table #projects
	Create table #projects ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #projects ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 101,
			--@idparentproject = NULL,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'

	-- Доступные пользователю дашборды по привилегии ЧТЕНИЕ
	if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#dashboards' ))
		Drop table #dashboards
	Create table #dashboards ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #dashboards ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 105,
			--@idparentproject = NULL,
			@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863'

	
	Select	d.Id,
			d.IdProject,
			d.Name,
			d.Creator,
			d.CreationDate
	From	[dbo].[WO_Dashboards_Dashboards] d
		Inner join #dashboards p on p.IdObject = d.Id
	Where ( d.IdProject is NULL )
		or exists ( Select IdObject From #projects pr Where pr.IdObject = d.IdProject )
	Order by d.Name asc
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Dashboards_GetListPrivileges]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Dashboards_GetListPrivileges]
GO

CREATE procedure [dbo].[WO_Dashboards_Dashboards_GetListPrivileges]
	@authorizeduserid	uniqueidentifier
as
begin

	-- Возвращает информацию о привилегиях пользователя по всем дашбордам
	
	-- Привилегии доступные пользователю по дашбордам
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#privileges' ))
		Drop table #privileges
	Create table #privileges ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #privileges ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 105
			--@idparentproject = NULL,
			--@idprivilege = '495DEB4D-D5FF-4C52-A219-541C3AEC6CAC'


	-- Набор привилегий для каждого из доступных дашбордов
	Select [idObject], [idPrivilege], [code]
	From #privileges
	Order by [idObject]
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Projects_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Projects_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Projects_GetList]
	@authorizeduserid	uniqueidentifier
as
begin

	-- Доступные пользователю проекты по привилегии ЧТЕНИЕ
	if exists ( Select * from tempdb..sysobjects where id=object_id ( 'tempdb..#projects' ))
		Drop table #projects
	Create table #projects ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200) collate database_default)
	Insert into #projects ( [idObject], [idPrivilege], [code] )
		exec A_Policy_SubjectPrivileges_GetForUserByObjectType
			@iduser = @authorizeduserid,
			@objecttype = 101,
			--@idparentproject = NULL,
			@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818'
			
	
	-- Достаем инфу по проектам
	Select	p.Id,
			p.Name
	From [dbo].[A_TaskManager_Projects] p
		Inner join #projects pp on pp.idObject = p.Id
	Order by p.Name asc
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Section_Create]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Section_Create]
GO

CREATE procedure [dbo].[WO_Dashboards_Section_Create]
	@authorizeduserid	uniqueidentifier,
	@dashboardid	uniqueidentifier,
	@sectionname	nvarchar(100)
as
begin
	
	declare	@can bit
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			
			declare	@sectionid uniqueidentifier
				set	@sectionid = newid()
			
			insert into [dbo].[WO_Dashboards_Sections]
				select	@sectionid,
						@sectionname,
						getutcdate(),
						null,
						@dashboardid
			
			select	@sectionid sectionid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Section_Remove]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Section_Remove]
GO

CREATE procedure [dbo].[WO_Dashboards_Section_Remove]
	@authorizeduserid	uniqueidentifier,
	@sectionid	uniqueidentifier
as
begin
	
	declare	@can bit
	declare	@dashboardid uniqueidentifier
	select @dashboardid = IdDashboard from [dbo].[WO_Dashboards_Sections] where Id = @sectionid
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			
			delete
			from	[dbo].[WO_Dashboards_Sections]
			where	Id = @sectionid
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Fields]
			where	IdFields in
								(
									select	IdFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection = @sectionid
													)
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
			where	IdConstructorFields in
								(
									select	IdConstructorFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection = @sectionid
													)
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Filters]
			where	IdFilters in
								(
									select	IdFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection = @sectionid
													)
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
			where	IdConstructorFilters in
								(
									select	IdConstructorFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id in
													(
														select	Id
														from	[dbo].[WO_Dashboards_Widgets]
														where	IdSection = @sectionid
													)
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets]
			where	IdSection = @sectionid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Section_Rename]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Section_Rename]
GO

CREATE procedure [dbo].[WO_Dashboards_Section_Rename]
	@authorizeduserid	uniqueidentifier,
	@sectionid	uniqueidentifier,
	@sectionnewname	nvarchar(100)
as
begin
	
	declare	@can bit
	declare	@dashboardid uniqueidentifier
	select @dashboardid = IdDashboard from [dbo].[WO_Dashboards_Sections] where Id = @sectionid
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			
			update	[dbo].[WO_Dashboards_Sections]
			set	Name = @sectionnewname
			where	Id = @sectionid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Sections_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Sections_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Sections_GetList]
	@authorizeduserid	uniqueidentifier,
	@dashboardid	uniqueidentifier
as
begin
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	Declare @pcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @projectid,
		@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
		@onlyreturncount = 1,
		@resultcount = @pcount OUTPUT
			
	if ((@pcount = 0) and (@projectid is not null)) 
		RAISERROR ('Access error', 16, 1)
	
	/* Привилегия чтения по дашборду есть?
	*/
	Declare @dcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @dashboardid,
		@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863',
		@onlyreturncount = 1,
		@resultcount = @dcount OUTPUT
		
	if (@dcount = 0) 
		RAISERROR ('Access error', 16, 1)
	
	
	
	select	Id,
			Name
	from	[dbo].[WO_Dashboards_Sections]
	where	IdDashboard = @dashboardid
	order by isnull(Position, 2147483647) asc, CreationDate asc
	
end
GO


IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_GetLang]'))
DROP PROCEDURE [dbo].[WO_Dashboards_GetLang]
GO

CREATE procedure [dbo].[WO_Dashboards_GetLang]
as
begin

	--if exists (select * from tempdb..sysobjects where id = object_id('tempdb..#langtext'))
	--	drop table #langtext	
	--create table #langtext (langkey nvarchar(200), langvalue nvarchar(4000) collate database_default)
	
	Declare @langtext table (langkey nvarchar(200) collate database_default, langvalue nvarchar(4000) collate database_default)
	
	insert into @langtext (langkey, langvalue) values ('intervals5minute', 'Интервалы по 5 минут')
	insert into @langtext (langkey, langvalue) values ('intervals15minute' ,'Интервалы по 15 минут')
	insert into @langtext (langkey, langvalue) values ('intervals30minute', 'Интервалы по 30 минут')
	insert into @langtext (langkey, langvalue) values ('intervals60minute', 'Интервалы по 60 минут (по часам)')
	insert into @langtext (langkey, langvalue) values ('intervalsday', 'Интервалы по дням')
	insert into @langtext (langkey, langvalue) values ('intervalsweek', 'Интервалы по неделям')
	insert into @langtext (langkey, langvalue) values ('intervalsmonth', 'Интервалы по месяцам')
	insert into @langtext (langkey, langvalue) values ('intervalsyear', 'Интервалы по годам')
	insert into @langtext (langkey, langvalue) values ('averagespeedresponseASA', 'Cредняя скорость ответа (ASA)')
	insert into @langtext (langkey, langvalue) values ('averagevalue', 'Среднее значение')
	insert into @langtext (langkey, langvalue) values ('totalvalue' ,'Суммарное значение')
	insert into @langtext (langkey, langvalue) values ('maximumvalue', 'Максимальное значение')
	insert into @langtext (langkey, langvalue) values ('minimumvalue', 'Минимальное значение')
	insert into @langtext (langkey, langvalue) values ('date', 'Дата')
	insert into @langtext (langkey, langvalue) values ('timeperiod', 'Время')
	insert into @langtext (langkey, langvalue) values ('today', 'Сегодня')
	insert into @langtext (langkey, langvalue) values ('operators', 'Операторы')
	insert into @langtext (langkey, langvalue) values ('tasks', 'Задачи')
	insert into @langtext (langkey, langvalue) values ('projects', 'Проекты')
	insert into @langtext (langkey, langvalue) values ('operator', 'Оператор')
	insert into @langtext (langkey, langvalue) values ('task', 'Задача')
	insert into @langtext (langkey, langvalue) values ('project', 'Проект')
	insert into @langtext (langkey, langvalue) values ('intervalsweekdays', 'Интервалы по дням недели')
	insert into @langtext (langkey, langvalue) values ('period', 'Период')
	insert into @langtext (langkey, langvalue) values ('countsuccessfulcallstasks', 'Количество успешных звонков по задачам')
	insert into @langtext (langkey, langvalue) values ('countunsuccessfulcallstasks', 'Количество неуспешных звонков по задачам')
	insert into @langtext (langkey, langvalue) values ('countcallstasks', 'Количество звонков по задачам')
	insert into @langtext (langkey, langvalue) values ('daterelative', 'Дата (относительная)')
	insert into @langtext (langkey, langvalue) values ('dateperiod', 'Дата (период)')
	
	Update temp
	Set temp.langvalue = lang.langvalue 
	From @langtext temp, [dbo].[A_LangTexts] lang
	Where temp.langkey = lang.langkey	
	
	Select *
	From @langtext
	
end
GO


IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Designer_Axis_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Designer_Axis_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Designer_Axis_GetList]
	@authorizeduserid	uniqueidentifier
as
begin

	-- lang
	if exists (select * from tempdb..sysobjects where id = object_id('tempdb..#langtext'))
		drop table #langtext	
	create table #langtext (langkey nvarchar(200) collate database_default, langvalue nvarchar(4000) collate database_default)
	
	insert into #langtext
		exec dbo.[WO_Dashboards_GetLang]
		
		
	select	a.Id DesignerAxisId,
			a.Axis DesignerAxisCode,
			lt.langvalue DesignerAxisName
	from	[dbo].[WO_Dashboards_Widgets_Designer_Axis] a
				inner join #langtext lt on a.Axis = lt.langkey
	order by a.Axis asc
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Designer_Filter_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Designer_Filter_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Designer_Filter_GetList]
	@authorizeduserid	uniqueidentifier
as
begin
	
	-- lang
	if exists (select * from tempdb..sysobjects where id = object_id('tempdb..#langtext'))
		drop table #langtext	
	create table #langtext (langkey nvarchar(200) collate database_default, langvalue nvarchar(4000) collate database_default)
	
	insert into #langtext
		exec dbo.[WO_Dashboards_GetLang]
		
		
	select	f.Id DesignerFilterId,
			f.Filter DesignerFilterCode,
			lt.langvalue DesignerFilterName
	from	[dbo].[WO_Dashboards_Widgets_Designer_Filter] f
				inner join #langtext lt on f.Filter = lt.langkey
	order by f.Filter asc
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Designer_Filter_Value_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Designer_Filter_Value_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Designer_Filter_Value_GetList]
	@authorizeduserid	uniqueidentifier,
	@filterid	int
as
begin
	
--declare	@filterid int = 4

	-- lang
	if exists (select * from tempdb..sysobjects where id = object_id('tempdb..#langtext'))
		drop table #langtext	
	create table #langtext (langkey nvarchar(200) collate database_default, langvalue nvarchar(4000) collate database_default)
	
	insert into #langtext
		exec dbo.[WO_Dashboards_GetLang]
	
	-- дата
	if (@filterid = 1)
		begin
			
			select	fv.IdUniqueidentifier DesignerFilterValueId,
					fv.Value DesignerFilterValueCode,
					lt.langvalue DesignerFilterValueName
			from	[dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] fv
						inner join #langtext lt on fv.Value = lt.langkey
			where	fv.IdFilter = @filterid
			order by fv.Value asc
			
		end
	
	-- время
	if (@filterid = 2)
		begin
			
			select	fv.IdUniqueidentifier DesignerFilterValueId,
					fv.Value DesignerFilterValueCode,
					lt.langvalue DesignerFilterValueName
			from	[dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] fv
						inner join #langtext lt on fv.Value = lt.langkey
			where	fv.IdFilter = @filterid
			order by fv.Value asc
			
		end
	
	-- оператор
	if (@filterid = 3)
		begin
			
			select	Id DesignerFilterValueId,
					null DesignerFilterValueCode,
					Name DesignerFilterValueName
			from	[oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo]
			order by Name asc
			
		end
	
	-- задача
	if (@filterid = 4)
		begin
			
			select	Id DesignerFilterValueId,
					null DesignerFilterValueCode,
					Name DesignerFilterValueName
			from	[oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task]
			order by Name asc
			
		end
	
	-- проект
	if (@filterid = 5)
		begin
			
			select	Id DesignerFilterValueId,
					null DesignerFilterValueCode,
					Name DesignerFilterValueName
			from	[oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project]
			order by Name asc
			
		end	
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Designer_Function_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Designer_Function_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Designer_Function_GetList]
	@authorizeduserid	uniqueidentifier
as
begin

	-- lang
	if exists (select * from tempdb..sysobjects where id = object_id('tempdb..#langtext'))
		drop table #langtext	
	create table #langtext (langkey nvarchar(200) collate database_default, langvalue nvarchar(4000) collate database_default)
	
	insert into #langtext
		exec dbo.[WO_Dashboards_GetLang]
		
	
	select	f.Id DesignerFunctionId,
			f.[Function] DesignerFunctionCode,
			lt.langvalue DesignerFunctionName
	from	[dbo].[WO_Dashboards_Widgets_Designer_Function] f
				inner join #langtext lt on f.[Function] = lt.langkey
	order by f.[Function] asc
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Designer_Value_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Designer_Value_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Designer_Value_GetList]
	@authorizeduserid	uniqueidentifier
as
begin

	-- lang
	if exists (select * from tempdb..sysobjects where id = object_id('tempdb..#langtext'))
		drop table #langtext	
	create table #langtext (langkey nvarchar(200) collate database_default, langvalue nvarchar(4000) collate database_default)
	
	insert into #langtext
		exec dbo.[WO_Dashboards_GetLang]
	
	select	v.Id DesignerValueId,
			v.Value DesignerValueCode,
			lt.langvalue DesignerValueName
	from	[dbo].[WO_Dashboards_Widgets_Designer_Value] v
				inner join #langtext lt on v.Value = lt.langkey
	order by v.Value asc
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Designer_Values_Dependences_Get]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Designer_Values_Dependences_Get]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Designer_Values_Dependences_Get]
	@authorizeduserid	uniqueidentifier
as
begin
	
	declare	@en bit
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode = 'DashboardCreate', @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	if ((select @en) = 1)
		begin
			
			select	ValueId,
					FunctionId
			from	[dbo].[WO_Dashboards_Widgets_Designer_Dependences_Value_Function]
			
		end
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Query_Fields_Get]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Query_Fields_Get]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Query_Fields_Get]
	@authorizeduserid	uniqueidentifier,
	@querytext	ntext
as
begin
	
	declare	@en bit
	declare @caneditSQL bit
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode = 'DashboardCreate', @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	if ((select @en) = 1)
		begin
			
			exec A_UserRoles_CheckPrivilege @privilegecode = 'CanEditSQL', @authorizeduserid = @authorizeduserid, @isenabled = @caneditSQL OUTPUT
					if ((select @caneditSQL) = 1)
						begin
							--exec	('set fmtonly on ' + @querytext)
							exec	(@querytext)
						end
					else
						begin
							RAISERROR ('Access error', 16, 1)
						end
			
		end
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Remove]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Remove]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Remove]
	@authorizeduserid	uniqueidentifier,
	@widgetid	uniqueidentifier
as
begin
	
	declare	@can bit
	declare	@dashboardid uniqueidentifier
	select @dashboardid = IdDashboard from [dbo].[WO_Dashboards_Sections] where Id = (select IdSection from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
		
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Fields]
			where	IdFields =
								(
									select	IdFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id = @widgetid
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
			where	IdConstructorFields =
								(
									select	IdConstructorFields
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id = @widgetid
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Filters]
			where	IdFilters =
								(
									select	IdFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id = @widgetid
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
			where	IdConstructorFilters =
								(
									select	IdConstructorFilters
									from	[dbo].[WO_Dashboards_Widgets]
									where	Id = @widgetid
								)
			
			delete
			from	[dbo].[WO_Dashboards_Widgets]
			where	Id = @widgetid
			
		end
	
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Save]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Save]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Save]
	@authorizeduserid	uniqueidentifier,
	@sectionid	uniqueidentifier,
	@widgetparams	ntext
as
begin
	
	declare	@can bit
	declare	@caneditSQL bit
	declare	@dashboardid uniqueidentifier
	select @dashboardid = IdDashboard from [dbo].[WO_Dashboards_Sections] where Id = @sectionid
	exec A_Dashboards_CheckCanEdit @authorizeduserid = @authorizeduserid, @dashboardid = @dashboardid, @canedit = @can OUTPUT
	if ((select @can) = 1)
		begin
			
			declare	@widgetnewid uniqueidentifier
				set	@widgetnewid = newid()
			declare	@widgetfieldsnewid uniqueidentifier
				set	@widgetfieldsnewid = newid()
			declare	@widgetfiltersnewid uniqueidentifier
				set	@widgetfiltersnewid = newid()
			declare	@widgetconstructorfieldsnewid uniqueidentifier
				set	@widgetconstructorfieldsnewid = newid()
			declare	@widgetconstructorfiltersnewid uniqueidentifier
				set	@widgetconstructorfiltersnewid = newid()
			
			declare	@widgetid uniqueidentifier
			declare	@widgetname nvarchar(100)
			declare	@widgettype int
			declare	@widgetsource int
			
			
			declare	@ixml int
			
			
			
			exec sp_xml_preparedocument @ixml output, @widgetparams
			-------------------------------------------------------
			
			-- Создаем табличные переменные для хранения данных о проверках
			declare @tempxmldata table
				(
					[id] nvarchar(50) collate database_default,
					[name] nvarchar(100) collate database_default,
					[type] nvarchar(5) collate database_default,
					[source] nvarchar(5) collate database_default,
					[table_name] nvarchar(100) collate database_default,
					[query] ntext collate database_default,
					[order_by] nvarchar(100) collate database_default,
					[show_top_rows] nvarchar(10) collate database_default,
					[options] ntext collate database_default,
					[constructor_axis_id] nvarchar(5) collate database_default
				)
			
			declare @tempxmldatafields table
				(
					[name] nvarchar(100) collate database_default,
					[value] nvarchar(5) collate database_default
				)
			
			declare @tempxmldatafilters table
				(
					[name] nvarchar(100) collate database_default,
					[operation] nvarchar(5) collate database_default,
					[value] nvarchar(100) collate database_default
				)
			
			declare @tempxmldataconstructorfields table
				(
					[field_id] nvarchar(5) collate database_default,
					[function_id] nvarchar(5) collate database_default,
					[value] nvarchar(50) collate database_default
				)
			
			declare @tempxmldataconstructorfilter table
				(
					[filter_id] nvarchar(5) collate database_default,
					[operation] nvarchar(5) collate database_default,
					[value_id] nvarchar(50) collate database_default,
					[value_1] nvarchar(50) collate database_default,
					[value_2] nvarchar(50) collate database_default
				)
			
			-- Заполняем табличные переменные данными о проверках, которые требуется выполнить
			exec A_UserRoles_CheckPrivilege @privilegecode = 'CanEditSQL', @authorizeduserid = @authorizeduserid, @isenabled = @caneditSQL OUTPUT
					if ((select @caneditSQL) = 1)
						begin
							insert into @tempxmldata
								select	*
								from openxml (@ixml,  '/root', 2)
								with	(
											[id] nvarchar(50) 'id',
											[name] nvarchar(100) 'name',
											[type] nvarchar(5) 'type',
											[source] nvarchar(5) 'source',
											[table_name] nvarchar(100) 'table_name',
											[query] ntext 'query',
											[order_by] nvarchar(100) 'order_by',
											[show_top_rows] nvarchar(10) 'show_top_rows',
											[options] ntext 'options',
											[constructor_axis_id] nvarchar(5) 'constructor_axis_id'
										)
						end
					else
						begin
							insert into @tempxmldata
								select	*
								from openxml (@ixml,  '/root', 2)
								with	(
											[id] nvarchar(50) 'id',
											[name] nvarchar(100) 'name',
											[type] nvarchar(5) 'type',
											[source] nvarchar(5) 'source',
											[table_name] nvarchar(100) 'table_name',
											[query] ntext 'null',
											[order_by] nvarchar(100) 'order_by',
											[show_top_rows] nvarchar(10) 'show_top_rows',
											[options] ntext 'options',
											[constructor_axis_id] nvarchar(5) 'constructor_axis_id'
										)
						end
			
			--select * from @tempxmldata
			
			
			insert into @tempxmldatafields
				select	*
				from openxml (@ixml,  '/root/table_fields', 2)
				with	(
							[name] nvarchar(100) 'name',
							[value] nvarchar(5) 'value'
						)
			
			--select * from @tempxmldatafields
			
			
			insert into @tempxmldatafilters
				select	*
				from openxml (@ixml,  '/root/table_filter', 2)
				with	(
							[name] nvarchar(100) 'name',
							[operation] nvarchar(5) 'operation',
							[value] nvarchar(100) 'value'
						)
			
			--select * from @tempxmldatafilters
			
			
			insert into @tempxmldataconstructorfields
				select	*
				from openxml (@ixml,  '/root/constructor_fields', 2)
				with	(
							[field_id] nvarchar(5) 'field_id',
							[function_id] nvarchar(5) 'function_id',
							[value] nvarchar(50) 'value'
						)
			
			--select * from @tempxmldataconstructorfields
			
			
			insert into @tempxmldataconstructorfilter
				select	*
				from openxml (@ixml,  '/root/constructor_filter', 2)
				with	(
							[filter_id] nvarchar(5) 'filter_id',
							[operation] nvarchar(5) 'operation',
							[value_id] nvarchar(50) 'value_id',
							[value_1] nvarchar(50) 'value_1',
							[value_2] nvarchar(50) 'value_2'
						)
			
			--select * from @tempxmldataconstructorfilter
			
			--------------------------------
			exec sp_xml_removedocument @ixml
			
			
				
			if ((select [id] from @tempxmldata) is null or (select [id] from @tempxmldata) = '')
				begin
					--select	'new'
					-- Widget новый (добавляем)
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets]
					insert into [dbo].[WO_Dashboards_Widgets] ([Id], [IdSection], [Name], [Type], [Source], [IdFields], [IdFilters], [NameTable], [Query], [OrderBy], [ShowTopRows], [Options], [IdConstructorFields], [IdConstructorFilters], [IdConstructorAxis])
						select	@widgetnewid [Id],
								@sectionid [IdSection],
								txmld.name [Name],
								case when txmld.type = '' then null else cast(txmld.type as int) end [Type],
								case when txmld.source = '' then null else cast(txmld.source as int) end [Source],
								@widgetfieldsnewid [IdFields],
								@widgetfiltersnewid [IdFilters],
								txmld.table_name [NameTable],
								txmld.query [Query],
								txmld.order_by [OrderBy],
								case when txmld.show_top_rows = '' then null else cast(txmld.show_top_rows as int) end [ShowTopRows],
								txmld.options [Options],
								@widgetconstructorfieldsnewid [IdConstructorFields],
								@widgetconstructorfiltersnewid [IdConstructorFilters],
								case when txmld.constructor_axis_id = '' then null else cast(txmld.constructor_axis_id as int) end [IdConstructorAxis]
						from	@tempxmldata txmld
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets_Fields]
					insert into [dbo].[WO_Dashboards_Widgets_Fields] ([Id], [IdFields], [Name], [Value])
						select	newid() [Id],
								@widgetfieldsnewid [IdFields],
								txmldf.name [Name],
								case when txmldf.value = '' then null else cast(txmldf.value as int) end [Value]
						from	@tempxmldatafields txmldf
						where	txmldf.name is not null and txmldf.value is not null					-- update 2013-10-30
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets_Filters]
					insert into [dbo].[WO_Dashboards_Widgets_Filters] ([Id], [IdFilters], [Name], [Operation], [Value])
						select	newid() [Id],
								@widgetfiltersnewid [IdFields],
								txmldfl.name [Name],
								case when txmldfl.operation = '' then null else cast(txmldfl.operation as int) end [Operation],
								txmldfl.value [Value]
						from	@tempxmldatafilters txmldfl
						where	txmldfl.name is not null and txmldfl.operation is not null					-- update 2013-10-30
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets_Constructor_Fields]
					insert into [dbo].[WO_Dashboards_Widgets_Constructor_Fields] ([Id], [IdConstructorFields], [ConstructorValueId], [ConstructorFunctionId], [ConstantValue])
						select	newid() [Id],
								@widgetconstructorfieldsnewid [IdConstructorFields],
								case when txmldcf.field_id = '' then null else cast(txmldcf.field_id as int) end [ConstructorValueId],
								case when txmldcf.function_id = '' then null else cast(txmldcf.function_id as int) end [ConstructorFunctionId],
								txmldcf.value [ConstantValue]
						from	@tempxmldataconstructorfields txmldcf
						--where	txmldcf.function_id is not null or txmldcf.value is not null
					
					-- Заполнение [dbo].[WO_Dashboards_Widgets_Constructor_Filters]
					insert into [dbo].[WO_Dashboards_Widgets_Constructor_Filters] ([Id], [IdConstructorFilters], [ConstructorFilterId], [ConstructorFilterOperationId], [ConstructorFilterValueId], [ConstructorFilterValue1], [ConstructorFilterValue2])
						select	newid() [Id],
								@widgetconstructorfiltersnewid [IdConstructorFilters],
								case when txmldcfl.filter_id = '' then null else cast(txmldcfl.filter_id as int) end [ConstructorFilterId],
								case when txmldcfl.operation = '' then 0 else cast(txmldcfl.operation as int) end [ConstructorFilterOperationId],
								case when txmldcfl.value_id = '' then null else cast(txmldcfl.value_id as uniqueidentifier) end [ConstructorFilterValueId],
								txmldcfl.value_1 [ConstructorFilterValue1],
								txmldcfl.value_2 [ConstructorFilterValue2]
						from	@tempxmldataconstructorfilter txmldcfl
						where	((isnull(txmldcfl.value_id, '') != '') and (isnull(txmldcfl.value_1, '') = '') and (isnull(txmldcfl.value_2, '') = ''))
								or ((select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = txmldcfl.value_id) = 'today' and (isnull(txmldcfl.value_1, '') = '') and (isnull(txmldcfl.value_2, '') = ''))
								or ((isnull(txmldcfl.value_id, '') = '') and (isnull(txmldcfl.value_1, '') != '') and (isnull(txmldcfl.value_2, '') != ''))
								--								
								--(((select Filter from [dbo].[WO_Dashboards_Widgets_Designer_Filter] where IdUniqueidentifier = txmldcfl.value_id) = 'dateperiod') and txmldcfl.value_1 is not null and txmldcfl.value_2 is not null)
								--or
								--(((select Filter from [dbo].[WO_Dashboards_Widgets_Designer_Filter] where IdUniqueidentifier = txmldcfl.value_id) = 'timeperiod') and txmldcfl.value_1 is not null and txmldcfl.value_2 is not null)
								--or
								--txmldcfl.value_id is not null
					
					
					select	@widgetnewid WidgetId
				end
			else
				begin
					--select	'old'
					-- Widget есть уже такой (изменяем)
					
					-- Если какой-то из параметров не высылается (null) считаем, что он не менялся
					
					-- IdSection
					if ((select IdSection from [dbo].[WO_Dashboards_Widgets] where Id = (select cast([id] as uniqueidentifier) from @tempxmldata)) != @sectionid)
						update [dbo].[WO_Dashboards_Widgets] set [IdSection] = @sectionid
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Name
					if ((select [name] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [Name] = (select [name] from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Type
					if ((select [type] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [Type] = (select case when [type] = '' then null else cast([type] as int) end from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Source
					if ((select [source] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [Source] = (select case when [source] = '' then null else cast([source] as int) end from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					
					
					
					-- Проверка, есть ли изменения в fields. Если нет, то оставляем как есть, иначе ...
					if ((select count(*) from @tempxmldatafields) != 0)
						begin
							--select	'Есть fields'
							-- Перезаписываем все
							
							-- ... удаляем
							delete
								from	[dbo].[WO_Dashboards_Widgets_Fields]
								where	Id in
											(
												select	Id
												from	[dbo].[WO_Dashboards_Widgets_Fields]
												where	IdFields =
																	(
																		select	IdFields
																		from	[dbo].[WO_Dashboards_Widgets]
																		where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
																	)
											)
							-- ... записываем новые
							insert into [dbo].[WO_Dashboards_Widgets_Fields] ([Id], [IdFields], [Name], [Value])
								select	newid() [Id],
										(
											select	IdFields
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) [IdFields],
										txmldf.name [Name],
										case when txmldf.value = '' then null else cast(txmldf.value as int) end [Value]
								from	@tempxmldatafields txmldf
								-- исключая непривязанные id-ки
								where	(
											select	IdFields
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) is not null
										and txmldf.name is not null and txmldf.value is not null					-- update 2013-10-22
						end
						
						
						
					-- Проверка, есть ли изменения в filters. Если нет, то оставляем как есть, иначе ...
					if ((select count(*) from @tempxmldatafilters) != 0)
						begin
							--select	'Есть filters'
							-- Перезаписываем все
							
							-- ... удаляем
							delete
								from	[dbo].[WO_Dashboards_Widgets_Filters]
								where	Id in
											(
												select	Id
												from	[dbo].[WO_Dashboards_Widgets_Filters]
												where	IdFilters =
																	(
																		select	IdFilters
																		from	[dbo].[WO_Dashboards_Widgets]
																		where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
																	)
											)
							-- ... записываем новые
							insert into [dbo].[WO_Dashboards_Widgets_Filters] ([Id], [IdFilters], [Name], [Operation], [Value])
								select	newid() [Id],
										(
											select	IdFilters
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) [IdFilters],
										txmldfl.name [Name],
										case when txmldfl.operation = '' then null else cast(txmldfl.operation as int) end [Operation],
										txmldfl.value [Value]
								from	@tempxmldatafilters txmldfl
								-- исключая непривязанные id-ки
								where	(
											select	IdFilters
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) is not null
										and txmldfl.name is not null and txmldfl.operation is not null					-- update 2013-10-22
						end
					
					
					
					-- NameTable
					if ((select [table_name] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [NameTable] = (select [table_name] from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Query
					if ((select cast([query] as nvarchar(4000)) from @tempxmldata) is not null)
						update w
						set w.[Query] = txd.[query]
						from	[dbo].[WO_Dashboards_Widgets] w, @tempxmldata txd
						where	w.[Id] = cast(txd.[id] as uniqueidentifier)
					-- OrderBy
					if ((select [order_by] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [OrderBy] = (select [order_by] from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- [ShowTopRows]
					if ((select [show_top_rows] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [ShowTopRows] = (select case when [show_top_rows] = '' then null else cast([show_top_rows] as int) end from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					-- Options
					if ((select cast([options] as nvarchar(4000)) from @tempxmldata) is not null)
						update w
						set	w.[Options] = txd.[options]
						from	[dbo].[WO_Dashboards_Widgets] w, @tempxmldata txd
						where	w.[Id] = cast(txd.[id] as uniqueidentifier)
					
					
					
					-- Проверка, есть ли изменения в constructor_fields. Если нет, то оставляем как есть, иначе ...
					if ((select count(*) from @tempxmldataconstructorfields) != 0)
						begin
							--select	'Есть constructor_fields'
							-- Перезаписываем все
							
							-- ... удаляем
							delete
								from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
								where	Id in
											(
												select	Id
												from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields]
												where	IdConstructorFields =
																	(
																		select	IdConstructorFields
																		from	[dbo].[WO_Dashboards_Widgets]
																		where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
																	)
											)
							-- ... записываем новые
							insert into [dbo].[WO_Dashboards_Widgets_Constructor_Fields] ([Id], [IdConstructorFields], [ConstructorValueId], [ConstructorFunctionId], [ConstantValue])
								select	newid() [Id],
										(
											select	IdConstructorFields
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) [IdConstructorFields],
										case when txmldcf.field_id = '' then null else cast(txmldcf.field_id as int) end [ConstructorValueId],
										case when txmldcf.function_id = '' then null else cast(txmldcf.function_id as int) end [ConstructorFunctionId],
										txmldcf.value [ConstantValue]
								from	@tempxmldataconstructorfields txmldcf
								-- исключая непривязанные id-ки
								where	(
											select	IdConstructorFields
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) is not null
										--and txmldcf.function_id is not null or txmldcf.value is not null
						end
						
						
						
					-- Проверка, есть ли изменения в constructor_filter. Если нет, то оставляем как есть, иначе ...
					if ((select count(*) from @tempxmldataconstructorfilter) != 0)
						begin
							--select	'Есть filters'
							-- Перезаписываем все
							
							-- ... удаляем
							delete
								from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
								where	Id in
											(
												select	Id
												from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
												where	IdConstructorFilters =
																	(
																		select	IdConstructorFilters
																		from	[dbo].[WO_Dashboards_Widgets]
																		where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
																	)
											)
							-- ... записываем новые
							insert into [dbo].[WO_Dashboards_Widgets_Constructor_Filters] ([Id], [IdConstructorFilters], [ConstructorFilterId], [ConstructorFilterOperationId], [ConstructorFilterValueId], [ConstructorFilterValue1], [ConstructorFilterValue2])
								select	newid() [Id],
										(
											select	IdConstructorFilters
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) [IdConstructorFilters],
										case when txmldcfl.filter_id = '' then null else cast(txmldcfl.filter_id as int) end [ConstructorFilterId],
										case when txmldcfl.operation = '' then 0 else cast(txmldcfl.operation as int) end [ConstructorFilterOperationId],
										case when txmldcfl.value_id = '' then null else cast(txmldcfl.value_id as uniqueidentifier) end [ConstructorFilterValueId],
										txmldcfl.value_1 [ConstructorFilterValue1],
										txmldcfl.value_2 [ConstructorFilterValue2]
								from	@tempxmldataconstructorfilter txmldcfl
								-- исключая непривязанные id-ки
								where	(
											select	IdConstructorFilters
											from	[dbo].[WO_Dashboards_Widgets]
											where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
										) is not null
										and
										(
											((isnull(txmldcfl.value_id, '') != '') and (isnull(txmldcfl.value_1, '') = '') and (isnull(txmldcfl.value_2, '') = ''))
											or ((select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = txmldcfl.value_id) = 'today' and (isnull(txmldcfl.value_1, '') = '') and (isnull(txmldcfl.value_2, '') = ''))
											or ((isnull(txmldcfl.value_id, '') = '') and (isnull(txmldcfl.value_1, '') != '') and (isnull(txmldcfl.value_2, '') != ''))
											--
											--(((select Filter from [dbo].[WO_Dashboards_Widgets_Designer_Filter] where IdUniqueidentifier = txmldcfl.value_id) = 'dateperiod') and txmldcfl.value_1 is not null and txmldcfl.value_2 is not null)
											--or
											--(((select Filter from [dbo].[WO_Dashboards_Widgets_Designer_Filter] where IdUniqueidentifier = txmldcfl.value_id) = 'timeperiod') and txmldcfl.value_1 is not null and txmldcfl.value_2 is not null)
											--or
											--txmldcfl.value_id is not null
										)
						end
					
					
					
					-- ConstructorAxisId
					if ((select [constructor_axis_id] from @tempxmldata) is not null)
						update [dbo].[WO_Dashboards_Widgets] set [IdConstructorAxis] = (select case when [constructor_axis_id] = '' then null else cast([constructor_axis_id] as int) end from @tempxmldata)
						where	[Id] = (select cast([id] as uniqueidentifier) from @tempxmldata)
					
					
				end
				
				
		end
		
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Table_Fields_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Table_Fields_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Table_Fields_GetList]
	@authorizeduserid	uniqueidentifier,
	@tablename	nvarchar(1000)
as
begin
	
	declare	@en bit
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode = 'DashboardCreate', @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	if ((select @en) = 1)
		begin
			
			select	f.name FieldName
			from	[dbo].[sysobjects] t, [dbo].[syscolumns] f
			where	t.id = f.id
					and t.type in ('U', 'V')
					and t.name = @tablename
			
		end
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widget_Tables_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widget_Tables_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widget_Tables_GetList]
	@authorizeduserid	uniqueidentifier
as
begin
	
	declare	@en bit
	exec dbo.A_UserRoles_CheckPrivilege @privilegecode = 'DashboardCreate', @authorizeduserid = @authorizeduserid, @isenabled = @en OUTPUT
	if ((select @en) = 1)
		begin
			
			select	t.name TableName
			from	[dbo].[sysobjects] t
			where	t.type in ('U', 'V')
			order by t.name asc
			
		end
	else
		begin
			RAISERROR ('Access error', 16, 1)
		end
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widgets_Filters_Operation_Fill]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widgets_Filters_Operation_Fill]
GO

CREATE procedure [dbo].[WO_Dashboards_Widgets_Filters_Operation_Fill]
as
begin
	
	TRUNCATE TABLE [oktell_settings].[dbo].[WO_Dashboards_Widgets_Filters_Operation]
	INSERT INTO [oktell_settings].[dbo].[WO_Dashboards_Widgets_Filters_Operation] (OperationText) VALUES ('=')
	INSERT INTO [oktell_settings].[dbo].[WO_Dashboards_Widgets_Filters_Operation] (OperationText) VALUES ('!=')
	INSERT INTO [oktell_settings].[dbo].[WO_Dashboards_Widgets_Filters_Operation] (OperationText) VALUES ('>')
	INSERT INTO [oktell_settings].[dbo].[WO_Dashboards_Widgets_Filters_Operation] (OperationText) VALUES ('<')
	INSERT INTO [oktell_settings].[dbo].[WO_Dashboards_Widgets_Filters_Operation] (OperationText) VALUES ('like ''%_%''')
	INSERT INTO [oktell_settings].[dbo].[WO_Dashboards_Widgets_Filters_Operation] (OperationText) VALUES ('not like ''%_%''')
	INSERT INTO [oktell_settings].[dbo].[WO_Dashboards_Widgets_Filters_Operation] (OperationText) VALUES ('like ''_%''')
	INSERT INTO [oktell_settings].[dbo].[WO_Dashboards_Widgets_Filters_Operation] (OperationText) VALUES ('like ''%_''')
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widgets_GetConfig]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widgets_GetConfig]
GO

CREATE procedure [dbo].[WO_Dashboards_Widgets_GetConfig]
	@authorizeduserid	uniqueidentifier,
	@widgetid	uniqueidentifier
as
begin
	
	
	-- Дашборд какой у виджета?
	Declare @dashboardid uniqueidentifier
	Select @dashboardid = IdDashboard
	From [dbo].[WO_Dashboards_Sections]
	Where Id = (select IdSection from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	Declare @pcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @projectid,
		@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
		@onlyreturncount = 1,
		@resultcount = @pcount OUTPUT
	
	if ((@pcount = 0) and (@projectid is not null)) 
		RAISERROR ('Access error', 16, 1)
	
	/* Привилегия чтения по дашборду есть?
	*/
	Declare @dcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @dashboardid,
		@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863',
		@onlyreturncount = 1,
		@resultcount = @dcount OUTPUT
	
	if (@dcount= 0) 
		RAISERROR ('Access error', 16, 1)
	
	
	
	select	dw.NameTable TableName,
			dw.Query
	from	[dbo].[WO_Dashboards_Widgets] dw
	where	dw.Id = @widgetid
	
	select	wf.Name WidgetFilterName,
			wf.Operation WidgetFilterOperation,
			wf.Value WidgetFilterValue
	from	[dbo].[WO_Dashboards_Widgets] w 
				inner join [dbo].[WO_Dashboards_Widgets_Filters] wf on w.IdFilters = wf.IdFilters
	where	w.Id = @widgetid
											-- привязать к словарю?
	select	wcf.ConstructorFilterId,
			wcf.ConstructorFilterOperationId,
			wcf.ConstructorFilterValueId,
			wcf.ConstructorFilterValue1,
			wcf.ConstructorFilterValue2
	from	[dbo].[WO_Dashboards_Widgets] w 
				inner join [dbo].[WO_Dashboards_Widgets_Constructor_Filters] wcf on w.IdConstructorFilters = wcf.IdConstructorFilters
	where	w.Id = @widgetid
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widgets_GetData]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widgets_GetData]
GO

CREATE procedure [dbo].[WO_Dashboards_Widgets_GetData]
	@authorizeduserid	uniqueidentifier,
	@widgetid	uniqueidentifier
as
begin
	
	
	-- Дашборд какой у виджета?
	Declare @dashboardid uniqueidentifier
	Select @dashboardid = IdDashboard
	From [dbo].[WO_Dashboards_Sections]
	Where Id = (select IdSection from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	Declare @pcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @projectid,
		@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
		@onlyreturncount = 1,
		@resultcount = @pcount OUTPUT
	
	if ((@pcount = 0) and (@projectid is not null)) 
		RAISERROR ('Access error', 16, 1)
	
	/* Привилегия чтения по дашборду есть?
	*/
	Declare @dcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @dashboardid,
		@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863',
		@onlyreturncount = 1,
		@resultcount = @dcount OUTPUT
	
	if (@dcount = 0) 
		RAISERROR ('Access error', 16, 1)
	
	
	
	-- конструктор
	if ((select w.[Source] from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid) = 0)
		begin
			
			--select	'конструктор'
			
			declare @widgetidn nvarchar(50)
			set @widgetidn = replace(cast(@widgetid as nvarchar(50)), '-', '')
			
			
			
			declare	@droptables nvarchar(4000)
			set	@droptables = ''
			select	@droptables = @droptables + 'drop table ' + name + ' ' from tempdb.sys.tables where name like '##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP%'
			
			--select @droptables
			
			exec(@droptables)
			
			
			
			if ((select count(*) from [dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf where cf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid) and cf.ConstructorValueId = 1) != 0)
				begin
					--select	'есть "Cредняя скорость ответа (ASA)"'
					----------------------------------------------
					
					declare	@sqldesigner1 nvarchar(4000)
					
					declare	@constructorvalues nvarchar(4000)
					set	@constructorvalues = ''
					select	@constructorvalues = @constructorvalues +
						case
							when cv.ConstructorFunctionId = 1 then 'round(avg(lq.LenQueue), 0) [' + cv.Value + ',' + cv.[Function] + '], '
							when cv.ConstructorFunctionId = 2 then 'round(sum(lq.LenQueue), 0) [' + cv.Value + ',' + cv.[Function] + '], '
							when cv.ConstructorFunctionId = 3 then 'round(max(lq.LenQueue), 0) [' + cv.Value + ',' + cv.[Function] + '], '
							when cv.ConstructorFunctionId = 4 then 'round(min(lq.LenQueue), 0) [' + cv.Value + ',' + cv.[Function] + '], '
						end
					from
					(
						select	cf.ConstructorValueId,
								dv.Value,
								cf.ConstructorFunctionId,
								df.[Function],
								df.FunctionText
						from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf
									left join [dbo].[WO_Dashboards_Widgets_Designer_Function] df on cf.ConstructorFunctionId = df.Id
										inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id
						where	cf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
								and cf.ConstructorValueId = 1
					) cv
					
					--select @constructorvalues
					
					declare	@constructorvalues1 nvarchar(4000)
					set	@constructorvalues1 = ''
					select	@constructorvalues1 = @constructorvalues1 +
						case
							when cv.ConstructorFunctionId = 1 then 'lq.[' + cv.Value + ',' + cv.[Function] + '], '
							when cv.ConstructorFunctionId = 2 then 'lq.[' + cv.Value + ',' + cv.[Function] + '], '
							when cv.ConstructorFunctionId = 3 then 'lq.[' + cv.Value + ',' + cv.[Function] + '], '
							when cv.ConstructorFunctionId = 4 then 'lq.[' + cv.Value + ',' + cv.[Function] + '], '
						end
					from
					(
						select	cf.ConstructorValueId,
								dv.Value,
								cf.ConstructorFunctionId,
								df.[Function],
								df.FunctionText
						from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf
									inner join [dbo].[WO_Dashboards_Widgets_Designer_Function] df on cf.ConstructorFunctionId = df.Id
										inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id
						where	cf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
								and cf.ConstructorValueId = 1
					) cv
					
					--select @constructorvalues1
					
					declare	@constructorfilters1 nvarchar(4000)
					set	@constructorfilters1 = 'and '
					select	@constructorfilters1 = @constructorfilters1 +
							case
								when cf.ConstructorFilterId = 1 then
									case
										when (select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 'today' then 'cast(floor(cast(effconn_1.DateTimeStart as float)) as datetime)'
										-- различные варианты значений дат (сегодня)
										else ''
									end
								when cf.ConstructorFilterId = 6 then 'effconn_1.DateTimeStart'
								when cf.ConstructorFilterId = 2 then 'effconn_1.TimeStart'
								when cf.ConstructorFilterId = 3 then 'effconn_1.IdOperator'
								when cf.ConstructorFilterId = 4 then 'effconn_1.IdTask'
								when cf.ConstructorFilterId = 5 then 'effconn_1.IdProject'
							end
							
							+ ' ' +
							
							case
								when cf.ConstructorFilterOperationId = 0 then
									case
										when cf.ConstructorFilterId = 6 then 'between'
										when cf.ConstructorFilterId = 2 then 'between'
										else '='
									end
							end
							
							+ ' ' +
							
							case
								when cf.ConstructorFilterId = 1 then
									case
										when (select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 'today' then 'cast(floor(cast(getdate() as float)) as datetime)'
										-- различные варианты значений дат (сегодня)
										else ''
									end
								when cf.ConstructorFilterId = 6 then 'cast(''' + cf.ConstructorFilterValue1 + ' ' + '00:00:00.000' + ''' as datetime) and cast(''' + cf.ConstructorFilterValue2 + ' ' + '23:59:59.900' + ''' as datetime)'
								when cf.ConstructorFilterId = 2 then 'cast(''' + '1900-01-01' + ' ' + cf.ConstructorFilterValue1 + '.000' + ''' as datetime) and cast(''' + '1900-01-01' + ' ' + cf.ConstructorFilterValue2 + '.000' + ''' as datetime)'
								when cf.ConstructorFilterId = 3 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
								when cf.ConstructorFilterId = 4 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
								when cf.ConstructorFilterId = 5 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
							end
							
							+ ' and '
					from
					(
						select	ConstructorFilterId,
								ConstructorFilterOperationId,
								ConstructorFilterValueId,
								ConstructorFilterValue1,
								ConstructorFilterValue2
						from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
						where	IdConstructorFilters = (select IdConstructorFilters from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
					) cf
					
					--select @constructorfilters1
					
					-- шаблон для подсчета значения
					select	distinct
							@sqldesigner1 = dwdv.ModelQuery
					from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields] dwcf
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dwdv on dwcf.ConstructorValueId = dwdv.Id
					where	dwcf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
							and dwcf.ConstructorValueId = 1
					
					--select @sqldesigner1
					
					-- в зависимости от оси добавляем столбец
					select @sqldesigner1 =
						case w.IdConstructorAxis
							when 1 then replace(@sqldesigner1, '_select_', 'convert(nvarchar(5), cast((floor(cast(effconn_2.TimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 5, cast((floor(cast(effconn_2.TimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime)), 108) ' + da.Axis)
							when 2 then replace(@sqldesigner1, '_select_', 'convert(nvarchar(5), cast((floor(cast(effconn_2.TimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 15, cast((floor(cast(effconn_2.TimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime)), 108) ' + da.Axis)
							when 3 then replace(@sqldesigner1, '_select_', 'convert(nvarchar(5), cast((floor(cast(effconn_2.TimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 30, cast((floor(cast(effconn_2.TimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime)), 108) ' + da.Axis)
							when 4 then replace(@sqldesigner1, '_select_', 'convert(nvarchar(5), cast((floor(cast(effconn_2.TimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(hh, 1, cast((floor(cast(effconn_2.TimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(effconn_2.TimeStart as float))) as datetime)), 108) ' + da.Axis)
							when 5 then replace(@sqldesigner1, '_select_', 'cast(datepart(yy, cast(floor(cast(effconn_2.DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), cast(floor(cast(effconn_2.DateTimeStart as float)) as datetime), 10) ' + da.Axis)
							when 6 then replace(@sqldesigner1, '_select_', 'cast(datepart(yy, cast(floor(cast(effconn_2.DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), cast(floor(cast(effconn_2.DateTimeStart as float) - datepart(dw, effconn_2.DateTimeStart) + 1) as datetime), 10) + '' - '' + cast(datepart(yy, cast(floor(cast(effconn_2.DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), dateadd(dd, 6, cast(floor(cast(effconn_2.DateTimeStart as float) - datepart(dw, effconn_2.DateTimeStart) + 1) as datetime)), 10) ' + da.Axis)
    						when 7 then replace(@sqldesigner1, '_select_', '
      											case datepart(mm, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime))
      												when 1 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-01''
      												when 2 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-02''
      												when 3 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-03''
      												when 4 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-04''
      												when 5 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-05''
      												when 6 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-06''
      												when 7 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-07''
      												when 8 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-08''
      												when 9 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-09''
      												when 10 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-10''
      												when 11 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-11''
      												when 12 then cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-'' + cast(datepart(mm, effconn_2.DateTimeStart) as nvarchar (2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-12''
      											end ' + da.Axis)
    						when 8 then replace(@sqldesigner1, '_select_', 'cast(datepart(yy, cast(cast(datepart(yy, effconn_2.DateTimeStart) as nvarchar (4)) + ''-01-01'' as datetime)) as nvarchar(4)) ' + da.Axis)
    						when 9 then replace(@sqldesigner1, '_select_', 'effconn_1.IdOperator ')
    						when 10 then replace(@sqldesigner1, '_select_', 'effconn_1.IdTask ')
    						when 11 then replace(@sqldesigner1, '_select_', 'effconn_1.IdProject ')
    						when 12 then replace(@sqldesigner1, '_select_', '
    											case datepart(dw, effconn_2.DateTimeStart)
    												when 1 then ''1''
      												when 2 then ''2''
      												when 3 then ''3''
      												when 4 then ''4''
      												when 5 then ''5''
      												when 6 then ''6''
      												when 7 then ''7''
    											end ' + da.Axis)
    					end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
								 inner join [dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 1
					
					--select @sqldesigner1
					
					-- в зависимости от оси добавляем прнивязку
					set @sqldesigner1 =
						case (select IdConstructorAxis from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
							when 9 then replace(@sqldesigner1, '_join_', 'inner join (select IdChain, LenQueue, min(DateTimeStart) minDateTimeStart from [oktell_cc_temp].[dbo].[A_Cube_CC_EffortConnections] where IdOperator not in (''AB000000-0000-0000-0000-000000000000'', ''BF000000-0000-0000-0000-000000000000'', ''00000000-0000-0000-0000-000000000000'') and IdOperator is not null group by IdChain, LenQueue) effconn_3
																			on effconn_1.IdChain = effconn_3.IdChain and effconn_1.LenQueue = effconn_3.LenQueue and effconn_1.DateTimeStart = effconn_3.minDateTimeStart')
							else replace(@sqldesigner1, '_join_', '')
						end +
						case
							when @constructorfilters1 = 'and ' then ''
							else ' ' + (left(@constructorfilters1, len(@constructorfilters1)-len(' and ')))
						end
					
					-- в зависимости от значений вычисляем их
					declare	@sqldesigner11 nvarchar(4000)
					select @sqldesigner11 =
						case w.IdConstructorAxis
							when 9 then 'select lq.' + da.Axis + ', ' + (left(@constructorvalues1, len(@constructorvalues1)-len(', '))) + ' into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP1 from ('
							when 10 then 'select lq.' + da.Axis + ', ' + (left(@constructorvalues1, len(@constructorvalues1)-len(', '))) + ' into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP1 from ('
							when 11 then 'select lq.' + da.Axis + ', ' + (left(@constructorvalues1, len(@constructorvalues1)-len(', '))) + ' into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP1 from ('
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct IdConstructorFields, ConstructorValueId from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields]) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 1
					
					declare	@sqldesigner12 nvarchar(4000)
					select @sqldesigner12 =
						case w.IdConstructorAxis
							when 1 then 'lq.' + da.Axis
							when 2 then 'lq.' + da.Axis
							when 3 then 'lq.' + da.Axis
							when 4 then 'lq.' + da.Axis
							when 5 then 'lq.' + da.Axis
							when 6 then 'lq.' + da.Axis
							when 7 then 'lq.' + da.Axis
							when 8 then 'lq.' + da.Axis
							when 9 then 'lq.IdOperator, oi.Name ' + da.Axis
							when 10 then 'lq.IdTask, t.Name ' + da.Axis
							when 11 then 'lq.IdProject, p.Name ' + da.Axis
							when 12 then 'lq.' + da.Axis
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct IdConstructorFields, ConstructorValueId from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields]) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 1
					
					declare	@sqldesigner13 nvarchar(4000)
					select @sqldesigner13 = (left(@constructorvalues, len(@constructorvalues)-len(', ')))
					
					declare	@sqldesigner14_ nvarchar(4000)
					select @sqldesigner14_ =
						case w.IdConstructorAxis
							when 9 then ''
							when 10 then ''
							when 11 then ''
							else ' into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP1'
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct IdConstructorFields, ConstructorValueId from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields]) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 1
					
					declare	@sqldesigner14 nvarchar(4000)
					select @sqldesigner14 =
						case w.IdConstructorAxis
							when 9 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] oi on lq.IdOperator = oi.Id '
							when 10 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task] t on lq.IdTask = t.Id '
							when 11 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project] p on lq.IdProject = p.Id '
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct IdConstructorFields, ConstructorValueId from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields]) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 1
					
					declare	@sqldesigner15 nvarchar(4000)
					select @sqldesigner15 =
						case w.IdConstructorAxis
							when 1 then 'lq.' + da.Axis
							when 2 then 'lq.' + da.Axis
							when 3 then 'lq.' + da.Axis
							when 4 then 'lq.' + da.Axis
							when 5 then 'lq.' + da.Axis
							when 6 then 'lq.' + da.Axis
							when 7 then 'lq.' + da.Axis
							when 8 then 'lq.' + da.Axis
							when 9 then 'lq.IdOperator, oi.Name'
							when 10 then 'lq.IdTask, t.Name'
							when 11 then 'lq.IdProject, p.Name'
							when 12 then 'lq.' + da.Axis
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct IdConstructorFields, ConstructorValueId from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields]) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 1
					
					declare	@sqldesigner16 nvarchar(4000)
					select @sqldesigner16 =
						case w.IdConstructorAxis
							when 1 then ' order by lq.' + da.Axis + ' asc'
							when 2 then ' order by lq.' + da.Axis + ' asc'
							when 3 then ' order by lq.' + da.Axis + ' asc'
							when 4 then ' order by lq.' + da.Axis + ' asc'
							when 5 then ' order by lq.' + da.Axis + ' asc'
							when 6 then ' order by lq.' + da.Axis + ' asc'
							when 7 then ' order by lq.' + da.Axis + ' asc'
							when 8 then ' order by lq.' + da.Axis + ' asc'
							when 12 then ' order by lq.' + da.Axis + ' asc'
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct IdConstructorFields, ConstructorValueId from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields]) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 1
					
					declare	@sqldesigner17 nvarchar(4000)
					select @sqldesigner17 =
						case w.IdConstructorAxis
							when 9 then ') lq'
							when 10 then ') lq'
							when 11 then ') lq'
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct IdConstructorFields, ConstructorValueId from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields]) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 1
					
					declare	@sqldesigner18 nvarchar(4000)
					select @sqldesigner18 =
						case w.IdConstructorAxis
							when 9 then ' order by lq.' + da.Axis + ' asc'
							when 10 then ' order by lq.' + da.Axis + ' asc'
							when 11 then ' order by lq.' + da.Axis + ' asc'
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct IdConstructorFields, ConstructorValueId from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields]) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 1
					
															
					exec('SET DATEFIRST 1 ' + @sqldesigner11 + 'select ' + @sqldesigner12 + ', ' + @sqldesigner13 + @sqldesigner14_ + ' from (' + @sqldesigner1 + ') lq ' + @sqldesigner14 + 'group by ' + @sqldesigner15 + @sqldesigner16 + @sqldesigner17 + @sqldesigner18)
					
				end
			
			
			if ((select count(*) from [dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf where cf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid) and cf.ConstructorValueId = 2) != 0)
				begin
					--select	'есть "Количество успешных звонков по задачам"'
					-------------------------------------------------------
					
					declare	@sqldesigner2 nvarchar(4000)
					
					declare	@constructorfilters2 nvarchar(4000)
					set	@constructorfilters2 = 'and '
					select	@constructorfilters2 = @constructorfilters2 +
							case
								when cf.ConstructorFilterId = 1 then
									case
										when (select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 'today' then 'cast(floor(cast(DateTimeStart as float)) as datetime)'
										-- различные варианты значений дат (сегодня)
										else ''
									end
								when cf.ConstructorFilterId = 6 then 'DateTimeStart'
								when cf.ConstructorFilterId = 2 then 'TimeStart'
								when cf.ConstructorFilterId = 3 then 'IdOperator'
								when cf.ConstructorFilterId = 4 then 'IdTask'
								when cf.ConstructorFilterId = 5 then 'IdProject'
							end
							
							+ ' ' +
							
							case
								when cf.ConstructorFilterOperationId = 0 then
									case
										when cf.ConstructorFilterId = 6 then 'between'
										when cf.ConstructorFilterId = 2 then 'between'
										else '='
									end
							end
							
							+ ' ' +
							
							case
								when cf.ConstructorFilterId = 1 then
									case
										when (select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 'today' then 'cast(floor(cast(getdate() as float)) as datetime)'
										-- различные варианты значений дат (сегодня)
										else ''
									end
								when cf.ConstructorFilterId = 6 then 'cast(''' + cf.ConstructorFilterValue1 + ' ' + '00:00:00.000' + ''' as datetime) and cast(''' + cf.ConstructorFilterValue2 + ' ' + '23:59:59.900' + ''' as datetime)'
								when cf.ConstructorFilterId = 2 then 'cast(''' + '1900-01-01' + ' ' + cf.ConstructorFilterValue1 + '.000' + ''' as datetime) and cast(''' + '1900-01-01' + ' ' + cf.ConstructorFilterValue2 + '.000' + ''' as datetime)'
								when cf.ConstructorFilterId = 3 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
								when cf.ConstructorFilterId = 4 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
								when cf.ConstructorFilterId = 5 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
							end
							
							+ ' and '
					from
					(
						select	ConstructorFilterId,
								ConstructorFilterOperationId,
								ConstructorFilterValueId,
								ConstructorFilterValue1,
								ConstructorFilterValue2
						from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
						where	IdConstructorFilters = (select IdConstructorFilters from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
					) cf
					
					--select @constructorfilters2
					
					-- шаблон для подсчета значения
					select	distinct
							@sqldesigner2 = dwdv.ModelQuery
					from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields] dwcf
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dwdv on dwcf.ConstructorValueId = dwdv.Id
					where	dwcf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
							and dwcf.ConstructorValueId = 2
					
					--select @sqldesigner2
					
					-- в зависимости от оси добавляем столбец
					select @sqldesigner2 =
						case w.IdConstructorAxis
							when 1 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 5, cast((floor(cast(DateTimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 2 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 15, cast((floor(cast(DateTimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 3 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 30, cast((floor(cast(DateTimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 4 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(hh, 1, cast((floor(cast(DateTimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 5 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'cast(datepart(yy, cast(floor(cast(DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), cast(floor(cast(DateTimeStart as float)) as datetime), 10)')
							when 6 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'cast(datepart(yy, cast(floor(cast(DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), cast(floor(cast(DateTimeStart as float) - datepart(dw, DateTimeStart) + 1) as datetime), 10) + '' - '' + cast(datepart(yy, cast(floor(cast(DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), dateadd(dd, 7, cast(floor(cast(DateTimeStart as float) - datepart(dw, DateTimeStart) + 1) as datetime)), 10)')
							when 7 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + '
												case datepart(mm, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime))
													when 1 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-01''
													when 2 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-02''
													when 3 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-03''
													when 4 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-04''
													when 5 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-05''
													when 6 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-06''
													when 7 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-07''
													when 8 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-08''
													when 9 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-09''
													when 10 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-10''
													when 11 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-11''
													when 12 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-12''
												end')
							when 8 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar (4)) + ''-01-01'' as datetime)) as nvarchar(4))')
							when 9 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'cast(IdOperator as nvarchar(50))')
							when 10 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'cast(IdTask as nvarchar(50))')
							when 11 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + 'cast(IdProject as nvarchar(50))')
							when 12 then replace(@sqldesigner2, '_select_', da.Axis + ' = ' + '
												case datepart(dw, DateTimeStart)
													when 1 then ''1''
													when 2 then ''2''
													when 3 then ''3''
													when 4 then ''4''
													when 5 then ''5''
													when 6 then ''6''
													when 7 then ''7''
												end')
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
								 inner join [dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					--select @sqldesigner2
					
					-- в зависимости от значений вычисляем их
					declare	@sqldesigner21 nvarchar(4000)
					select @sqldesigner21 =
						case w.IdConstructorAxis
							when 1 then 'DateTimeStart datetime'
							when 2 then 'DateTimeStart datetime'
							when 3 then 'DateTimeStart datetime'
							when 4 then 'DateTimeStart datetime'
							when 5 then 'DateTimeStart datetime'
							when 6 then 'DateTimeStart datetime'
							when 7 then 'DateTimeStart datetime'
							when 8 then 'DateTimeStart datetime'
							when 9 then 'IdOperator uniqueidentifier'
							when 10 then 'IdTask uniqueidentifier'
							when 11 then 'IdProject uniqueidentifier'
							when 12 then 'DateTimeStart datetime'
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner22 nvarchar(4000)
					select @sqldesigner22 =
						')
						declare @temp2 table (IdChain uniqueidentifier, Successful int, ' + da.Axis + ' nvarchar(50))
						declare @temp table (IdChain uniqueidentifier,  Successful int, ' + da.Axis + ' nvarchar(50))
						insert into @temp1
							select IdChain, CallResult, '
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner23 nvarchar(4000)
					select @sqldesigner23 =
						case w.IdConstructorAxis
							when 1 then 'DateTimeStart'
							when 2 then 'DateTimeStart'
							when 3 then 'DateTimeStart'
							when 4 then 'DateTimeStart'
							when 5 then 'DateTimeStart'
							when 6 then 'DateTimeStart'
							when 7 then 'DateTimeStart'
							when 8 then 'DateTimeStart'
							when 9 then 'IdOperator'
							when 10 then 'IdTask'
							when 11 then 'IdProject'
							when 12 then 'DateTimeStart'
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner24 nvarchar(4000)
					select @sqldesigner24 =
						case
							when @constructorfilters2 = 'and ' then ''
							else ' ' + (left(@constructorfilters2, len(@constructorfilters2)-len(' and ')))
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner25 nvarchar(4000)
					select @sqldesigner25 =
						'
						insert into @temp
							select IdChain, Successful, ' + da.Axis + ' from @temp2'
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner26 nvarchar(4000)
					select @sqldesigner26 =
						case w.IdConstructorAxis
							when 9 then 'select oc.Name ' + da.Axis + ', oc.[' + cf.Value + ',] into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP2 from('
							when 10 then 'select oc.Name ' + da.Axis + ', oc.[' + cf.Value + ',] into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP2 from('
							when 11 then 'select oc.Name ' + da.Axis + ', oc.[' + cf.Value + ',] into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP2 from('
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner27 nvarchar(4000)
					select @sqldesigner27 =
						'select temp.' + da.Axis + ', count(temp.Successful) ' + '[' + cf.Value + ',]'
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner28 nvarchar(4000)
					select @sqldesigner28 =
						case w.IdConstructorAxis
							when 9 then ', oi.Name from @temp temp '
							when 10 then ', t.Name from @temp temp '
							when 11 then ', p.Name from @temp temp '
							else ' into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP2 from @temp temp '
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner29 nvarchar(4000)
					select @sqldesigner29 =
						case w.IdConstructorAxis
							when 9 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] oi on temp.' + da.Axis + ' = oi.Id '
							when 10 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task] t on temp.' + da.Axis + ' = t.Id '
							when 11 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project] p on temp.' + da.Axis + ' = p.Id '
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner210 nvarchar(4000)
					select @sqldesigner210 =
						case w.IdConstructorAxis
							when 1 then 'temp.' + da.Axis
							when 2 then 'temp.' + da.Axis
							when 3 then 'temp.' + da.Axis
							when 4 then 'temp.' + da.Axis
							when 5 then 'temp.' + da.Axis
							when 6 then 'temp.' + da.Axis
							when 7 then 'temp.' + da.Axis
							when 8 then 'temp.' + da.Axis
							when 9 then 'temp.' + da.Axis + ', oi.Name'
							when 10 then 'temp.' + da.Axis + ', t.Name'
							when 11 then 'temp.' + da.Axis + ', p.Name'
							when 12 then 'temp.' + da.Axis
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner211 nvarchar(4000)
					select @sqldesigner211 =
						case w.IdConstructorAxis
							when 9 then ') oc'
							when 10 then ') oc'
							when 11 then ') oc'
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					declare	@sqldesigner212 nvarchar(4000)
					select @sqldesigner212 =
						case w.IdConstructorAxis
							when 1 then ' order by temp.' + da.Axis + ' asc'
							when 2 then ' order by temp.' + da.Axis + ' asc'
							when 3 then ' order by temp.' + da.Axis + ' asc'
							when 4 then ' order by temp.' + da.Axis + ' asc'
							when 5 then ' order by temp.' + da.Axis + ' asc'
							when 6 then ' order by temp.' + da.Axis + ' asc'
							when 7 then ' order by temp.' + da.Axis + ' asc'
							when 8 then ' order by temp.' + da.Axis + ' asc'
							when 9 then ' order by oc.Name asc'
							when 10 then ' order by oc.Name asc'
							when 11 then ' order by oc.Name asc'
							when 12 then ' order by temp.' + da.Axis + ' asc'
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 2
					
					
					exec('SET DATEFIRST 1 declare @temp1 table (IdChain uniqueidentifier, CallResult int, ' + @sqldesigner21 + @sqldesigner22 + @sqldesigner23 + ' from [oktell_cc_temp].[dbo].[A_Cube_CC_EffortConnections] where IdChain is not null ' + @sqldesigner24 + ' ' + @sqldesigner2 + ' ' + @sqldesigner25 + ' ' + @sqldesigner26 + @sqldesigner27 + @sqldesigner28 + @sqldesigner29 + ' group by ' + @sqldesigner210 + @sqldesigner211 + @sqldesigner212)
					
				end
			
			
			if ((select count(*) from [dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf where cf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid) and cf.ConstructorValueId = 3) != 0)
				begin
					--select	'есть "Количество неуспешных звонков по задачам"'
					---------------------------------------------------------
					
					declare	@sqldesigner3 nvarchar(4000)
					
					declare	@constructorfilters3 nvarchar(4000)
					set	@constructorfilters3 = 'and '
					select	@constructorfilters3 = @constructorfilters3 +
							case
								when cf.ConstructorFilterId = 1 then
									case
										when (select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 'today' then 'cast(floor(cast(DateTimeStart as float)) as datetime)'
										-- различные варианты значений дат (сегодня)
										else ''
									end
								when cf.ConstructorFilterId = 6 then 'DateTimeStart'
								when cf.ConstructorFilterId = 2 then 'TimeStart'
								when cf.ConstructorFilterId = 3 then 'IdOperator'
								when cf.ConstructorFilterId = 4 then 'IdTask'
								when cf.ConstructorFilterId = 5 then 'IdProject'
							end
							
							+ ' ' +
							
							case
								when cf.ConstructorFilterOperationId = 0 then
									case
										when cf.ConstructorFilterId = 6 then 'between'
										when cf.ConstructorFilterId = 2 then 'between'
										else '='
									end
							end
							
							+ ' ' +
							
							case
								when cf.ConstructorFilterId = 1 then
									case
										when (select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 'today' then 'cast(floor(cast(getdate() as float)) as datetime)'
										-- различные варианты значений дат (сегодня)
										else ''
									end
								when cf.ConstructorFilterId = 6 then 'cast(''' + cf.ConstructorFilterValue1 + ' ' + '00:00:00.000' + ''' as datetime) and cast(''' + cf.ConstructorFilterValue2 + ' ' + '23:59:59.900' + ''' as datetime)'
								when cf.ConstructorFilterId = 2 then 'cast(''' + '1900-01-01' + ' ' + cf.ConstructorFilterValue1 + '.000' + ''' as datetime) and cast(''' + '1900-01-01' + ' ' + cf.ConstructorFilterValue2 + '.000' + ''' as datetime)'
								when cf.ConstructorFilterId = 3 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
								when cf.ConstructorFilterId = 4 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
								when cf.ConstructorFilterId = 5 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
							end
							
							+ ' and '
					from
					(
						select	ConstructorFilterId,
								ConstructorFilterOperationId,
								ConstructorFilterValueId,
								ConstructorFilterValue1,
								ConstructorFilterValue2
						from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
						where	IdConstructorFilters = (select IdConstructorFilters from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
					) cf
					
					--select @constructorfilters3
					
					-- шаблон для подсчета значения
					select	distinct
							@sqldesigner3 = dwdv.ModelQuery
					from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields] dwcf
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dwdv on dwcf.ConstructorValueId = dwdv.Id
					where	dwcf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
							and dwcf.ConstructorValueId = 3
					
					--select @sqldesigner3
					
					-- в зависимости от оси добавляем столбец
					select @sqldesigner3 =
						case w.IdConstructorAxis
							when 1 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 5, cast((floor(cast(DateTimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 2 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 15, cast((floor(cast(DateTimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 3 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 30, cast((floor(cast(DateTimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 4 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(hh, 1, cast((floor(cast(DateTimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 5 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'cast(datepart(yy, cast(floor(cast(DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), cast(floor(cast(DateTimeStart as float)) as datetime), 10)')
							when 6 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'cast(datepart(yy, cast(floor(cast(DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), cast(floor(cast(DateTimeStart as float) - datepart(dw, DateTimeStart) + 1) as datetime), 10) + '' - '' + cast(datepart(yy, cast(floor(cast(DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), dateadd(dd, 7, cast(floor(cast(DateTimeStart as float) - datepart(dw, DateTimeStart) + 1) as datetime)), 10)')
							when 7 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + '
												case datepart(mm, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime))
													when 1 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-01''
													when 2 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-02''
													when 3 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-03''
													when 4 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-04''
													when 5 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-05''
													when 6 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-06''
													when 7 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-07''
													when 8 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-08''
													when 9 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-09''
													when 10 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-10''
													when 11 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-11''
													when 12 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-12''
												end')
							when 8 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar (4)) + ''-01-01'' as datetime)) as nvarchar(4))')
							when 9 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'cast(IdOperator as nvarchar(50))')
							when 10 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'cast(IdTask as nvarchar(50))')
							when 11 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + 'cast(IdProject as nvarchar(50))')
							when 12 then replace(@sqldesigner3, '_select_', da.Axis + ' = ' + '
												case datepart(dw, DateTimeStart)
													when 1 then ''1''
													when 2 then ''2''
													when 3 then ''3''
													when 4 then ''4''
													when 5 then ''5''
													when 6 then ''6''
													when 7 then ''7''
												end')
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
								 inner join [dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					--select @sqldesigner3
					
					-- в зависимости от значений вычисляем их
					declare	@sqldesigner31 nvarchar(4000)
					select @sqldesigner31 =
						case w.IdConstructorAxis
							when 1 then 'DateTimeStart datetime'
							when 2 then 'DateTimeStart datetime'
							when 3 then 'DateTimeStart datetime'
							when 4 then 'DateTimeStart datetime'
							when 5 then 'DateTimeStart datetime'
							when 6 then 'DateTimeStart datetime'
							when 7 then 'DateTimeStart datetime'
							when 8 then 'DateTimeStart datetime'
							when 9 then 'IdOperator uniqueidentifier'
							when 10 then 'IdTask uniqueidentifier'
							when 11 then 'IdProject uniqueidentifier'
							when 12 then 'DateTimeStart datetime'
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner32 nvarchar(4000)
					select @sqldesigner32 =
						')
						declare @temp2 table (IdChain uniqueidentifier, UnSuccessful int, ' + da.Axis + ' nvarchar(50))
						declare @temp table (IdChain uniqueidentifier,  UnSuccessful int, ' + da.Axis + ' nvarchar(50))
						insert into @temp1
							select IdChain, CallResult, '
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner33 nvarchar(4000)
					select @sqldesigner33 =
						case w.IdConstructorAxis
							when 1 then 'DateTimeStart'
							when 2 then 'DateTimeStart'
							when 3 then 'DateTimeStart'
							when 4 then 'DateTimeStart'
							when 5 then 'DateTimeStart'
							when 6 then 'DateTimeStart'
							when 7 then 'DateTimeStart'
							when 8 then 'DateTimeStart'
							when 9 then 'IdOperator'
							when 10 then 'IdTask'
							when 11 then 'IdProject'
							when 12 then 'DateTimeStart'
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner34 nvarchar(4000)
					select @sqldesigner34 =
						case
							when @constructorfilters3 = 'and ' then ''
							else ' ' + (left(@constructorfilters3, len(@constructorfilters3)-len(' and ')))
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner35 nvarchar(4000)
					select @sqldesigner35 =
						'
						insert into @temp
							select IdChain, UnSuccessful, ' + da.Axis + ' from @temp2'
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner36 nvarchar(4000)
					select @sqldesigner36 =
						case w.IdConstructorAxis
							when 9 then 'select oc.Name ' + da.Axis + ', oc.[' + cf.Value + ',] into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP3 from('
							when 10 then 'select oc.Name ' + da.Axis + ', oc.[' + cf.Value + ',] into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP3 from('
							when 11 then 'select oc.Name ' + da.Axis + ', oc.[' + cf.Value + ',] into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP3 from('
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner37 nvarchar(4000)
					select @sqldesigner37 =
						'select temp.' + da.Axis + ', count(temp.UnSuccessful) ' + '[' + cf.Value + ',]'
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner38 nvarchar(4000)
					select @sqldesigner38 =
						case w.IdConstructorAxis
							when 9 then ', oi.Name from @temp temp '
							when 10 then ', t.Name from @temp temp '
							when 11 then ', p.Name from @temp temp '
							else ' into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP3 from @temp temp '
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner39 nvarchar(4000)
					select @sqldesigner39 =
						case w.IdConstructorAxis
							when 9 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] oi on temp.' + da.Axis + ' = oi.Id '
							when 10 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task] t on temp.' + da.Axis + ' = t.Id '
							when 11 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project] p on temp.' + da.Axis + ' = p.Id '
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner310 nvarchar(4000)
					select @sqldesigner310 =
						case w.IdConstructorAxis
							when 1 then 'temp.' + da.Axis
							when 2 then 'temp.' + da.Axis
							when 3 then 'temp.' + da.Axis
							when 4 then 'temp.' + da.Axis
							when 5 then 'temp.' + da.Axis
							when 6 then 'temp.' + da.Axis
							when 7 then 'temp.' + da.Axis
							when 8 then 'temp.' + da.Axis
							when 9 then 'temp.' + da.Axis + ', oi.Name'
							when 10 then 'temp.' + da.Axis + ', t.Name'
							when 11 then 'temp.' + da.Axis + ', p.Name'
							when 12 then 'temp.' + da.Axis
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner311 nvarchar(4000)
					select @sqldesigner311 =
						case w.IdConstructorAxis
							when 9 then ') oc'
							when 10 then ') oc'
							when 11 then ') oc'
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
					
					declare	@sqldesigner312 nvarchar(4000)
					select @sqldesigner312 =
						case w.IdConstructorAxis
							when 1 then ' order by temp.' + da.Axis + ' asc'
							when 2 then ' order by temp.' + da.Axis + ' asc'
							when 3 then ' order by temp.' + da.Axis + ' asc'
							when 4 then ' order by temp.' + da.Axis + ' asc'
							when 5 then ' order by temp.' + da.Axis + ' asc'
							when 6 then ' order by temp.' + da.Axis + ' asc'
							when 7 then ' order by temp.' + da.Axis + ' asc'
							when 8 then ' order by temp.' + da.Axis + ' asc'
							when 9 then ' order by oc.Name asc'
							when 10 then ' order by oc.Name asc'
							when 11 then ' order by oc.Name asc'
							when 12 then ' order by temp.' + da.Axis + ' asc'
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 3
										
					
					exec('SET DATEFIRST 1 declare @temp1 table (IdChain uniqueidentifier, CallResult int, ' + @sqldesigner31 + @sqldesigner32 + @sqldesigner33 + ' from [oktell_cc_temp].[dbo].[A_Cube_CC_EffortConnections] where IdChain is not null ' + @sqldesigner34 + ' ' + @sqldesigner3 + ' ' + @sqldesigner35 + ' ' + @sqldesigner36 + @sqldesigner37 + @sqldesigner38 + @sqldesigner39 + ' group by ' + @sqldesigner310 + @sqldesigner311 + @sqldesigner312)
					
				end
			
			
			if ((select count(*) from [dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf where cf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid) and cf.ConstructorValueId = 4) != 0)
				begin
					--select	'есть "Количество звонков по задачам"'
					----------------------------------------------
					
					declare	@sqldesigner4 nvarchar(4000)
					
					declare	@constructorfilters4 nvarchar(4000)
					set	@constructorfilters4 = 'and '
					select	@constructorfilters4 = @constructorfilters4 +
							case
								when cf.ConstructorFilterId = 1 then
									case
										when (select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 'today' then 'cast(floor(cast(DateTimeStart as float)) as datetime)'
										-- различные варианты значений дат (сегодня)
										else ''
									end
								when cf.ConstructorFilterId = 6 then 'DateTimeStart'
								when cf.ConstructorFilterId = 2 then 'TimeStart'
								when cf.ConstructorFilterId = 3 then 'IdOperator'
								when cf.ConstructorFilterId = 4 then 'IdTask'
								when cf.ConstructorFilterId = 5 then 'IdProject'
							end
							
							+ ' ' +
							
							case
								when cf.ConstructorFilterOperationId = 0 then
									case
										when cf.ConstructorFilterId = 6 then 'between'
										when cf.ConstructorFilterId = 2 then 'between'
										else '='
									end
							end
							
							+ ' ' +
							
							case
								when cf.ConstructorFilterId = 1 then
									case
										when (select Value from [dbo].[WO_Dashboards_Widgets_Designer_Filter_Value] where IdUniqueidentifier = cf.ConstructorFilterValueId) = 'today' then 'cast(floor(cast(getdate() as float)) as datetime)'
										-- различные варианты значений дат (сегодня)
										else ''
									end
								when cf.ConstructorFilterId = 6 then 'cast(''' + cf.ConstructorFilterValue1 + ' ' + '00:00:00.000' + ''' as datetime) and cast(''' + cf.ConstructorFilterValue2 + ' ' + '23:59:59.900' + ''' as datetime)'
								when cf.ConstructorFilterId = 2 then 'cast(''' + '1900-01-01' + ' ' + cf.ConstructorFilterValue1 + '.000' + ''' as datetime) and cast(''' + '1900-01-01' + ' ' + cf.ConstructorFilterValue2 + '.000' + ''' as datetime)'
								when cf.ConstructorFilterId = 3 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
								when cf.ConstructorFilterId = 4 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
								when cf.ConstructorFilterId = 5 then '''' + cast(cf.ConstructorFilterValueId as nvarchar(50)) + ''''
							end
							
							+ ' and '
					from
					(
						select	ConstructorFilterId,
								ConstructorFilterOperationId,
								ConstructorFilterValueId,
								ConstructorFilterValue1,
								ConstructorFilterValue2
						from	[dbo].[WO_Dashboards_Widgets_Constructor_Filters]
						where	IdConstructorFilters = (select IdConstructorFilters from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
					) cf
					
					--select @constructorfilters4
					
					-- шаблон для подсчета значения
					select	distinct
							@sqldesigner4 = dwdv.ModelQuery
					from	[dbo].[WO_Dashboards_Widgets_Constructor_Fields] dwcf
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dwdv on dwcf.ConstructorValueId = dwdv.Id
					where	dwcf.IdConstructorFields = (select IdConstructorFields from [dbo].[WO_Dashboards_Widgets] where Id = @widgetid)
							and dwcf.ConstructorValueId = 4
					
					--select @sqldesigner4
					
					-- в зависимости от оси добавляем столбец
					select @sqldesigner4 =
						case w.IdConstructorAxis
							when 1 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 5, cast((floor(cast(DateTimeStart as float) * 287.999999999999) / 287.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 2 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 15, cast((floor(cast(DateTimeStart as float) * 95.999999999999) / 95.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 3 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(mi, 30, cast((floor(cast(DateTimeStart as float) * 47.999999999999) / 47.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 4 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'convert(nvarchar(5), cast((floor(cast(DateTimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(DateTimeStart as float))) as datetime), 108) + '' - '' + convert(nvarchar(5), dateadd(hh, 1, cast((floor(cast(DateTimeStart as float) * 23.999999999999) / 23.999999999999 - floor(cast(DateTimeStart as float))) as datetime)), 108)')
							when 5 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'cast(datepart(yy, cast(floor(cast(DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), cast(floor(cast(DateTimeStart as float)) as datetime), 10)')
							when 6 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'cast(datepart(yy, cast(floor(cast(DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), cast(floor(cast(DateTimeStart as float) - datepart(dw, DateTimeStart) + 1) as datetime), 10) + '' - '' + cast(datepart(yy, cast(floor(cast(DateTimeStart as float)) as datetime)) as nvarchar(4)) + ''-'' + convert(nvarchar(5), dateadd(dd, 7, cast(floor(cast(DateTimeStart as float) - datepart(dw, DateTimeStart) + 1) as datetime)), 10)')
							when 7 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + '
												case datepart(mm, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime))
													when 1 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-01''
													when 2 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-02''
													when 3 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-03''
													when 4 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-04''
													when 5 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-05''
													when 6 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-06''
													when 7 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-07''
													when 8 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-08''
													when 9 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-09''
													when 10 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-10''
													when 11 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-11''
													when 12 then cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar(4)) + ''-'' + cast(datepart(mm, DateTimeStart) as nvarchar(2)) + ''-01'' as datetime)) as nvarchar(4)) + ''-12''
												end')
							when 8 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'cast(datepart(yy, cast(cast(datepart(yy, DateTimeStart) as nvarchar (4)) + ''-01-01'' as datetime)) as nvarchar(4))')
							when 9 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'cast(IdOperator as nvarchar(50))')
							when 10 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'cast(IdTask as nvarchar(50))')
							when 11 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + 'cast(IdProject as nvarchar(50))')
							when 12 then replace(@sqldesigner4, '_select_', da.Axis + ' = ' + '
												case datepart(dw, DateTimeStart)
													when 1 then ''1''
													when 2 then ''2''
													when 3 then ''3''
													when 4 then ''4''
													when 5 then ''5''
													when 6 then ''6''
													when 7 then ''7''
												end')
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
								 inner join [dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					--select @sqldesigner4
					
					-- в зависимости от значений вычисляем их
					declare	@sqldesigner41 nvarchar(4000)
					select @sqldesigner41 =
						case w.IdConstructorAxis
							when 1 then 'DateTimeStart datetime'
							when 2 then 'DateTimeStart datetime'
							when 3 then 'DateTimeStart datetime'
							when 4 then 'DateTimeStart datetime'
							when 5 then 'DateTimeStart datetime'
							when 6 then 'DateTimeStart datetime'
							when 7 then 'DateTimeStart datetime'
							when 8 then 'DateTimeStart datetime'
							when 9 then 'IdOperator uniqueidentifier'
							when 10 then 'IdTask uniqueidentifier'
							when 11 then 'IdProject uniqueidentifier'
							when 12 then 'DateTimeStart datetime'
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner42 nvarchar(4000)
					select @sqldesigner42 =
						')
						declare @temp2 table (IdChain uniqueidentifier, ' + da.Axis + ' nvarchar(50))
						declare @temp table (IdChain uniqueidentifier, ' + da.Axis + ' nvarchar(50))
						insert into @temp1
							select IdChain, CallResult, '
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner43 nvarchar(4000)
					select @sqldesigner43 =
						case w.IdConstructorAxis
							when 1 then 'DateTimeStart'
							when 2 then 'DateTimeStart'
							when 3 then 'DateTimeStart'
							when 4 then 'DateTimeStart'
							when 5 then 'DateTimeStart'
							when 6 then 'DateTimeStart'
							when 7 then 'DateTimeStart'
							when 8 then 'DateTimeStart'
							when 9 then 'IdOperator'
							when 10 then 'IdTask'
							when 11 then 'IdProject'
							when 12 then 'DateTimeStart'
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner44 nvarchar(4000)
					select @sqldesigner44 =
						case
							when @constructorfilters4 = 'and ' then ''
							else ' ' + (left(@constructorfilters4, len(@constructorfilters4)-len(' and ')))
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner45 nvarchar(4000)
					select @sqldesigner45 =
						'
						insert into @temp
							select IdChain, ' + da.Axis + ' from @temp2'
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner46 nvarchar(4000)
					select @sqldesigner46 =
						case w.IdConstructorAxis
							when 9 then 'select oc.Name ' + da.Axis + ', oc.[' + cf.Value + ',] into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP4 from('
							when 10 then 'select oc.Name ' + da.Axis + ', oc.[' + cf.Value + ',] into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP4 from('
							when 11 then 'select oc.Name ' + da.Axis + ', oc.[' + cf.Value + ',] into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP4 from('
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner47 nvarchar(4000)
					select @sqldesigner47 =
						'select temp.' + da.Axis + ', count(*) ' + '[' + cf.Value + ',]'
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner48 nvarchar(4000)
					select @sqldesigner48 =
						case w.IdConstructorAxis
							when 9 then ', oi.Name from @temp temp '
							when 10 then ', t.Name from @temp temp '
							when 11 then ', p.Name from @temp temp '
							else ' into ##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP4 from @temp temp '
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner49 nvarchar(4000)
					select @sqldesigner49 =
						case w.IdConstructorAxis
							when 9 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] oi on temp.' + da.Axis + ' = oi.Id '
							when 10 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task] t on temp.' + da.Axis + ' = t.Id '
							when 11 then 'inner join [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project] p on temp.' + da.Axis + ' = p.Id '
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner410 nvarchar(4000)
					select @sqldesigner410 =
						case w.IdConstructorAxis
							when 1 then 'temp.' + da.Axis
							when 2 then 'temp.' + da.Axis
							when 3 then 'temp.' + da.Axis
							when 4 then 'temp.' + da.Axis
							when 5 then 'temp.' + da.Axis
							when 6 then 'temp.' + da.Axis
							when 7 then 'temp.' + da.Axis
							when 8 then 'temp.' + da.Axis
							when 9 then 'temp.' + da.Axis + ', oi.Name'
							when 10 then 'temp.' + da.Axis + ', t.Name'
							when 11 then 'temp.' + da.Axis + ', p.Name'
							when 12 then 'temp.' + da.Axis
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner411 nvarchar(4000)
					select @sqldesigner411 =
						case w.IdConstructorAxis
							when 9 then ') oc'
							when 10 then ') oc'
							when 11 then ') oc'
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					declare	@sqldesigner412 nvarchar(4000)
					select @sqldesigner412 =
						case w.IdConstructorAxis
							when 1 then ' order by temp.' + da.Axis + ' asc'
							when 2 then ' order by temp.' + da.Axis + ' asc'
							when 3 then ' order by temp.' + da.Axis + ' asc'
							when 4 then ' order by temp.' + da.Axis + ' asc'
							when 5 then ' order by temp.' + da.Axis + ' asc'
							when 6 then ' order by temp.' + da.Axis + ' asc'
							when 7 then ' order by temp.' + da.Axis + ' asc'
							when 8 then ' order by temp.' + da.Axis + ' asc'
							when 9 then ' order by oc.Name asc'
							when 10 then ' order by oc.Name asc'
							when 11 then ' order by oc.Name asc'
							when 12 then ' order by temp.' + da.Axis + ' asc'
							else ''
						end
					from	[dbo].[WO_Dashboards_Widgets] w
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
									inner join (select distinct cf.IdConstructorFields, cf.ConstructorValueId, dv.Value from [oktell_settings].[dbo].[WO_Dashboards_Widgets_Constructor_Fields] cf inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] dv on cf.ConstructorValueId = dv.Id) cf on w.IdConstructorFields = cf.IdConstructorFields
					where	w.Id = @widgetid
							and cf.ConstructorValueId = 4
					
					
					exec('SET DATEFIRST 1 declare @temp1 table (IdChain uniqueidentifier, CallResult int, ' + @sqldesigner41 + @sqldesigner42 + @sqldesigner43 + ' from [oktell_cc_temp].[dbo].[A_Cube_CC_EffortConnections] where IdChain is not null ' + @sqldesigner44 + ' ' + @sqldesigner4 + ' ' + @sqldesigner45 + ' ' + @sqldesigner46 + @sqldesigner47 + @sqldesigner48 + @sqldesigner49 + ' group by ' + @sqldesigner410 + @sqldesigner411 + @sqldesigner412)
					
				end
			
			
			
			
			
			declare @axis nvarchar(50)
			select	@axis = da.Axis
			from	[dbo].[WO_Dashboards_Widgets] w
						inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] da on w.IdConstructorAxis = da.Id
			where	w.Id = @widgetid
			
			--select @axis
			
			
			
			declare @axistable nvarchar(4000)
			set @axistable = ''
			select	@axistable = @axistable +
					'select ' + @axis + ' from ' + name + ' union '
			from	tempdb.sys.tables where name like '##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP%'
			
			select	@axistable = 'declare @axis table (' + @axis + ' nvarchar(1000) collate database_default) insert into @axis ' + left(@axistable, len(@axistable)-len(' union ')) + ' order by ' + @axis + ' asc'
			
			
			
			declare	@selecttables nvarchar(4000)
			set	@selecttables = ''
			select	@selecttables = @selecttables +
					+ ' left join ' +
					name
					+ ' ' +
					right(name, len(name)-62)
					+ ' on a.' + @axis + ' = ' + right(name, len(name)-62) + '.' + @axis
			from	tempdb.sys.tables where name like '##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP%'
			
			--select @selecttables
			
			
			
			declare @valuestable nvarchar(4000)
			set @valuestable = ''
			select	@valuestable = @valuestable +
					+ 'isnull(' +
					right(t.name, len(t.name)-62) + '.[' + tc.name + '], 0) ' + '[' + tc.name + ']' + ', '
			from	tempdb.sys.tables t inner join tempdb.sys.syscolumns tc on t.object_id = tc.id
			where	t.name like '##DashboardsWidgetsConstructor' + @widgetidn + 'TEMP%'
					and tc.name != @axis
			
			select	@valuestable = left(@valuestable, len(@valuestable)-len(', '))
			
			--select @valuestable
			
			
			exec(@axistable + ' select a.' + @axis + ', ' + @valuestable + ' from @axis a' + @selecttables)
			
			
			
		end
	
	
	-- таблица бд
	if ((select w.[Source] from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid) = 1)
		begin
			--select	'таблица бд'
			
			declare	@sqltable nvarchar(4000)
			
			
			declare	@fields nvarchar(4000)
			set	@fields = ''
			select	@fields = @fields + wf.Name + ', '
			from	[dbo].[WO_Dashboards_Widgets] w 
						inner join [dbo].[WO_Dashboards_Widgets_Fields] wf on w.IdFields = wf.IdFields
			where	w.Id = @widgetid
			
			declare	@filters nvarchar(4000)
			set	@filters = ''
			select	@filters = @filters + w.OperationTextValue + ' and ' from
			(
				select	wf.Name +
						' ' +
						case
							when wfo.Id in (4,5,6,7) then replace(wfo.OperationText, '_', wf.Value)
							else wfo.OperationText + ' ''' + wf.Value + ''''
						end OperationTextValue
				from	[dbo].[WO_Dashboards_Widgets] w
							inner join [dbo].[WO_Dashboards_Widgets_Filters] wf on w.IdFilters = wf.IdFilters
							inner join [dbo].[WO_Dashboards_Widgets_Filters_Operation] wfo on wf.Operation = wfo.Id
				where	w.Id = @widgetid
			) w
			
			
			
			set	@sqltable =
				case
					when ( select count(*) 
						   from [dbo].[WO_Dashboards_Widgets] w 
								inner join [dbo].[WO_Dashboards_Widgets_Fields] wf on w.IdFields = wf.IdFields 
						   where w.Id = @widgetid and wf.Name is not null and wf.Value is not null) = 0 
						then 'select null'
					else
						'select ' +
							case 
								when (( select count(*) 
										from [dbo].[WO_Dashboards_Widgets] w 
										where (w.Id = @widgetid) 
											and (w.Type = 4) 
											and (w.ShowTopRows is not null)) != 0 ) 
									then 'top(' + cast((select w.ShowTopRows from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid) as varchar) + ') '
								else ''
							end +
						(left(@fields, len(@fields)-len(', '))) +
						' from ' +
						(
							select	w.NameTable
							from	[dbo].[WO_Dashboards_Widgets] w
							where	w.Id = @widgetid
						) +
						case
							when (( select count(*) 
								   from [dbo].[WO_Dashboards_Widgets] w 
										inner join [dbo].[WO_Dashboards_Widgets_Filters] wf on w.IdFilters = wf.IdFilters 
								   where (w.Id = @widgetid) and (wf.Name is not null) and (wf.Operation is not null)) = 0 ) 
								then ''
							else
								' where ' + (left(@filters, len(@filters)-len(' and ')))
						end +
						case
							when (( select count(*) 
									from [dbo].[WO_Dashboards_Widgets] w 
									where (w.Id = @widgetid) and (w.Type = 4) and (w.OrderBy is not null) and (w.OrderBy != '')) != 0 ) 
								then ' order by ' + (select w.OrderBy from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid)
							else ''
						end
				end
			
						
			exec(@sqltable)
		end
	
	
	-- sql-запрос
	if ((select w.[Source] from [dbo].[WO_Dashboards_Widgets] w where w.Id = @widgetid) = 2)
		begin
			
			--select	'sql-запрос'
			
			declare	@sqlquery1 nvarchar(4000)
			declare	@sqlquery2 nvarchar(4000)
			declare	@sqlquery3 nvarchar(4000)
			declare	@sqlquery4 nvarchar(4000)
			declare	@sqlquery5 nvarchar(4000)
			declare	@sqlquery6 nvarchar(4000)
			declare	@sqlquery7 nvarchar(4000)
			declare	@sqlquery8 nvarchar(4000)
			declare	@sqlquery9 nvarchar(4000)
			declare	@sqlquery10 nvarchar(4000)
			
			
			set	@sqlquery1 =
			(
				select	substring(w.Query, 1, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			set	@sqlquery2 =
			(
				select	substring(w.Query, 4001, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			set	@sqlquery3 =
			(
				select	substring(w.Query, 8001, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			set	@sqlquery4 =
			(
				select	substring(w.Query, 12001, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			set	@sqlquery5 =
			(
				select	substring(w.Query, 16001, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			set	@sqlquery6 =
			(
				select	substring(w.Query, 20001, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			set	@sqlquery7 =
			(
				select	substring(w.Query, 24001, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			set	@sqlquery8 =
			(
				select	substring(w.Query, 28001, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			set	@sqlquery9 =
			(
				select	substring(w.Query, 32001, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			set	@sqlquery10 =
			(
				select	substring(w.Query, 36001, 4000)
				from	[dbo].[WO_Dashboards_Widgets] w
				where	w.Id = @widgetid
			)
			
			
			exec(@sqlquery1 + @sqlquery2 + @sqlquery3 + @sqlquery4 + @sqlquery5 + @sqlquery6 + @sqlquery7 + @sqlquery8 + @sqlquery9 + @sqlquery10)
			
		end
	
end
GO



IF  EXISTS (SELECT * FROM sysobjects WHERE id = object_id ('[dbo].[WO_Dashboards_Widgets_GetList]'))
DROP PROCEDURE [dbo].[WO_Dashboards_Widgets_GetList]
GO

CREATE procedure [dbo].[WO_Dashboards_Widgets_GetList]
	@authorizeduserid	uniqueidentifier,
	@sectionid	uniqueidentifier
as
begin
	
	-- lang
	if exists (select * from tempdb..sysobjects where id = object_id('tempdb..#langtext'))
		drop table #langtext	
	create table #langtext (langkey nvarchar(200) collate database_default, langvalue nvarchar(4000) collate database_default)
	
	insert into #langtext
		exec dbo.[WO_Dashboards_GetLang]
		
	
	-- Дашборд какой у виджета?
	Declare @dashboardid uniqueidentifier
	Select @dashboardid = IdDashboard
	From [dbo].[WO_Dashboards_Sections]
	Where Id = @sectionid
	
	-- Проект какой у дашборда?
	Declare @projectid uniqueidentifier	
	Select @projectid = d.IdProject
	From [dbo].[WO_Dashboards_Dashboards] d
	Where d.Id = @dashboardid
			
	/* Привилегия чтения по проекту есть?
	*/
	Declare @pcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @projectid,
		@idprivilege = 'F876E650-7972-475A-AE96-BCCF831AF818',
		@onlyreturncount = 1,
		@resultcount = @pcount OUTPUT
			
	if ((@pcount = 0) and (@projectid is not null)) 
		RAISERROR ('Access error', 16, 1)
	
	/* Привилегия чтения по дашборду есть?
	*/
	Declare @dcount int
	exec A_Policy_SubjectPrivileges_GetForUserByObject
		@iduser = @authorizeduserid,
		@idobject = @dashboardid,
		@idprivilege = '95176304-7E76-46FE-A473-0DF01F29B863',
		@onlyreturncount = 1,
		@resultcount = @dcount OUTPUT
		
	if (@dcount = 0) 
		RAISERROR ('Access error', 16, 1)
	
	
	
	
	select	w.Id,
			w.Name,
			w.[Type],
			w.[Source],
			w.Options
	from	[dbo].[WO_Dashboards_Widgets] w
	where	w.IdSection = @sectionid
	
	
	declare @widgets table
		(
			WidgetId uniqueidentifier,
			WidgetFieldCode nvarchar(100) collate database_default,
			WidgetFunctionCode nvarchar(50) collate database_default,
			WidgetFunctionValue nvarchar(50) collate database_default,
			WidgetFieldName nvarchar(250) collate database_default,
			WidgetFieldValue int
		)
	
	if ((select count(*) from [dbo].[WO_Dashboards_Widgets] where IdSection = @sectionid and [Source] in (1,2)) != 0)
		begin
			insert into @widgets (WidgetId, WidgetFieldCode, WidgetFunctionCode, WidgetFunctionValue, WidgetFieldName, WidgetFieldValue)
				select	w.Id WidgetId,
						null WidgetFieldCode,
						null WidgetFunctionCode,
						null WidgetFunctionValue,
						wf.Name WidgetFieldName,
						wf.Value WidgetFieldValue
				from	[dbo].[WO_Dashboards_Widgets] w 
							inner join [dbo].[WO_Dashboards_Widgets_Fields] wf on w.IdFields = wf.IdFields
				where	w.IdSection = @sectionid
						and w.[Source] in (1,2)
		end
	
	if ((select count(*) from [dbo].[WO_Dashboards_Widgets] where IdSection = @sectionid and [Source] in (0)) != 0)
		begin
			insert into @widgets (WidgetId, WidgetFieldCode, WidgetFunctionCode, WidgetFunctionValue, WidgetFieldName, WidgetFieldValue)
				select	w.Id WidgetId,
						wdv.Value WidgetFieldCode,
						wdf.[Function] WidgetFunctionCode,
						wcf.ConstantValue WidgetFunctionValue,
						lt1.langvalue WidgetFieldName,
						1 WidgetFieldValue
				from	[dbo].[WO_Dashboards_Widgets] w 
							inner join [dbo].[WO_Dashboards_Widgets_Constructor_Fields] wcf on w.IdConstructorFields = wcf.IdConstructorFields
								inner join [dbo].[WO_Dashboards_Widgets_Designer_Value] wdv on wdv.Id = wcf.ConstructorValueId
								inner join #langtext lt1 on wdv.Value = lt1.langkey
									left join [dbo].[WO_Dashboards_Widgets_Designer_Function] wdf on wcf.ConstructorFunctionId = wdf.Id
				where	w.IdSection = @sectionid
						and w.[Source] in (0)
				
					union all
				
				select	w.Id WidgetId,
						wda.Axis WidgetFieldCode,
						null WidgetFunctionCode,
						null WidgetFunctionValue,
						lt.langvalue WidgetFieldName,
						0 WidgetFieldValue
				from	[dbo].[WO_Dashboards_Widgets] w 
							inner join [dbo].[WO_Dashboards_Widgets_Designer_Axis] wda on w.IdConstructorAxis = wda.Id
								inner join #langtext lt on wda.Axis = lt.langkey
				where	w.IdSection = @sectionid
						and w.[Source] in (0)
		end
	
	
	select	WidgetId,
			WidgetFieldCode,
			WidgetFunctionCode,
			WidgetFunctionValue,
			WidgetFieldName,
			WidgetFieldValue
	from	@widgets
	
end
GO