﻿

If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_RegistrarSession' ))
  If not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_RegistrarSession' 
								and TABLE_CATALOG like 'oktell'
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME in ('ServerId') )
	Alter Table [dbo].[A_RegistrarSession]
	Add
		[ServerId] nvarchar(200) default NULL
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Registrar_GetData' ))
	Drop procedure dbo.A_Registrar_GetData
GO

Create procedure [dbo].[A_Registrar_GetData]
	@serverid nvarchar(200) = NULL
as
begin
	-- Сначала лишние отфильтруем
	exec dbo.A_Registrar_FilterSessions

	-- Потом оставшиеся вернем
	Select CallId, Account, RecvAddr, IfaceAddr, OverNAT, VIA, CSeq, Nonce, NC, StartTimeX,
		DisplayName, URI, Type, Expires
	From A_RegistrarSession s
		Inner join A_RegistrarContact c on c.SessionId = s.Id
	Where (c.LocalTimeExpire > GetDate())
		and ((@serverid is NULL) or (@serverid = '') or (@serverid = ServerId))
	Order by Account, CallId
end
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Registrar_UpdateSession' ))
	Drop procedure dbo.A_Registrar_UpdateSession
GO

Create procedure [dbo].[A_Registrar_UpdateSession]
	@sessionid int = -1,
	@callid nvarchar(200),
	@account nvarchar(100),
	@recvaddr nvarchar(50),
	@ifaceaddr nvarchar(50),
	@overnat nvarchar(10),
	@via nvarchar(200),
	@cseq nvarchar(10) = '0',
	@nonce nvarchar(100),
	@nc nvarchar(50),
	@starttimex nvarchar(50),
	@serverid nvarchar(200) = NULL
as
begin

	Declare @dtnow Datetime
	Set @dtnow = GetDate()
	
	if ( @sessionid <= 0 ) begin

		Select @sessionid = Id
		From A_RegistrarSession
		Where (CallId = @callid)
			and (Account = @account)
			and ((@serverid is NULL) or (@serverid = '') or (@serverid = ServerId))

		if ( @sessionid is NULL )
			Set @sessionid = -1
	end

	if ( @sessionid > 0 ) begin

	    Update A_RegistrarSession
	    Set RecvAddr = @recvaddr,
			IfaceAddr = @ifaceaddr,
			OverNat = @overnat,
			VIA = @via,
			CSeq = @cseq,
			Nonce = @nonce,
			NC = @nc,
			StartTimeX = @starttimex,
			LocalTimeUpdate = @dtnow,
			ServerId = @serverid	
	    Where (Id = @sessionid)
			and ((@serverid is NULL) or (@serverid = '') or (@serverid = ServerId))

		if ( @@rowcount > 0 ) begin
			Select cast ( @sessionid as int )
			return
		end
	
	end


	Insert into A_RegistrarSession ( CallId, Account, RecvAddr, IfaceAddr, OverNAT, VIA, CSeq, Nonce, NC, StartTimeX, LocalTimeCreate, LocalTimeUpdate, ServerId )
	Values ( @callid, @account, @recvaddr, @ifaceaddr, @overnat, @via, @cseq, @nonce, @nc, @starttimex, @dtnow, @dtnow, @serverid )

	Select cast ( @@identity as int )
	
end
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Registrar_DropSession' ))
	Drop procedure dbo.A_Registrar_DropSession
GO

Create procedure [dbo].[A_Registrar_DropSession]
	@sessionid int = -1,
	@account nvarchar(100) = '',
	@callid nvarchar(200) = '',
	@serverid nvarchar(200) = NULL	
as
begin

	-- Удаляет сессию и контактные данные по указанной связке аккаунта, callid
	
	Declare @id int		
	Set @id = @sessionid

	-- Если сессия явно не указана - по CallId найдем
	if ( @id <= 0 ) begin	
	    Select @id = Id 
	    From A_RegistrarSession
	    Where (CallId like @callid)
			and (Account like @account)
			and ((@serverid is NULL) or (@serverid = '') or (@serverid = ServerId))
	end

	-- Сессию чистим
	Delete 
	From A_RegistrarSession
	Where Id = @id
	
	-- Контакты дочищаем по отсутствию сессий
	Delete
	From A_RegistrarContact
	Where not exists ( Select Id From A_RegistrarSession s Where s.Id = A_RegistrarContact.SessionId )

end
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_Registrar_DropAccount' ))
	Drop procedure dbo.A_Registrar_DropAccount
GO

Create procedure [dbo].[A_Registrar_DropAccount]
	@account nvarchar(100) = '',
	@serverid nvarchar(200) = NULL
as
begin
	
	-- Удаляет все сессии и контакты, соответствующие указанному аккаунту
	
	-- Сессии по аккаунту
	Delete 
	From A_RegistrarSession
	Where (Account = @account)
		and ((@serverid is NULL) or (@serverid = '') or (@serverid = ServerId))
	
	-- Контакты дочищаем по отсутствию сессий
	Delete
	From A_RegistrarContact
	Where not exists ( Select Id From A_RegistrarSession s Where s.Id = A_RegistrarContact.SessionId )

end
GO




If exists ( Select * From sysobjects where id = object_id ( 'dbo.A_StreamRegistration' ))
  If not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_StreamRegistration' 
								and TABLE_CATALOG like 'oktell'
								and TABLE_SCHEMA like 'dbo'
								and COLUMN_NAME in ('ServerId') )
	Alter Table [dbo].[A_StreamRegistration]
	Add
		[ServerId] nvarchar(200) default NULL
GO


if exists ( select * from sysobjects where id = object_id ( 'dbo.A_StreamRegistration_Clear' ))
	Drop procedure dbo.A_StreamRegistration_Clear
GO


Create procedure [dbo].[A_StreamRegistration_Clear]
	@serverid nvarchar(200) = NULL
as
Begin
	
	--Truncate table dbo.A_StreamRegistration
	Delete 
	From dbo.A_StreamRegistration
	Where (@serverid is NULL) or (@serverid = '') or (@serverid = ServerId)

End
GO



if exists ( select * from sysobjects where id = object_id ( 'dbo.A_StreamRegistration_Save' ))
	Drop procedure dbo.A_StreamRegistration_Save
GO


Create procedure [dbo].[A_StreamRegistration_Save]
	@streamid uniqueidentifier,
	@isphone bit,
	@isregistered bit,
	@dtnow datetime = null,
	@serverid nvarchar(200) = null
as
Begin
	
	if ( @dtnow is null )
		Set @dtnow = GetDate()
	
	Update dbo.A_StreamRegistration
	Set IsRegistered = @isregistered,
		IsPhone = @isphone,
		OperationDT = @dtnow,
		ServerId = @serverid
	Where (StreamId = @streamid)
		And ((@serverid is NULL) or (@serverid = '') or (@serverid = ServerId))

	if ( @@rowcount = 0 ) begin

		Insert into dbo.A_StreamRegistration ( StreamId, IsPhone, IsRegistered, OperationDT, ServerId )
			Values ( @streamid, @isphone, @isregistered, @dtnow, @serverid )		
	end

End
GO