﻿
-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_User_UpdatePass' ))
	Drop procedure dbo.A_Admin_UserGroup_User_UpdatePass
GO

Create procedure [dbo].[A_Admin_UserGroup_User_UpdatePass]
	@userid uniqueidentifier,
	@pass nvarchar(100),
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
				
	Update dbo.A_Users
	Set [Password] = @pass
	Where [Id] = @userid
	
end
GO


-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_User_Delete' ))
	Drop procedure dbo.A_Admin_UserGroup_User_Delete
GO

Create procedure [dbo].[A_Admin_UserGroup_User_Delete]
	@id uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	Delete 
	From A_UserControls
	Where UserA = @id or UserB = @id

	Delete 
	From A_UserControlLines
	Where UserA = @id

	Delete
	From A_UserOutcomingLines
	Where IdUser = @id
		
	Delete 
	From A_Users
	Where Id = @id	
	
	
	-- Заказ на очистку таблиц политики доступа
	exec A_Policy_SetFilter
	
	-- Очистка всех хвостов и зависимостей
	exec A_DB_RemoveTails
end
GO


-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_Group_Save' ))
	Drop procedure dbo.A_Admin_UserGroup_Group_Save
GO

Create procedure [dbo].[A_Admin_UserGroup_Group_Save]
	@id uniqueidentifier,
	@name nvarchar ( 200 ),
	@info nvarchar ( 1000 ),
	@parentgroupid	uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	Update A_Groups
	Set 	
		Name = @name,
		Info = @info,
		ParentId = @parentgroupid	
	Where 
		Id = @id
	
	if ( @@rowcount = 0 ) begin
		
		-- Добавление нового отдела
		Insert Into A_Groups ( Id, Name, Info, ParentId )
		Values ( @id, @name, @info, @parentgroupid )
		
		-- Заказ на пересчет кэша политики доступа
		exec A_Policy_SubjectGroupCache_SetUpdate
		
	end
end
GO

-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserGroup_Group_Delete' ))
	Drop procedure dbo.A_Admin_UserGroup_Group_Delete
GO

Create procedure [dbo].[A_Admin_UserGroup_Group_Delete]
	@id uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	Delete 
	From A_Groups 
	Where Id = @id

	while ( @@rowcount > 0 ) begin
		Delete 
		From A_Groups 
		Where ParentId != '00000000-0000-0000-0000-0000000000000000'
			And not ParentId in ( Select Id From A_Groups ) 
	end

	Delete 
	From A_Users
	Where ParentGroupId != '00000000-0000-0000-0000-0000000000000000' 
		And not ParentGroupId in ( Select Id From A_Groups )
	
	
	-- Заказ на очистку таблиц политики доступа
	exec A_Policy_SetFilter
	
	-- Очистка всех хвостов и зависимостей
	exec A_DB_RemoveTails
	
end
GO


-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_Rule_Save' ))
	Drop procedure dbo.A_Admin_Rule_Save
GO

Create Procedure [dbo].[A_Admin_Rule_Save]
	@id uniqueidentifier,
	@name nvarchar ( 100 ), 
	@priority int,
	@type int = 0,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	Delete 
	From A_RuleRecords 
	Where RuleId = @id


	Update A_Rules 
	Set Name = @name, 
		Type = @type,
		Priority = @priority
	Where ID = @id


	if ( @@rowcount = 0 ) begin
		Insert 
		Into A_Rules ( ID, Name, Type, Priority ) 
		Values ( @id, @name, @type, @priority )
	end

end
GO

-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_Rule_Records_Save' ))
	Drop procedure dbo.A_Admin_Rule_Records_Save
GO

Create Procedure [dbo].[A_Admin_Rule_Records_Save]
	@id uniqueidentifier, 
	@starttime int, 
	@finishtime int, 
	@caption nvarchar ( 100 ), 
	@addrtype int, 
	@reactid uniqueidentifier, 
	@ruleid uniqueidentifier,
	@account nvarchar ( 100 ) = '',
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	if ( @FinishTime < 0 ) 
	  	return

	Update A_RuleRecords
	Set
		InnerAddressType = @addrtype,
		StartTime = @starttime,
		FinishTime = @finishtime,
		RuleID = @ruleid,
		ReactID = @reactid,
		Account = @account
	Where Id = @id

	if ( @@rowcount = 0 )
	begin	

		Insert 
		Into A_RuleRecords ( ID, InnerAddressType, StartTime, FinishTime, RuleID, ReactID, Account )
		Values ( @id, @addrtype, @starttime, @finishtime, @ruleid, @reactid, @account )

	end
end
GO



-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_Number_Save' ))
	Drop procedure dbo.A_Admin_Number_Save
GO

Create Procedure [dbo].[A_Admin_Number_Save]
	@id uniqueidentifier,
	@prefix nvarchar ( 20 ),
	@visible bit,
	@description nvarchar ( 100 ),
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
	
	Delete 
	From dbo.A_NumberPlanAction
	Where NumId = @id


	Update dbo.A_NumberPlan
	Set [Id] = @id,	
		[Prefix] = @prefix,
		[Visible] = @visible,
		[Description] = @description
	Where [Id] = @id

	if ( @@rowcount = 0 ) begin
	
		Insert 
		Into A_NumberPlan 
			( [Id], [Prefix], [Visible], [Description] ) 
		Values 
			( @id, @prefix, @visible, @description )
	end
end
GO


-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_Number_Action_Save' ))
	Drop procedure dbo.A_Admin_Number_Action_Save
GO

Create Procedure [dbo].[A_Admin_Number_Action_Save]
	@id uniqueidentifier, 
	@address nvarchar ( 100 ), 
	@priority int, 
	@reactid uniqueidentifier, 
	@specialtype int, 
	@type int, 
	@npid uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	Update A_NumberPlanAction 
	Set numid = @npid, 
		numtype = @type, 
		extratype = @specialtype, 
		extraid = @reactid, 
		extrastr = @address, 
		priority = @priority 
	Where id = @id
	

	if ( @@rowcount = 0 ) begin		
	
		Insert Into A_NumberPlanAction ( id, numid, numtype, extratype, extraid, extrastr, priority, allowshare ) 
		Values ( @id, @npid, @type, @specialtype, @reactid, @address, @priority, 1 )
	end
end
GO


-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_Number_Delete' ))
	Drop procedure dbo.A_Admin_Number_Delete
GO

Create Procedure [dbo].[A_Admin_Number_Delete]
	@id uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	Delete
	From A_NumberPlan
	Where Id = @id

	Delete
	From A_NumberPlanAction
	Where NumId = @id


	exec A_Admin_Rule_SiftByNumbers
end
GO

-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_Line_Ext_Set' ))
	Drop procedure dbo.A_Admin_Line_Ext_Set
GO

Create Procedure [dbo].[A_Admin_Line_Ext_Set]
	@id uniqueidentifier,
	@extnumber nvarchar ( 200 ),
	@canautodial bit,
	@canipout bit,
	@canmobile bit,
	@cansimpleout bit,
	@caption nvarchar ( 200 ),
	@dt int,
	@enabled bit,
	@prefix nvarchar ( 200 ),
	@connectedpbx bit,
	@type int = 1,
	@linecode nvarchar ( 200 ) = '',
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	Update dbo.A_ServerExtLines 
	Set [ExtNumber] = @extnumber,
		[CanAutoDial] = @canautodial,
		[CanIp] = @canipout,
		[CanMobile] = @canmobile,
		[CanSimpleOut] = @cansimpleout,
		[Caption] = @caption,
		[DialType] = @dt,
		[Enabled] = @enabled,
		[Prefix] = @prefix,
		[ConnectedPBX] = @connectedpbx,
		[Type] = @type,
		[LineCode] = @linecode
	Where [Id] = @id
		
end
GO


-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_Line_Int_Set' ))
	Drop procedure dbo.A_Admin_Line_Int_Set
GO

Create Procedure [dbo].[A_Admin_Line_Int_Set]
	@id uniqueidentifier,
	@citystraight bit,
	@caption nvarchar ( 200 ),
	@enabled bit,
	@parentgroupid uniqueidentifier,
	@linecode nvarchar ( 200 ) = '',
	@authorizeduserid uniqueidentifier = NULL
as
begin
-- Задает общие свойства внутренним линиям

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid		
	

	Update dbo.A_ServerIntLines 
	Set [CityStraight] = @citystraight,
		[Caption] = @caption,
		[Enabled] = @enabled,
		[ParentGroupId] = @parentgroupid,
		[LineCode] = @linecode
	Where [Id] = @id

	Update dbo.A_ServerUsbLines 
	Set [CityStraight] = @citystraight,
		[Caption] = @caption,
		[Enabled] = @enabled,
		[ParentGroupId]	= @parentgroupid,
		[LineCode] = @linecode
	Where [Id] = @id
		
	Update dbo.A_ServerSipLines 
	Set [CityStraight] = @citystraight,
		[Caption] = @caption,
		[Enabled] = @enabled,
		[ParentGroupId]	= @parentgroupid,
		[LineCode] = @linecode
	Where [Id] = @id

	Update dbo.A_ServerVirtualLines 
	Set [CityStraight] = @citystraight,
		[Caption] = @caption,
		[Enabled] = @enabled,
		[ParentGroupId]	= @parentgroupid,
		[LineCode] = @linecode
	Where [Id] = @id
end
GO


-------------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Admin_UserPhoto_Save' ))
	Drop procedure dbo.A_Admin_UserPhoto_Save
GO

Create procedure [dbo].[A_Admin_UserPhoto_Save]
	@user uniqueidentifier,
	@photo image = null,
	@filename nvarchar(200) = null,
	@modifieduser uniqueidentifier,
	@dtnow datetime = null,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid		
		
		
	if ( @dtnow is null )
	   Set @dtnow = GetDate()

	Update A_UserPhoto
	Set [photo] = @photo,
		[filename] = @filename,
		[modifieduser] = @modifieduser,
		[modifieddt] = @dtnow
	Where id = @user

	if ( @@rowcount = 0 ) 
	   Insert into A_UserPhoto ( [id], [photo], [filename], [modifieduser], [modifieddt] )
	   Values ( @user, @photo, @filename, @modifieduser, @dtnow )	
end
GO