﻿Use oktell
GO

-------------------------------------------------------------------------------------------------
-- Процедура переливания данных из Tmp_A_Cube_CC_EffortConnections в A_Cube_CC_EffortConnections_Update_From_TmpCube 
IF OBJECT_ID('dbo.A_Cube_CC_EffortConnections_Update_From_TmpCube') IS NOT NULL
    DROP PROCEDURE dbo.A_Cube_CC_EffortConnections_Update_From_TmpCube
GO

Create procedure dbo.A_Cube_CC_EffortConnections_Update_From_TmpCube
as
begin

    Declare @last_id int, @curr_id int, @last_delete_dt datetime
    Select @last_id = curr_id,
           @last_delete_dt = last_delete_dt
    From [oktell_cc_temp]..tmp_cube_track
       
    -- Записи в [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections Держим только 1 день
    -- В полночь по серверному времени, произведется копирование очередной партии недостающих и чистка всех записей (в транзакции жесткой)
    If (DateDiff(dd, @last_delete_dt, GetDate()) > 0) begin
		
		-- В тразакции с удалением
        Begin Tran UpdAndTruncate
        Set Transaction Isolation Level Serializable
            
            Select @curr_id = max(id) 
            From [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections 
            
            If (@curr_id > @last_id)
				Insert into [oktell_cc_temp]..A_Cube_CC_EffortConnections 
					Select  IdProject,IdTask, IsOutput, IdEffort, IdOperator, IdInList, AbonentNumber, CallResult, CallResultInfo, DateTimeStart, DateStart, TimeStart, 
						DateTimeStop, IdConn, IdChain, IdAUser, IdBUser, IdExtLine, LenTime, LenQueue, IsBetweenOp, OpponentInfo, IsHandled, IsSuccess, 
						IsRecorded, IdRecDir, ALineNum, BLineNum, UserResult  
					From [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections
					Where id BETWEEN @last_id AND @curr_id
            
            -- Можно было бы делать Delete с условием и без транзакции, но тогда чаще
            Truncate table [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections
        
        Commit Tran UpdAndTruncate
        
        Update [oktell_cc_temp]..tmp_cube_track 
        Set last_delete_dt = GetDate(),
			last_id = @last_id, 
			curr_id = @curr_id        
    
    end else begin
    
		-- Простое копирование
        Select @curr_id = Max(id) 
        From [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections with (NOLOCK)
        
        If (@curr_id > @last_id)
			Insert [oktell_cc_temp]..A_Cube_CC_EffortConnections 
				Select IdProject, IdTask, IsOutput, IdEffort, IdOperator, IdInList, AbonentNumber, CallResult, CallResultInfo, DateTimeStart, DateStart, TimeStart, 
					DateTimeStop, IdConn, IdChain, IdAUser, IdBUser, IdExtLine, LenTime, LenQueue, IsBetweenOp, OpponentInfo, IsHandled, IsSuccess,
					IsRecorded, IdRecDir, ALineNum, BLineNum, UserResult  
				From [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections with (NOLOCK)
				Where id BETWEEN @last_id AND @curr_id

        Update [oktell_cc_temp]..tmp_cube_track 
        Set last_id = @last_id, 
			curr_id = @curr_id
    
    end
    
end
GO


----------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Timer_MinuteProc' ))
	Drop procedure dbo.A_Timer_MinuteProc
GO	

Create Procedure [dbo].[A_Timer_MinuteProc]
	@dtnow datetime = null
as
begin
	if ( @dtnow is null )	
		set @dtnow = GetDate()
	--------------------------------------------------------------------------------------------
	-- Обеспечивается однократность исполнения (при нескольких серверах к одной базе) 
	--------------------------------------------------------------------------------------------
   	Begin tran DbProcActive

		Declare @dtactive datetime
		Select @dtactive = valuedatetime from A_Settings where name like 'DB_procactive'
		if ( not @dtactive is null )
 		  if ( DateDiff ( mi, @dtactive, @dtnow ) < 10 ) begin
		    Rollback tran DbProcActive
		    return
		  end

		Update A_Settings Set valuedatetime = @dtnow where name like 'DB_procactive'
		if ( @@rowcount = 0 )
		  Insert into A_Settings (name, valuedatetime) Values ( 'DB_procactive', @dtnow )

   	Commit tran DbProcActive
	--------------------------------------------------------------------------------------------
	--------------------------------------------------------------------------------------------

	-- Очистка неактуальных регистрационных сессий
	exec dbo.A_Registrar_FilterSessions
    
    -- Копируем данные из времянки в Cube_CC_EffortConnections
    exec dbo.A_Cube_CC_EffortConnections_Update_From_TmpCube
    
	--Время по которое производить расчет намеренно назад на 30 секунд отводим
	-- чтобы исключить возможность попадания в интервалы, когда отложенные сохранения в БД идут
	-- и когда одна таблица уже заполнена, а другая еще нет данными до текущего момента.
	-- ну и еще случаи частичного заполнения строки, когда данные вставляются в момент выборки
	Declare @tilldt DateTime
	Set @tilldt = DateAdd ( ss, -30, @dtnow )

	exec A_CallCenter_Temp_UpdateOperatorStates @tilldt
	
	-- Очистка неактуальных пользовательских сессий
	exec dbo.A_SessionStorage_FilterSessions @dtnow
	
	
	-- Обновление кэша политики прав доступа к объектам
	exec dbo.A_Policy_SubjectGroupCache_CheckUpdate
	
	-- Очистка пустых ссылок в таблицах политики прав доступа к объектам
	exec dbo.A_Policy_CheckFilter
	
	
	-- Пользовательская процедура, которой можно пользоваться и изменять
	exec dbo.A_Timer_UserDefinedProc
	
	--------------------------------------------------------------------------------------------
	-- Обеспечивается однократность исполнения (при нескольких серверах к одной базе) 
	--------------------------------------------------------------------------------------------
	Delete From A_Settings where name like 'DB_procactive'
	--------------------------------------------------------------------------------------------
	--------------------------------------------------------------------------------------------
end
GO




------------------------------------------------------------------------------------
if exists ( select * from sysobjects where id = object_id ( 'dbo.A_CallCenter_Temp_UpdateAfterCallResult_Old' ))
	Drop procedure dbo.A_CallCenter_Temp_UpdateAfterCallResult_Old
GO

Create procedure [dbo].[A_CallCenter_Temp_UpdateAfterCallResult_Old]
	@idchain uniqueidentifier,
	@ideffort uniqueidentifier,
	@idinlist int,
	@idtask uniqueidentifier,
	@number nvarchar ( 20 ),
	@callresult int,
	@callresultinfo nvarchar ( 100 ),
	@lenqueue float,
	@timestart datetime,
	@timeexit datetime = NULL,--16.09.2010 выход из задачи, а то неявными способами определяет границу конца и ошибается на входящей IVR задаче без выхода.
	@idextline uniqueidentifier,
	@userresult int = 0,
	@idoperator uniqueidentifier = '00000000-0000-0000-0000-000000000000',
	@isattached bit = 0,
	@dtnow datetime = null
	
as 
begin	
	
/*	----------------------------------------
	Declare @idchain uniqueidentifier
	Declare @ideffort uniqueidentifier
	Declare @idinlist int
	Declare @idtask uniqueidentifier
	Declare @number nvarchar ( 20 )
	Declare @callresult int
	Declare @callresultinfo nvarchar ( 100 )
	Declare @lenqueue int
	Declare @timestart datetime
	Declare @idextline uniqueidentifier
	Select @idchain = 'FD024817-A629-49E6-B550-19EC14787AC8', @ideffort = '1C3468C3-A6FE-4B36-9393-B813CFFB2815', 
		@idinlist = 63, @idtask = 'D48D5F57-6B93-4529-A205-A6B5063320A4', 
		@number = 'Номер не определен', @callresult = 5, @callresultinfo = '', @lenqueue = 15, @timestart = '26.02.2007 12:04:57',
		@idextline = '00000000-0000-0000-0000-000000000000'
	-----------------------------------------
*/


	if ( @dtnow is null )
		set @dtnow = GetDate()

	-- выясняем направление и проект
	Declare @idproject uniqueidentifier
	Declare @isoutput bit
	Declare @idlist uniqueidentifier

	Select 	@idproject = IdProject,
		@isoutput = IsOutputTask,
		@idlist = ListId
	From A_TaskManager_Tasks 
	Where Id = @idtask

	-------------------------------------------------------------
	---- КУБ КОММУТАЦИЙ И ПОПЫТОК -------------------------------
	-------------------------------------------------------------

	if exists ( select * from tempdb..sysobjects where id = object_id ( 'tempdb..#temp' ))
		Drop table #temp

	-- создаем времянку
	--Declare @temp table 
	Create table #temp 
	( 
		[IdProject] uniqueidentifier,
		[IdTask] uniqueidentifier,
		[IsOutput] bit,
		[IdEffort] uniqueidentifier,
		[IdOperator] uniqueidentifier,
		[IdInList] int,
		[AbonentNumber] nvarchar ( 20 ) collate database_default,
		[CallResult] int,
		[CallResultInfo] nvarchar ( 100 ) collate database_default,
		[DateTimeStart] DateTime,
		[DateStart] DateTime,
		[TimeStart] DateTime,
		[DateTimeStop] DateTime,
		[IdConn] uniqueidentifier,
		[IdChain] uniqueidentifier,
		[IdAUser] uniqueidentifier,
		[IdBUser] uniqueidentifier,
		[IdExtLine] uniqueidentifier,
		[LenTime] float, 
		[LenQueue] float,
		[IsBetweenOp] bit,
		[OpponentInfo] nvarchar ( 100 ) collate database_default,
		[IsHandled] bit,
		[IsSuccess] bit,
		[IsRecorded] bit,
		[IdRecDir] int,
		[ALineNum] nvarchar ( 12 ) collate database_default,
		[BLineNum] nvarchar ( 12 ) collate database_default,
		[UserResult] int default 0
	)

 	Create Index [#xx] ON #temp ([IdAUser], [IdBUser])
	

	Declare @idempty uniqueidentifier
	Set @idempty = '00000000-0000-0000-0000-000000000000'

	Declare @idop uniqueidentifier
	if ( @idoperator != @idempty )
	    Set @idop = @idoperator


	--Alex 101118
	Declare @idch uniqueidentifier
	set @idch = @idchain

	if ( not @callresult in (6, 7, 20, 13) ) -- оператора в пустышку заносим в случае отклонения и/или пропуска вызова оператором -- или 13 :)
	    Select @idop = NULL, @idch= NULL	--Alex 101118

	-- вставляем коммутации во времянку (0, 1)
	Insert Into #temp ( IdProject, IdTask, IsOutput, IdEffort, IdInList, AbonentNumber, CallResult, 
				CallResultInfo, DateTimeStart, DateStart, TimeStart, DateTimeStop, 
				IdConn, IdChain, IdAUser, IdBUser, IdExtLine, LenTime, LenQueue, 
				IsRecorded, IdRecDir, ALineNum, BLineNum, UserResult ) --ARecFile, BRecFile )
	Select @idproject, @idtask, @isoutput, @ideffort, @idinlist, @number, @callresult, @callresultinfo, 
		TimeStart, 
		Cast ( Floor ( Cast ( TimeStart as Float ) ) as DateTime ),
		Cast ( Cast ( TimeStart as Float ) - FLOOR ( Cast ( IsNull ( TimeStart, @dtnow ) as Float ) ) as DateTime ),
		IsNull ( TimeStop, @dtnow ),		
		Id, IdChain, AUserId, BUserId, 
		@idextline,
		Cast ( ( IsNull ( TimeStop, @dtnow ) - TimeAnswer ) as float ) * 86400 as LenTime, @lenqueue,
		IsRecorded, IdRecDir, ALineNum, BLineNum, --ARecFile, BRecFile
		@userresult
	FROM A_Stat_Connections_1x1 With ( INDEX ( idx_StatConnections_IdChain ), NOLOCK )
	Where IdChain = @idchain
	
	-- если эта попытка без коммутаций ?
	If ( @@rowcount = 0 )
	   begin

		Declare @dtendeff DateTime
		if ( @timeexit is null )
		    Set @dtendeff = @dtnow
		else
		    Set @dtendeff = @timeexit

		Insert Into #temp 
		( 	IdProject, IdTask, IsOutput, IdEffort, 
			IdOperator, 
			IdInList, AbonentNumber, CallResult, CallResultInfo, 
			DateTimeStart, 
			DateStart, 
			TimeStart, 
			DateTimeStop,
			IdChain, 
			--IdConn, IdAUser, IdBUser, 
			IdExtLine, LenTime, LenQueue, 
			--IsBetweenOp, 
			IsHandled, IsSuccess,
			--IsRecorded, IdRecDir, ARecFile, BRecFile,
			UserResult
		)
		Values 
		( 	@idproject, @idtask, @isoutput, @ideffort, 
			@idop, 
			@idinlist, @number, @callresult, @callresultinfo, 
			@timestart,
			Cast ( Floor ( Cast ( @timestart as float ) ) as DateTime ), 
			Cast ( ( Cast ( @timestart as float ) - Floor ( Cast ( @timestart as float ) ) ) as DateTime ), 
			@dtendeff,
			@idchain,
			--@idempty, @idempty, @idempty, 
			@idextline, 
			Cast ( @dtnow - @timestart as float ) * 86400,
			@lenqueue, 
			--0, 
			0, 0,
			--0, 0, '', '',
			@userresult
		)
		--if ( @idoperator != '00000000-0000-0000-0000-000000000000' )
		--    Update #temp
		--    Set IdOperator = @idoperator
	   end
	else
	   begin
		-- удаляем записи по времени не пересекающиеся с попыткой
		if ( @isattached = 1 ) begin
			-- Присоединение к звонку происходит посередине. Поэтому малость изменений
			Delete from #temp
			Where DateTimeStop <= @timestart
			-- Но IVR тоже тем не менее удаляем по старому
			Delete from #temp
			Where DateTimeStart < @timestart
				And (( IdAUser = 'AB000000-0000-0000-0000-000000000000' ) or ( IdBUser = 'AB000000-0000-0000-0000-000000000000' ))
		end else begin
			-- По прежнему
			Delete from #temp
			Where DateTimeStop <= @timestart
		end
		

		Declare @timeendeff datetime
		Select @timeendeff = min ( DateTimeStart ) From #temp

		if ( @timeendeff is null )
		    Set @timeendeff = @timeexit

		if ( @timeendeff < @timestart )
			Set @timestart = @timeendeff

		-- Пустышку добавим для общности, потом исключится если не нужна
		Insert Into #temp 
		( 	IdProject, IdTask, IsOutput, IdEffort, 
			IdOperator, 
			IdInList, AbonentNumber, CallResult, CallResultInfo, DateTimeStart, DateStart, TimeStart, DateTimeStop,
			IdChain,	--Alex 101118
			--IdConn, IdAUser, IdBUser, 
			IdExtLine, LenTime, LenQueue, 
			--IsBetweenOp, 
			IsHandled, IsSuccess,
			--IsRecorded, IdRecDir, ARecFile, BRecFile 
			UserResult
			)
		Values 
		( 	@idproject, @idtask, @isoutput, @ideffort, 
			@idop, 
			@idinlist, @number, @callresult, @callresultinfo, @timestart,
			Cast ( Floor ( Cast ( @timestart as float ) ) as DateTime ), 
			Cast ( ( Cast ( @timestart as float ) - Floor ( Cast ( @timestart as float ) ) ) as DateTime ), @timeendeff,
			@idch,   --@idchain,  --Alex 101118
			--@idempty, @idempty, @idempty, 
			@idextline, 
			Cast ( @timeendeff - @timestart as float ) * 86400,
			@lenqueue, 
			--0, 
			0, 0,
			--0, 0, '', '',
			@userresult
		)
		
		-- обновляем оператора и инфу о коммутации (3)---------
/*
		Declare @users table ( Id uniqueidentifier primary key )
		Insert into @users
			Select Id From A_Users

		Insert into @users Values ( 'AB000000-0000-0000-0000-000000000000' )


		Update t
		Set 	IdOperator = case when ( @isoutput > 0 ) then IdAUser else IdBUser end, 
			IsBetweenOp = 0
		From #temp t, @users u1, @users u2
		Where t.IdAUser = u1.Id or t.IDBUser = u2.Id

		Update t
		Set 	IdOperator = IdAUser, 
			IsBetweenOp = 1
		From #temp t, @users u1, @users u2
		Where t.IdAUser = u1.Id and t.IdBUser = u2.Id

*/

		Declare @ivrid uniqueidentifier
		Set @ivrid = 'AB000000-0000-0000-0000-000000000000'

		Update t
		Set 	IdOperator = case when ( @isoutput > 0 ) then IdAUser else IdBUser end, 
			IsBetweenOp = 0
		From #temp t
		Where exists ( select id from A_Users where t.IdAUser = id or t.IdBUser = id )
			or ( t.IdAuser = @ivrid )
			or ( t.IdBuser = @ivrid )

		Update t
		Set 	IdOperator = IdAUser, 
			IsBetweenOp = 1
		From #temp t
		Where ( exists ( select id from A_Users where t.IdAUser = id ) or ( t.IdAuser = @ivrid ))
			and ( exists ( select id from A_Users where t.IdBUser = id ) or ( t.IdBuser = @ivrid ))

--

		-- вызов обработан? (4)--------------------------------
		Declare @cnt int
		Select @cnt = Count ( * ) FROM #temp t 
		Where DateTimeStart >= @timestart
		
		Update t
		Set IsHandled = case when ( @cnt > 1 ) then 1 else 0 end --одна попытка всегда есть
		From #temp t

		-- успешно ? (5)---------------------------------------
		Update t
		Set IsSuccess = case When ( @callresult = 5 ) then 1 else 0 end
		From #temp t
	end
	
	-- информация по абоненту (6)----------------
	Declare @opinfo nvarchar ( 255 )
	Select @opinfo = ''
	
	Update t
	Set OpponentInfo = @opinfo
	From #temp t

	-- выдираем те, которые относятся к куску цепочки, принадлежащему данной задаче (необходимо при переключениях из задачи в задачу)
	if exists ( Select * from tempdb..sysobjects Where id = object_id ( 'tempdb..#textract' ) )
		drop table #textract
	
	Select * into #textract 
	From #temp 
	Order by datetimestart

/*
	if ( @isoutput = 0 ) -- вырезаем свой кусок если задача входящая и флаг соответствующий передан
	begin
		Declare @dt1 datetime, @dt2 datetime
		--своя начальная
		Select @dt1 = Max ( DateTimeStart ) From #textract Where DateTimeStart <= @timestart and (( IdOperator is null ) or ( IdOperator = 'AB000000-0000-0000-0000-000000000000' ))
		--несвоя начальная
		Select @dt2 = Min ( DateTimeStart ) From #textract Where DateTimeStart > @timestart and (( IdOperator is null ) or ( IdOperator = 'AB000000-0000-0000-0000-000000000000' ))

		if ( not @dt2 is null )--28.03.2008 for special callresults
			Delete from #textract Where DateTimeStart < @dt1 or DateTimeStart >= @dt2 or LenTime is NULL
	end
*/
	if ( @timeexit is not NULL ) begin
	
		Delete From #textract 
		Where DateTimeStart > @timeexit	
			or LenTime is NULL	
	end

	-- поля которые могли остаться нуллами замещаем на дефолтовые значения
	Update #textract
	Set LenTime = 0 
	Where LenTime is null

	Update #textract
	Set DateTimeStop = DateTimeStart 
	Where DateTimeStop is null

	-- обновляем LastConnected (чтоб не по кубу менеджер задач шнырил)
	if ( @idinlist >= 0 )
	begin
		Declare @lc_maxdt DateTime

		Select @lc_maxdt = Max ( DateTimeStart ) 
		From #textract
		Where IdOperator != '00000000-0000-0000-0000-000000000000' 
			and IdOperator != 'AB000000-0000-0000-0000-000000000000'
			and IsHandled = 1

		if ( @@rowcount > 0 )
		begin
			Declare @lc_idoperator uniqueidentifier

			Select @lc_idoperator = IdOperator
			From #textract
			Where IdOperator != '00000000-0000-0000-0000-000000000000' 
				and IdOperator != 'AB000000-0000-0000-0000-000000000000'
				and IsHandled = 1
				and DateTimeStart = @lc_maxdt

			exec A_CallCenter_OperatorLastConnected_Set @lc_maxdt, @idproject, @idtask, @idlist, @idinlist, @number, @lc_idoperator
		end
	end
	

	-- обновляем куб EffortConnections ----------------------------
	Insert Into [oktell_cc_temp].[dbo].[A_Cube_CC_EffortConnections] --With ( TabLock )
	Select * FROM #textract

	--Информация по задачам в таблицу категорий A_Cube_CC_Cat_Task
	if not exists ( Select Id From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task] Where Id = @idtask )
		Insert Into [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task]
		Select Id, Name,Type, Isoutputtask From A_TaskManager_Tasks Where Id = @idtask	

	--Информация по проектам в таблицу категорий A_Cube_CC_Cat_Project
	if not exists ( Select Id From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project] Where Id = @idproject )
		Insert Into [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project]
		Select Id, Name From A_TaskManager_Projects Where Id = @idproject
	
	-------------------------------------------------------------
	---- КУБ СОСТОЯНИЙ ОПЕРАТОРОВ -------------------------------
	-------------------------------------------------------------
	-- обновляется по таймеру из DBProvider

end
GO


-----------------------------------------------------------------------------------------
if exists ( select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_Temp_UpdateAfterCallResult' ))
	Drop procedure dbo.A_CallCenter_Temp_UpdateAfterCallResult
GO

Create procedure [dbo].[A_CallCenter_Temp_UpdateAfterCallResult]
	@idchain uniqueidentifier,
	@ideffort uniqueidentifier,
	@idinlist int,
	@idtask uniqueidentifier,
	@number nvarchar ( 20 ),
	@callresult int,
	@callresultinfo nvarchar ( 100 ),
	@lenqueue float,
	@timestart datetime,
	@timeexit datetime = NULL,--16.09.2010 выход из задачи, а то неявными способами определяет границу конца и ошибается на входящей IVR задаче без выхода.
	@idextline uniqueidentifier,
	@userresult int = 0,
	@idoperator uniqueidentifier = '00000000-0000-0000-0000-000000000000',
	@isattached bit = 0,
	@dtnow datetime = null
	
as 
begin	
	
/*	----------------------------------------
	Declare @idchain uniqueidentifier
	Declare @ideffort uniqueidentifier
	Declare @idinlist int
	Declare @idtask uniqueidentifier
	Declare @number nvarchar ( 20 )
	Declare @callresult int
	Declare @callresultinfo nvarchar ( 100 )
	Declare @lenqueue int
	Declare @timestart datetime
	Declare @idextline uniqueidentifier
	Select @idchain = 'FD024817-A629-49E6-B550-19EC14787AC8', @ideffort = '1C3468C3-A6FE-4B36-9393-B813CFFB2815', 
		@idinlist = 63, @idtask = 'D48D5F57-6B93-4529-A205-A6B5063320A4', 
		@number = 'Номер не определен', @callresult = 5, @callresultinfo = '', @lenqueue = 15, @timestart = '26.02.2007 12:04:57',
		@idextline = '00000000-0000-0000-0000-000000000000'
	-----------------------------------------
*/
	if ( @dtnow is null )
		set @dtnow = GetDate()

	-- выясняем направление и проект
	Declare @idproject uniqueidentifier,
            @isoutput bit,
            @idlist uniqueidentifier

	Select 	@idproject = IdProject,
		@isoutput = IsOutputTask,
		@idlist = ListId
	From A_TaskManager_Tasks 
	Where Id = @idtask

	-------------------------------------------------------------
	---- КУБ КОММУТАЦИЙ И ПОПЫТОК -------------------------------
	-------------------------------------------------------------

	if object_id ( 'tempdb..#temp' ) IS NOT NULL
		Drop table #temp

	-- создаем времянку
	--Declare @temp table 
	Create table #temp 
	( 
		[IdProject] uniqueidentifier,
		[IdTask] uniqueidentifier,
		[IsOutput] bit,
		[IdEffort] uniqueidentifier,
		[IdOperator] uniqueidentifier,
		[IdInList] int,
		[AbonentNumber] nvarchar ( 20 ) collate database_default,
		[CallResult] int,
		[CallResultInfo] nvarchar ( 100 ) collate database_default,
		[DateTimeStart] DateTime,
		[DateStart] DateTime,
		[TimeStart] DateTime,
		[DateTimeStop] DateTime,
		[IdConn] uniqueidentifier,
		[IdChain] uniqueidentifier,
		[IdAUser] uniqueidentifier,
		[IdBUser] uniqueidentifier,
		[IdExtLine] uniqueidentifier,
		[LenTime] float, 
		[LenQueue] float,
		[IsBetweenOp] bit,
		[OpponentInfo] nvarchar ( 100 ) collate database_default,
		[IsHandled] bit,
		[IsSuccess] bit,
		[IsRecorded] bit,
		[IdRecDir] int,
		[ALineNum] nvarchar ( 12 ) collate database_default,
		[BLineNum] nvarchar ( 12 ) collate database_default,
		[UserResult] int default 0
	)

 	--Create Index [#xx] ON #temp ([IdAUser], [IdBUser]) -- нужен ли??
	

	-- Declare @idempty uniqueidentifier -- используется только в 1ом месте 
	--Set @idempty = '00000000-0000-0000-0000-000000000000'

	Declare @idop uniqueidentifier
	Set @idop = nullif(@idoperator,'00000000-0000-0000-0000-000000000000')


	--Alex 101118
	Declare @idch uniqueidentifier
	set @idch = @idchain

	if ( not @callresult in (6, 7, 20, 13) ) -- оператора в пустышку заносим в случае отклонения и/или пропуска вызова оператором -- или 13 :)
	    Select @idop = NULL, @idch= NULL	--Alex 101118

    IF EXISTS (SELECT 1 FROM A_Stat_Connections_1x1 With ( INDEX ( idx_StatConnections_IdChain ), NOLOCK ) Where IdChain = @idchain)
    	BEGIN
        -- вставляем коммутации во времянку (0, 1)
            Insert Into #temp ( IdProject, IdTask, IsOutput, IdEffort, IdInList, AbonentNumber, CallResult, 
                        CallResultInfo, DateTimeStart, DateStart, TimeStart, DateTimeStop, 
                        IdConn, IdChain, IdAUser, IdBUser, IdExtLine, LenTime, LenQueue, 
                        IsRecorded, IdRecDir, ALineNum, BLineNum, UserResult ) --ARecFile, BRecFile )
            Select @idproject, @idtask, @isoutput, @ideffort, @idinlist, @number, @callresult, @callresultinfo, 
                TimeStart, 
                Cast ( Floor ( Cast ( TimeStart as Float ) ) as DateTime ),
                Cast ( Cast ( TimeStart as Float ) - FLOOR ( Cast ( IsNull ( TimeStart, @dtnow ) as Float ) ) as DateTime ),
                IsNull ( TimeStop, @dtnow ),		
                Id, IdChain, AUserId, BUserId, 
                @idextline,
                Cast ( ( IsNull ( TimeStop, @dtnow ) - TimeAnswer ) as float ) * 86400 as LenTime, @lenqueue,
                IsRecorded, IdRecDir, ALineNum, BLineNum, --ARecFile, BRecFile
                @userresult
            FROM A_Stat_Connections_1x1 c With ( INDEX ( idx_StatConnections_IdChain ), NOLOCK )
            Where IdChain = @idchain
            -- удаляем записи по времени не пересекающиеся с попыткой
            -- Присоединение к звонку происходит посередине. Поэтому малость изменений
              AND NOT (
                        (COALESCE(TimeStop, @dtnow) <= @timestart) OR
                        (   @isattached = 1 -- Но IVR тоже тем не менее удаляем по старому
                            AND TimeStart < @timestart
                            And ((AUserId = 'AB000000-0000-0000-0000-000000000000' ) or ( BUserId = 'AB000000-0000-0000-0000-000000000000' ))
                        )
                      )
                      
            Declare @timeendeff datetime
            SET @timeendeff = COALESCE((Select min ( DateTimeStart ) From #temp),@timeexit)

            if ( @timeendeff < @timestart )
                Set @timestart = @timeendeff

            -- Пустышку добавим для общности, потом исключится если не нужна
            Insert Into #temp 
            ( 	IdProject, IdTask, IsOutput, IdEffort, 
                IdOperator, 
                IdInList, AbonentNumber, CallResult, CallResultInfo, DateTimeStart, DateStart, TimeStart, DateTimeStop,
                IdChain,	--Alex 101118
                --IdConn, IdAUser, IdBUser, 
                IdExtLine, LenTime, LenQueue, 
                --IsBetweenOp, 
                IsHandled, IsSuccess,
                --IsRecorded, IdRecDir, ARecFile, BRecFile 
                UserResult,OpponentInfo
            )
            Values 
            ( 	@idproject, @idtask, @isoutput, @ideffort, 
                @idop, 
                @idinlist, @number, @callresult, @callresultinfo, @timestart,
                Cast ( Floor ( Cast ( @timestart as float ) ) as DateTime ), 
                Cast ( ( Cast ( @timestart as float ) - Floor ( Cast ( @timestart as float ) ) ) as DateTime ), @timeendeff,
                @idch,   --@idchain,  --Alex 101118
                --@idempty, @idempty, @idempty, 
                @idextline, 
                Cast ( @timeendeff - @timestart as float ) * 86400,
                @lenqueue, 
                --0, 
                0, 0,
                --0, 0, '', '',
                @userresult,
                ''
            )
            
            Update t
            Set IdOperator = case when (@isoutput > 0) then IdAUser else IdBUser end, 
                IsBetweenOp = CASE WHEN ((a_user.id IS NOT NULL OR t.IdAuser = 'AB000000-0000-0000-0000-000000000000')) AND
                                        ((b_user.id IS NOT NULL OR t.IdBuser = 'AB000000-0000-0000-0000-000000000000'))
                                   THEN 1
                                   ELSE 0
                              END,
                IsHandled = case when (Select Count (*) FROM #temp t Where DateTimeStart >= @timestart) > 1 then 1 else 0 end, --одна попытка всегда есть
                 -- успешно ? (5)---------------------------------------
                IsSuccess = case When ( @callresult = 5 ) then 1 else 0 end
            From #temp t
             LEFT JOIN A_Users a_user ON t.IdAUser = a_user.id 
             LEFT JOIN A_Users b_user ON t.IdBUser = b_user.id
            Where (a_user.id IS NOT NULL OR b_user.id IS NOT NULL)
                or ( t.IdAuser = 'AB000000-0000-0000-0000-000000000000' )
                or ( t.IdBuser = 'AB000000-0000-0000-0000-000000000000' )
            /*
            Update t
            Set -- вызов обработан? (4)--------------------------------
                  IsHandled = case when (Select Count (*) FROM #temp t Where DateTimeStart >= @timestart) > 1 then 1 else 0 end --одна попытка всегда есть
                 -- успешно ? (5)---------------------------------------
                , IsSuccess = case When ( @callresult = 5 ) then 1 else 0 end
            From #temp t
            
            */
    END ELSE BEGIN
    
            -- если эта попытка без коммутаций ?
            Declare @dtendeff DateTime
            Set @dtendeff = COALESCE(@timeexit,@dtnow)
            
            Insert Into #temp 
            ( 	IdProject, IdTask, IsOutput, IdEffort, 
                IdOperator, 
                IdInList, AbonentNumber, CallResult, CallResultInfo, 
                DateTimeStart, 
                DateStart, 
                TimeStart, 
                DateTimeStop,
                IdChain, 
                --IdConn, IdAUser, IdBUser, 
                IdExtLine, LenTime, LenQueue, 
                --IsBetweenOp, 
                IsHandled, IsSuccess,
                --IsRecorded, IdRecDir, ARecFile, BRecFile,
                UserResult, OpponentInfo
                )
            Values 
            ( 	@idproject, @idtask, @isoutput, @ideffort, 
                @idop, 
                @idinlist, @number, @callresult, @callresultinfo, 
                @timestart,
                Cast ( Floor ( Cast ( @timestart as float ) ) as DateTime ), 
                Cast ( ( Cast ( @timestart as float ) - Floor ( Cast ( @timestart as float ) ) ) as DateTime ), 
                @dtendeff,
                @idchain,
                --@idempty, @idempty, @idempty, 
                @idextline, 
                Cast ( @dtnow - @timestart as float ) * 86400,
                @lenqueue, 
                --0, 
                0, 0,
                --0, 0, '', '',
                @userresult,
                ''
            )
            --if ( @idoperator != '00000000-0000-0000-0000-000000000000' )
            --    Update #temp
            --    Set IdOperator = @idoperator
	END   
/*
	-- информация по абоненту (6)----------------
	Declare @opinfo nvarchar ( 255 )
	Select @opinfo = ''
	Update t
	Set OpponentInfo = @opinfo
	From #temp t

	-- выдираем те, которые относятся к куску цепочки, принадлежащему данной задаче (необходимо при переключениях из задачи в задачу)
	if OBJECT_ID('tempdb..#textract') IS NULL
		drop table #textract
        
	Select * into #textract 
	    From #temp 
*/
/*
	if ( @isoutput = 0 ) -- вырезаем свой кусок если задача входящая и флаг соответствующий передан
	begin
		Declare @dt1 datetime, @dt2 datetime
		--своя начальная
		Select @dt1 = Max ( DateTimeStart ) From #textract Where DateTimeStart <= @timestart and (( IdOperator is null ) or ( IdOperator = 'AB000000-0000-0000-0000-000000000000' ))
		--несвоя начальная
		Select @dt2 = Min ( DateTimeStart ) From #textract Where DateTimeStart > @timestart and (( IdOperator is null ) or ( IdOperator = 'AB000000-0000-0000-0000-000000000000' ))

		if ( not @dt2 is null )--28.03.2008 for special callresults
			Delete from #textract Where DateTimeStart < @dt1 or DateTimeStart >= @dt2 or LenTime is NULL
	end
*/
	IF ( @timeexit is not NULL ) begin	
		IF EXISTS(SELECT 1 FROM #temp Where DateTimeStart > @timeexit or LenTime is NULL) 
			Delete 
			From #temp 
			Where (DateTimeStart > @timeexit)
				or (LenTime is NULL)
	end

	-- поля которые могли остаться нуллами замещаем на дефолтовые значения
	Update #temp
	Set LenTime = COALESCE(LenTime,0),
        DateTimeStop = COALESCE(DateTimeStop,DateTimeStart)
	
	-- обновляем LastConnected (чтоб не по кубу менеджер задач шнырил)
	if ( @idinlist >= 0 ) begin
		
		Declare @lc_maxdt DateTime

		Select @lc_maxdt = Max ( DateTimeStart ) 
		From #temp
		Where IdOperator != '00000000-0000-0000-0000-000000000000' 
			and IdOperator != 'AB000000-0000-0000-0000-000000000000'
			and IsHandled = 1

		if ( @@rowcount > 0 ) begin
		
			Declare @lc_idoperator uniqueidentifier

			Select @lc_idoperator = IdOperator
			From #temp
			Where IdOperator != '00000000-0000-0000-0000-000000000000' 
				and IdOperator != 'AB000000-0000-0000-0000-000000000000'
				and IsHandled = 1
				and DateTimeStart = @lc_maxdt

			exec A_CallCenter_OperatorLastConnected_Set @lc_maxdt, @idproject, @idtask, @idlist, @idinlist, @number, @lc_idoperator
		end
	end


	-- обновляем куб Tmp_A_Cube_CC_EffortConnections ----------------------------
	Insert Into [oktell_cc_temp].[dbo].[Tmp_A_Cube_CC_EffortConnections] With ( TabLock )
        Select * FROM #temp

	--Информация по задачам в таблицу категорий A_Cube_CC_Cat_Task
	if not exists ( Select Id From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task] Where Id = @idtask )
		Insert Into [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task]
		Select Id, Name,Type, Isoutputtask From A_TaskManager_Tasks Where Id = @idtask	

	--Информация по проектам в таблицу категорий A_Cube_CC_Cat_Project
	if not exists ( Select Id From [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project] Where Id = @idproject )
		Insert Into [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project]
		Select Id, Name From A_TaskManager_Projects Where Id = @idproject

	-------------------------------------------------------------
	---- КУБ СОСТОЯНИЙ ОПЕРАТОРОВ -------------------------------
	-------------------------------------------------------------
	-- обновляется по таймеру из DBProvider

end
GO