﻿----------------------------------------------------------------------
Use oktell_settings
GO

if Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_ServerStreams' and TABLE_SCHEMA like 'dbo' )
if not Exists ( Select * From INFORMATION_SCHEMA.Columns Where TABLE_NAME like 'A_ServerStreams'
                                                                and TABLE_SCHEMA like 'dbo'
                                                                and COLUMN_NAME like 'SipPass' )
        Alter Table dbo.A_ServerStreams
        Add SipPass nvarchar(500),
			SipDisplay nvarchar(500),
			DefCallerId nvarchar(50)
GO

Update dbo.A_ServerStreams
Set SipPass = ''	
Where (SipPass is NULL)

Update dbo.A_ServerStreams
Set SipDisplay = ''
Where (SipDisplay is NULL)

Update dbo.A_ServerStreams
Set	DefCallerId = ''
Where (DefCallerId is NULL)
GO

Exec [dbo].H_createTableTrigger 'A_ServerStreams'
GO

Use oktell
GO

if exists ( Select* from sysobjects where id = object_id ( 'dbo.A_ServerStreams'))
	Drop view dbo.A_ServerStreams
GO

Create View dbo.A_ServerStreams
as
	Select *
	From oktell_settings.dbo.A_ServerStreams
GO


---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_Admin_OfficeMap_ServerStreams_Save'))
	Drop procedure dbo.A_Admin_OfficeMap_ServerStreams_Save
GO

Create procedure [dbo].[A_Admin_OfficeMap_ServerStreams_Save]
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		id uniqueidentifier primary key, 
		name nvarchar(500) collate database_default, 
		type int, 
		srvid uniqueidentifier, 
		sipuser nvarchar(500) collate database_default, 
		regtype int, 
		gateid uniqueidentifier, 
		dialmode int,
		sippass nvarchar(500) collate database_default,
		sipdisplay nvarchar(500) collate database_default,
		defcallerid nvarchar(50) collate database_default
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select id, name, type, srvid, sipuser, regtype, gateid, dialmode, sippass, sipdisplay, defcallerid
		From OpenXml (@idoc, '/items/stream', 2)	
		With ( 
			id varchar(50) '@id',
			name nvarchar(500) '@name',
			type int '@type',
			srvid varchar(50) '@srvid',
			sipuser nvarchar(500) '@sipuser',
			regtype int '@regtype',
			gateid varchar(50) '@gate',
			dialmode int '@dialmode',
			sippass nvarchar(500) '@sippass',
			sipdisplay nvarchar(500) '@sipdisp',
			defcallerid nvarchar(50) '@dcid'
		)	
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_ServerStreams
	Where (IsDynamicConfig = 0)
		and not exists ( Select 1 From #tmp_xml x Where A_ServerStreams.Id = x.id)
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Name = x.name,
		w.IdServer = x.srvid,
		w.Type = x.type,
		w.Account = x.sipuser,
		w.RegType = x.regtype,
		w.GateId = x.gateid,
		w.DialMode = x.dialmode,
		w.SipPass = x.sippass,
		w.SipDisplay = x.sipdisplay,
		w.DefCallerId = x.defcallerid
	From dbo.A_ServerStreams w, #tmp_xml x
	Where (w.Id = x.id)
		and (w.IsDynamicConfig = 0)
		and ((w.Name != x.name)
			or (w.IdServer != x.srvid)
			or (w.Type != x.type)
			or (w.Account != x.sipuser)
			or (w.RegType != x.regtype)
			or (w.GateId != x.gateid)
			or (w.DialMode != x.dialmode)
			or (w.SipPass != x.sippass)
			or (w.SipDisplay != x.sipdisplay)
			or (w.DefCallerId != x.defcallerid))			
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_ServerStreams (Id, Name, Type, IdServer, Account, RegType, GateId, DialMode, SipPass, SipDisplay, DefCallerId)
		Select Distinct id, name, type, srvid, sipuser, regtype, gateid, dialmode, sippass, sipdisplay, defcallerid
		From #tmp_xml x
		Where not exists ( Select 1 From dbo.A_ServerStreams w Where w.Id = x.id )
		
end
GO


---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_DynamicConfig_Add_Stream'))
	Drop procedure dbo.A_DynamicConfig_Add_Stream
GO

Create procedure [dbo].[A_DynamicConfig_Add_Stream]
 	@streamid uniqueidentifier,
	@streamname nvarchar(200),
	@streamtype int,
	@streamaccount nvarchar(200) = '',
	@streampass nvarchar(200) = '',
	@streamdisplay nvarchar(200) = '',
	@defcallerid nvarchar(50) = '',
	@regtype int,
	@gateid uniqueidentifier,
	@dialmode int = 0,
	@serverid uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
Begin

	Declare @srvid uniqueidentifier
	Set @srvid = @serverid
	if ( @srvid = '00000000-0000-0000-0000-000000000000' )
		Select @srvid = [Id] 
		From A_WorkPlaces 
		Where ( hasfastengine = 1 ) and ( ParentID = '00000000-0000-0000-0000-000000000000' )

	Insert into A_ServerStreams ( Id, Name, IdServer, Type, Account, RegType, GateId, DialMode, IsDynamicConfig, SipPass, SipDisplay, DefCallerId )
	Values ( @streamid, @streamname, @srvid, @streamtype, @streamaccount, @regtype, @gateid, @dialmode, 1, @streampass, @streamdisplay, @defcallerid )
	
End
GO


---------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_DynamicConfig_Modify_Stream'))
	Drop procedure dbo.A_DynamicConfig_Modify_Stream
GO

Create procedure [dbo].[A_DynamicConfig_Modify_Stream]
	@streamid uniqueidentifier,
	@streamname nvarchar(200) = NULL,
	@streamaccount nvarchar(200) = NULL,
	@streampass nvarchar(200) = NULL,
	@streamdisplay nvarchar(200) = NULL,
	@defcallerid nvarchar(50) = NULL
as
Begin	

	Update A_ServerStreams
	Set Name = case when (@streamname is not null) then @streamname else Name end,
		Account = case when (@streamname is not null) then @streamname else Name end,
		SipPass = case when (@streamname is not null) then @streamname else Name end,
		SipDisplay = case when (@streamname is not null) then @streamname else Name end,
		DefCallerId = case when (@streamname is not null) then @streamname else Name end
	Where (Id = @streamid)
		and ((( @streamname is not null ) and (@streamname != Name))
			or (( @streamaccount is not null ) and (@streamaccount != Account))
			or (( @streampass is not null ) and (@streampass != SipPass))
			or (( @streamdisplay is not null ) and (@streamdisplay != SipDisplay))
			or (( @defcallerid is not null ) and (@defcallerid != DefCallerId)))
End
GO


---------------------------------------------------------------------------
--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallRoutes_LineSave'))
--	Drop procedure dbo.A_CallRoutes_LineSave
--GO

--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallRoutes_LinesClear'))
--	Drop procedure dbo.A_CallRoutes_LinesClear
--GO

if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallRoutes_Save'))
	Drop procedure dbo.A_CallRoutes_Save
GO

Create procedure [dbo].[A_CallRoutes_Save]
	@id uniqueidentifier,
	@name nvarchar ( 200 ),
   	@description nvarchar ( 1000 ),
   	@command nvarchar ( 1000 ),
	@isexternal bit,
	@enabled bit,
	@type int,
	@code nvarchar ( 100 ) = '',
	@xmllines ntext = NULL,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

  -- Сохраняем направление  
	if exists ( Select 1 From A_CallRoutes where Id = @id ) begin
	
		Update A_CallRoutes
		Set Name = @name,
			Description = @description,
			Command = @command,
			IsExternal = @isexternal,
			Enabled = @enabled,
			Type = @type,
			Code = @code
		Where (Id = @id)
			and ((Name != @name)
				or (Description != @description)
				or (Command != @command)
				or (IsExternal != @isexternal)
				or (Enabled != @enabled)
				or (Type != @type)
				or (Code != @code))
	
	end else begin
	
		Insert into A_CallRoutes ( Id, Name, Description, Command, IsExternal, Enabled, Type, Code )
		Values ( @id, @name, @description, @command, @isexternal, @enabled, @type, @code )
		
	end
	
  -- Сохраняем состав линий
	if ( @xmllines is NULL )
		return

	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml
	Create table #tmp_xml
	( 
		id uniqueidentifier,
		pos int
	)
			
	Declare @idoc1 int
	Exec sp_xml_preparedocument @idoc1 OUTPUT, @xmllines
	
	Insert into #tmp_xml (id, pos)
		Select id, pos
		From OpenXml (@idoc1, '/items/l', 2)	
		With ( 
			id varchar(50) '@id',
			pos int '@pos'
		)
		
	-- Удаляем те, что отсутствуют в параметре, но есть в таблице
	Delete 
	From dbo.A_CallRoutes_Lines
	Where (IdRoute = @id)
		and not exists( Select 1
						From #tmp_xml x
						Where dbo.A_CallRoutes_Lines.IdLine = x.id )
	
	-- Обновляем, те что есть в обоих местах
	Update w	
	Set w.Pos = x.pos
	From dbo.A_CallRoutes_Lines w, #tmp_xml x
	Where (w.IdRoute = @id)
		and (w.IdLine = x.id)
		and (w.Pos != x.pos)			
		
	-- Добавляем те, что есть в параметре, но нет в таблице
	Insert Into dbo.A_CallRoutes_Lines ( IdRoute, IdLine, Pos )
		Select @id, x.id, x.pos
		From #tmp_xml x
		Where not exists ( Select 1
							From dbo.A_CallRoutes_Lines w
							Where (w.IdRoute = @id)
								and (w.IdLine = x.id))	
		
end
GO

-----------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallRoutes_ChangeEnabled'))
	Drop procedure dbo.A_CallRoutes_ChangeEnabled
GO

Create procedure [dbo].[A_CallRoutes_ChangeEnabled]
	@id uniqueidentifier,	
	@enabled bit,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	Update A_CallRoutes
	Set [Enabled] = @enabled
	Where ([Id] = @id)
		and ([Enabled] != @enabled)
	
end
GO

-----------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_CallRoutes_Delete'))
	Drop procedure dbo.A_CallRoutes_Delete
GO

Create procedure [dbo].[A_CallRoutes_Delete]
	@id uniqueidentifier,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	Delete 
	From A_CallRoutes
	Where Id = @id

	Delete
	From A_CallRoutes_Lines
	Where IdRoute = @id
	
end
GO


-----------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_SettingsTable_Clear'))
	Drop procedure dbo.A_SettingsTable_Clear
GO

Create procedure [dbo].[A_SettingsTable_Clear]
	@tablename nvarchar(500),
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	if exists ( select * From sysobjects where id = object_id ( @tablename )) begin
		
		Declare @sql nvarchar(4000)
		Set @sql = 'Delete From ' + @tablename
		
		exec sp_executesql @sql
		
	end
		
end
GO



-----------------------------------------------------
-----  A_HolidaysDefault ----------------------------
-----------------------------------------------------

Use oktell_settings
GO 

if (not object_id ( 'dbo.A_HolidaysDefault_tmp' ) is null)
	Drop table dbo.A_HolidaysDefault_tmp
GO

CREATE TABLE [dbo].[A_HolidaysDefault_tmp]
(
	[Date] [datetime] Primary key,
	[Type] [bit],
	[AsDayOfWeek] [int],
	[Description] [nvarchar](1000)
)
GO

if (not object_id ( 'tempdb..#tmp' ) is null)
	Drop table #tmp
GO

CREATE TABLE #tmp
(
	idx int identity (1,1),
	[Date] [datetime] NULL,
	[Type] [bit] NULL,
	[AsDayOfWeek] [int] NULL,
	[Description] [nvarchar](1000) collate database_default NULL	 
)

Insert into #tmp (date, type, asdayofweek, description)
	Select date, type, asdayofweek, description
	From A_HolidaysDefault	
				 
Insert into [dbo].[A_HolidaysDefault_tmp] (Date, Type, AsDayOfWeek, Description)
	Select x.date, x.type, x.asdayofweek, x.description
	From #tmp x
		Inner join ( Select date, Min(idx) as idx
					 From #tmp
					 Group by date ) a 
			on (a.date = x.date) and (x.idx = a.idx)
GO

if (exists ( Select 1 From A_HolidaysDefault_tmp )
	or not exists( Select 1 From A_HolidaysDefault))
begin
	Exec sp_rename A_HolidaysDefault, A_HolidaysDefault2
	Exec sp_rename A_HolidaysDefault_tmp, A_HolidaysDefault
	Drop table A_HolidaysDefault2
end
GO

if (not object_id ( 'dbo.H_createTableTrigger' ) is null)
	Exec [dbo].H_createTableTrigger 'A_HolidaysDefault'
GO

use oktell
GO

if (not object_id ( 'dbo.A_HolidaysDefault' ) is null)
	Drop View dbo.A_HolidaysDefault
GO

Create View dbo.A_HolidaysDefault
as 
	Select *
	From oktell_settings.dbo.A_HolidaysDefault
GO


-----------------------------------------------------
------ A_HolidaysUser -------------------------------
-----------------------------------------------------

Use oktell_settings
GO 

if (not object_id ( 'dbo.A_HolidaysUser_tmp' ) is null)
	Drop table dbo.A_HolidaysUser_tmp
GO

CREATE TABLE [dbo].[A_HolidaysUser_tmp]
(
	[Date] [datetime] Primary key,
	[Type] [bit],
	[AsDayOfWeek] [int],
	[Description] [nvarchar](1000)
)
GO

if (not object_id ( 'tempdb..#tmp' ) is null)
	Drop table #tmp
GO

CREATE TABLE #tmp
(
	idx int identity (1,1),
	[Date] [datetime] NULL,
	[Type] [bit] NULL,
	[AsDayOfWeek] [int] NULL,
	[Description] [nvarchar](1000) collate database_default NULL	 
)

Insert into #tmp (date, type, asdayofweek, description)
	Select date, type, asdayofweek, description
	From A_HolidaysUser
				 
Insert into [dbo].[A_HolidaysUser_tmp] (Date, Type, AsDayOfWeek, Description)
	Select x.date, x.type, x.asdayofweek, x.description
	From #tmp x
		Inner join ( Select date, Min(idx) as idx
					 From #tmp
					 Group by date ) a 
			on (a.date = x.date) and (x.idx = a.idx)
GO

if (exists ( Select 1 From A_HolidaysUser_tmp )
	or not exists( Select 1 From A_HolidaysUser))
begin
	Exec sp_rename A_HolidaysUser, A_HolidaysUser2
	Exec sp_rename A_HolidaysUser_tmp, A_HolidaysUser
	Drop table A_HolidaysUser2
end
GO

if (not object_id ( 'dbo.H_createTableTrigger' ) is null)
	Exec [dbo].H_createTableTrigger 'A_HolidaysUser'
GO

use oktell
GO

if (not object_id ( 'dbo.A_HolidaysUser' ) is null)
	Drop View dbo.A_HolidaysUser
GO

Create View dbo.A_HolidaysUser
as 
	Select *
	From oktell_settings.dbo.A_HolidaysUser
GO

-----------------------------------------------------------------------
--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_HolidaysUser_Add'))
--	Drop procedure dbo.A_HolidaysUser_Add
--GO

--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_HolidaysUser_ClearAll'))
--	Drop procedure dbo.A_HolidaysUser_ClearAll
--GO

if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_HolidaysUser_Save'))
	Drop procedure dbo.A_HolidaysUser_Save
GO

Create procedure [dbo].A_HolidaysUser_Save
	@xml ntext,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
	
		
	Select * FROM A_HolidaysUser
	
	-- Потоки (через XML)
	If object_id('tempdb..#tmp_xml') is not NULL
		Drop table #tmp_xml	
	Create table #tmp_xml 
	( 
		idx int identity (1,1) primary key,
		date datetime,
		type bit, 
		asdayofweek int, 
		description nvarchar(1000) collate database_default
	)
		
	Declare @idoc int
	Exec sp_xml_preparedocument @idoc OUTPUT, @xml
	
	Insert into #tmp_xml
		Select date, type, asdayofweek, description
		From OpenXml (@idoc, '/items/h', 2)	
		With ( 
			date datetime '@date',
			type bit '@type',
			asdayofweek int '@asdow',
			description nvarchar(1000) '@descr'
		)	
		
	If object_id('tempdb..#tmp_xml2') is not NULL
		Drop table #tmp_xml2
	Create table #tmp_xml2 
	( 	
		date datetime primary key,
		type bit, 
		asdayofweek int, 
		description nvarchar(1000) collate database_default
	)
	
	-- Уникальные строчки вырезаем
	Insert into #tmp_xml2 ( date, type, asdayofweek, description )
		Select x.date, x.type, x.asdayofweek, x.description
		From #tmp_xml x
			Inner join( Select x.date, Min(idx) as idx
						From #tmp_xml x
						Group by x.date )a 
					on a.idx = x.idx
			
		
	-- Удаляем тех, что нет в обновлении
	Delete
	From dbo.A_HolidaysUser
	Where not exists ( Select 1 
						From #tmp_xml2 x 
						Where (A_HolidaysUser.Date = x.date))
	
	-- Обновляем тех, что есть и там и там и у которых хоть что-то изменилось
	Update w
	Set w.Type = x.type,
		w.AsDayOfWeek = x.asdayofweek,
		w.Description = x.description
	From dbo.A_HolidaysUser w, #tmp_xml2 x
	Where (w.Date = x.date)
		and ((w.Type != x.type)
			or (w.AsDayOfWeek != x.asdayofweek)
			or (w.Description != x.description))			
		
	-- Добавляем тех, что есть только в обновлении
	Insert into dbo.A_HolidaysUser (Date, Type, AsDayOfWeek, Description)
		Select Distinct date, type, asdayofweek, description
		From #tmp_xml2 x
		Where not exists ( Select 1 
							From dbo.A_HolidaysUser w 
							Where (w.Date = x.date) )
		
end
GO


-----------------------------------------------------
----- A_Redirect_TimeTable --------------------------
-----------------------------------------------------

Use oktell_settings
GO 

if (not object_id ( 'dbo.A_Redirect_TimeTable_tmp' ) is null)
	Drop table dbo.A_Redirect_TimeTable_tmp
GO

CREATE TABLE [dbo].[A_Redirect_TimeTable_tmp]
(
	[IdRedir] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[StopTime] datetime not null
)
GO

if (not object_id ( 'tempdb..#tmp' ) is null)
	Drop table #tmp
GO

CREATE TABLE #tmp
(
	[idx] int identity (1,1),
	[IdRedir] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[StopTime] datetime  not null
)

Insert into #tmp ([IdRedir], [DayOfWeek], [StartTime], [StopTime])
	Select [IdRedir], [DayOfWeek], [StartTime], [StopTime]
	From A_Redirect_TimeTable
				 
Insert into [dbo].[A_Redirect_TimeTable_tmp] ([IdRedir], [DayOfWeek], [StartTime], [StopTime])
	Select x.[IdRedir], x.[DayOfWeek], x.[StartTime], x.[StopTime]
	From #tmp x
		Inner join ( Select [IdRedir], [DayOfWeek], [StartTime], [StopTime], Min([idx]) as idx
					 From #tmp
					 Group by [IdRedir], [DayOfWeek], [StartTime], [StopTime] ) a 
			on (a.idx = x.idx)
				and (a.[IdRedir] = x.[IdRedir]) 
				and (a.[DayOfWeek] = x.[DayOfWeek]) 
				and (a.[StartTime] = x.[StartTime]) 
				and (a.[StopTime] = x.[StopTime]) 
GO

if (exists ( Select 1 From A_Redirect_TimeTable_tmp )
	or not exists( Select 1 From A_Redirect_TimeTable))
begin
	Exec sp_rename A_Redirect_TimeTable, A_Redirect_TimeTable2
	Exec sp_rename A_Redirect_TimeTable_tmp, A_Redirect_TimeTable
	Drop table A_Redirect_TimeTable2
end
GO

Alter 
Table dbo.A_Redirect_TimeTable 
Add Constraint A_Redirect_TimeTable_pk Primary Key ([IdRedir], [DayOfWeek], [StartTime], [StopTime])
GO

if (not object_id ( 'dbo.H_createTableTrigger' ) is null)
	Exec [dbo].H_createTableTrigger 'A_Redirect_TimeTable'
GO

use oktell
GO

if (not object_id ( 'dbo.A_Redirect_TimeTable' ) is null)
	Drop View dbo.A_Redirect_TimeTable
GO

Create View dbo.A_Redirect_TimeTable
as 
	Select *
	From oktell_settings.dbo.A_Redirect_TimeTable
GO

-----------------------------------------------------
----- A_SmsTaskManager_RunPeriods -------------------
-----------------------------------------------------

Use oktell_settings
GO 

if (not object_id ( 'dbo.A_SmsTaskManager_RunPeriods_tmp' ) is null)
	Drop table dbo.A_SmsTaskManager_RunPeriods_tmp
GO

CREATE TABLE [dbo].[A_SmsTaskManager_RunPeriods_tmp]
(
	[TaskId] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[FinishTime] datetime not null
)
GO

if (not object_id ( 'tempdb..#tmp' ) is null)
	Drop table #tmp
GO

CREATE TABLE #tmp
(
	[idx] int identity (1,1),
	[TaskId] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[FinishTime] datetime  not null
)

Insert into #tmp ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select [TaskId], [DayOfWeek], [StartTime], [FinishTime]
	From A_SmsTaskManager_RunPeriods
				 
Insert into [dbo].[A_SmsTaskManager_RunPeriods_tmp] ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select x.[TaskId], x.[DayOfWeek], x.[StartTime], x.[FinishTime]
	From #tmp x
		Inner join ( Select [TaskId], [DayOfWeek], [StartTime], [FinishTime], Min([idx]) as idx
					 From #tmp
					 Group by [TaskId], [DayOfWeek], [StartTime], [FinishTime] ) a 
			on (a.idx = x.idx)
				and (a.[TaskId] = x.[TaskId]) 
				and (a.[DayOfWeek] = x.[DayOfWeek]) 
				and (a.[StartTime] = x.[StartTime]) 
				and (a.[FinishTime] = x.[FinishTime]) 
GO

if (exists ( Select 1 From A_SmsTaskManager_RunPeriods_tmp )
	or not exists( Select 1 From A_SmsTaskManager_RunPeriods))
begin
	Exec sp_rename A_SmsTaskManager_RunPeriods, A_SmsTaskManager_RunPeriods2
	Exec sp_rename A_SmsTaskManager_RunPeriods_tmp, A_SmsTaskManager_RunPeriods
	Drop table A_SmsTaskManager_RunPeriods2
end
GO

Alter 
Table dbo.A_SmsTaskManager_RunPeriods
Add Constraint A_SmsTaskManager_RunPeriods_pk Primary Key ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
GO

if (not object_id ( 'dbo.H_createTableTrigger' ) is null)
	Exec [dbo].H_createTableTrigger 'A_SmsTaskManager_RunPeriods'
GO

use oktell
GO

if (not object_id ( 'dbo.A_SmsTaskManager_RunPeriods' ) is null)
	Drop View dbo.A_SmsTaskManager_RunPeriods
GO

Create View dbo.A_SmsTaskManager_RunPeriods
as 
	Select *
	From oktell_settings.dbo.A_SmsTaskManager_RunPeriods
GO

-----------------------------------------------------
----- A_SvcTaskManager_RunPeriods -------------------
-----------------------------------------------------

Use oktell_settings
GO 

if (not object_id ( 'dbo.A_SvcTaskManager_RunPeriods_tmp' ) is null)
	Drop table dbo.A_SvcTaskManager_RunPeriods_tmp
GO

CREATE TABLE [dbo].[A_SvcTaskManager_RunPeriods_tmp]
(
	[TaskId] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[FinishTime] datetime not null
)
GO

if (not object_id ( 'tempdb..#tmp' ) is null)
	Drop table #tmp
GO

CREATE TABLE #tmp
(
	[idx] int identity (1,1),
	[TaskId] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[FinishTime] datetime  not null
)

Insert into #tmp ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select [TaskId], [DayOfWeek], [StartTime], [FinishTime]
	From A_SvcTaskManager_RunPeriods
				 
Insert into [dbo].[A_SvcTaskManager_RunPeriods_tmp] ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select x.[TaskId], x.[DayOfWeek], x.[StartTime], x.[FinishTime]
	From #tmp x
		Inner join ( Select [TaskId], [DayOfWeek], [StartTime], [FinishTime], Min([idx]) as idx
					 From #tmp
					 Group by [TaskId], [DayOfWeek], [StartTime], [FinishTime] ) a 
			on (a.idx = x.idx)
				and (a.[TaskId] = x.[TaskId]) 
				and (a.[DayOfWeek] = x.[DayOfWeek]) 
				and (a.[StartTime] = x.[StartTime]) 
				and (a.[FinishTime] = x.[FinishTime]) 
GO

if (exists ( Select 1 From A_SvcTaskManager_RunPeriods_tmp )
	or not exists( Select 1 From A_SvcTaskManager_RunPeriods))
begin
	Exec sp_rename A_SvcTaskManager_RunPeriods, A_SvcTaskManager_RunPeriods2
	Exec sp_rename A_SvcTaskManager_RunPeriods_tmp, A_SvcTaskManager_RunPeriods
	Drop table A_SvcTaskManager_RunPeriods2
end
GO

Alter 
Table dbo.A_SvcTaskManager_RunPeriods
Add Constraint A_SvcTaskManager_RunPeriods_pk Primary Key ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
GO

if (not object_id ( 'dbo.H_createTableTrigger' ) is null)
	Exec [dbo].H_createTableTrigger 'A_SvcTaskManager_RunPeriods'
GO

use oktell
GO

if (not object_id ( 'dbo.A_SvcTaskManager_RunPeriods' ) is null)
	Drop View dbo.A_SvcTaskManager_RunPeriods
GO

Create View dbo.A_SvcTaskManager_RunPeriods
as 
	Select *
	From oktell_settings.dbo.A_SvcTaskManager_RunPeriods
GO

-----------------------------------------------------
----- A_TaskManager_RunPeriods -------------------
-----------------------------------------------------

Use oktell_settings
GO 

if (not object_id ( 'dbo.A_TaskManager_RunPeriods_tmp' ) is null)
	Drop table dbo.A_TaskManager_RunPeriods_tmp
GO

CREATE TABLE [dbo].[A_TaskManager_RunPeriods_tmp]
(
	[TaskId] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[FinishTime] datetime not null
)
GO

if (not object_id ( 'tempdb..#tmp' ) is null)
	Drop table #tmp
GO

CREATE TABLE #tmp
(
	[idx] int identity (1,1),
	[TaskId] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[FinishTime] datetime  not null
)

Insert into #tmp ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select [TaskId], [DayOfWeek], [StartTime], [FinishTime]
	From A_TaskManager_RunPeriods
				 
Insert into [dbo].[A_TaskManager_RunPeriods_tmp] ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select x.[TaskId], x.[DayOfWeek], x.[StartTime], x.[FinishTime]
	From #tmp x
		Inner join ( Select [TaskId], [DayOfWeek], [StartTime], [FinishTime], Min([idx]) as idx
					 From #tmp
					 Group by [TaskId], [DayOfWeek], [StartTime], [FinishTime] ) a 
			on (a.idx = x.idx)
				and (a.[TaskId] = x.[TaskId]) 
				and (a.[DayOfWeek] = x.[DayOfWeek]) 
				and (a.[StartTime] = x.[StartTime]) 
				and (a.[FinishTime] = x.[FinishTime]) 
GO

if (exists ( Select 1 From A_TaskManager_RunPeriods_tmp )
	or not exists( Select 1 From A_TaskManager_RunPeriods))
begin
	Exec sp_rename A_TaskManager_RunPeriods, A_TaskManager_RunPeriods2
	Exec sp_rename A_TaskManager_RunPeriods_tmp, A_TaskManager_RunPeriods
	Drop table A_TaskManager_RunPeriods2
end
GO

Alter 
Table dbo.A_TaskManager_RunPeriods
Add Constraint A_TaskManager_RunPeriods_pk Primary Key ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
GO

if (not object_id ( 'dbo.H_createTableTrigger' ) is null)
	Exec [dbo].H_createTableTrigger 'A_TaskManager_RunPeriods'
GO

use oktell
GO

if (not object_id ( 'dbo.A_TaskManager_RunPeriods' ) is null)
	Drop View dbo.A_TaskManager_RunPeriods
GO

Create View dbo.A_TaskManager_RunPeriods
as 
	Select *
	From oktell_settings.dbo.A_TaskManager_RunPeriods
GO

-----------------------------------------------------
-------------------------------------------------------
if exists ( Select * FRom sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_SetActivity'))
	Drop procedure dbo.A_TaskManager_Task_SetActivity
GO

Create Procedure [dbo].[A_TaskManager_Task_SetActivity]
	@id uniqueidentifier,
	@activityflag int,
	@authorizeduserid uniqueidentifier = NULL
as
begin
	
	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	Update A_TaskManager_Tasks
	Set Active = @activityflag,
		Done = 0,
		StopReason = 0, 
		Resulted = 0, 
		ResultType = 0, 
		ResultInfo = '', 
		ResultDateTime = GetDate()
	Where (Id = @id)
		and ((Active != @activityflag)
			or (Done != 0)
			or (StopReason != 0)
			or (Resulted != 0)
			or (ResultType != 0)
			or (ResultInfo != ''))	
end
GO

-------------------------------------------------------
if exists ( Select * FRom sysobjects where id = object_id ( 'dbo.A_SmsTaskManager_Task_SetActivity'))
	Drop procedure dbo.A_SmsTaskManager_Task_SetActivity
GO

Create procedure [dbo].[A_SmsTaskManager_Task_SetActivity]
	@id uniqueidentifier,
	@activityflag int,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		

	Update A_SmsTaskManager_Tasks
	Set Active = @activityflag,
		Done = 0		
	Where (Id = @id)
		and ((Active != @activityflag)
			or (Done != 0))
	
end
GO


-------------------------------------------------------
if exists ( Select * FRom sysobjects where id = object_id ( 'dbo.A_SvcTaskManager_Task_SetActivity'))
	Drop procedure dbo.A_SvcTaskManager_Task_SetActivity
GO

Create procedure [dbo].[A_SvcTaskManager_Task_SetActivity]
	@id uniqueidentifier,
	@activityflag int,
	@authorizeduserid uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @authorizeduserid
		
		
	Update A_SvcTaskManager_Tasks
	Set Active = @activityflag
	Where (Id = @id)
		and (Active != @activityflag)		
					
end
GO

-------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_TaskManager_Task_Save'))
	Drop procedure dbo.[A_TaskManager_Task_Save]
GO

Create Procedure [dbo].[A_TaskManager_Task_Save]
	@id uniqueidentifier,
	@idproject uniqueidentifier,
	@name nvarchar(500),
	@active int,
	@done int,
	@type int,
	@ivrname nvarchar(500),
	@stopreason int,
	@talkscript nvarchar(500),
	@pausewhenbusy float,
	@pausewhennotrespond float,
	@maxeffortcountbusy int,
	@maxeffortcountnotrespond int,
	@mintimetocheckdone int,
	@tryconnecttime int,
	@choiceoperator int,
	@deleteaftersuccess int,
	@isextnumber int,
	@isoutputtask int,
	@resulted int,
	@resulttype int,
	@resultinfo nvarchar(500),
	@resultdatetime datetime,
	@creationuserid uniqueidentifier,
	@creationtype int,
	@creationdatetime datetime,
	@startrundatetime datetime,
	@stoprundatetime datetime,
	@canadddynamically int,
	@tagdouble float,
	@listid uniqueidentifier,
	@operatorshowquery bit,
	@operatorshownotify bit,
	@progresscall bit,
	@progresscallmaxcount int,
	@progresscallkeepqueue bit,
	@predictcall bit,
	@predictcalloffsetsec int,
	@queueuse bit,
	@queuemaxcount int,
	@queuemusic nvarchar ( 500 ),
	@queueloyality int,
	@choiceabonents int,
	@choicenumber int,
	@choicetable int,
	@choiceprogress int,
	@dontsetdone bit,
	@pausesynclistwaitingnewabonent int,
	@incomingcallsmaxcount int,
	@servicescript nvarchar ( 200 ),
	@dialcallerid nvarchar ( 20 ),
	@incomingupdateifexists bit,
	@activeonholidays bit,
	@code nvarchar ( 30 ),
	@pausesynclistforcedsec int,
	@queuemaxtimeminutes int,
	@predictcallerrprobability int,
	@queuerandomfrompath nvarchar ( 200 ),
	@savedb	bit,
	@directdbmethod int = 0,
	@directdbmethodproc nvarchar ( 200 ) = '',
	@streamparams image = NULL,
	@iduser uniqueidentifier = NULL
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser
		
		
	if ( exists ( Select Id From A_TaskManager_Tasks Where Id = @id )) begin
	
		-- Обновляет задачу
		Update A_TaskManager_Tasks
		Set 
			IdProject			= @idproject,
			Name 				= @name, 			
			Active 				= @active, 
			Done 				= @done, 
			Type 				= @type, 
			IvrName 			= @ivrname, 
			StopReason 			= @stopreason, 
			TalkScript 			= @talkscript, 
			PauseWhenBusy 			= @pausewhenbusy, 
			PauseWhenNotRespond 		= @pausewhennotrespond, 
			MaxEffortCountBusy 		= @maxeffortcountbusy, 
			MaxEffortCountNotRespond 	= @maxeffortcountnotrespond, 
			MinTimeToCheckDone 		= @mintimetocheckdone, 
			TryConnectTime 			= @tryconnecttime, 
			ChoiceOperator			= @choiceoperator,
			DeleteAfterSuccess 		= @deleteaftersuccess, 
			IsExtNumber 			= @isextnumber, 
			IsOutputTask			= @isoutputtask,
			Resulted 			= @resulted, 
			ResultType 			= @resulttype, 
			ResultInfo 			= @resultinfo, 
			ResultDateTime 			= @resultdatetime, 
			CreationUserId 			= @creationuserid, 
			CreationType 			= @creationtype, 
			CreationDateTime 		= @creationdatetime,
			StartRunDateTime		= @startrundatetime,
			StopRunDateTime			= @stoprundatetime,
			CanAddDynamically		= @canadddynamically,	
			TagDouble			= @tagdouble,	
			ListId				= @listid,
			OperatorShowQuery		= @operatorshowquery,
			OperatorShowNotify		= @operatorshownotify,
			ProgressCall			= @progresscall,
			ProgressCallMaxCount		= @progresscallmaxcount,
			ProgressCallKeepQueue		= @progresscallkeepqueue,
			PredictCall			= @predictcall,
			PredictCallOffsetSec		= @predictcalloffsetsec,
			QueueUse 			= @queueuse,
			QueueMaxCount			= @queuemaxcount,
			QueueMusic			= @queuemusic,
			QueueLoyality			= @queueloyality,
			ChoiceAbonent			= @choiceabonents,
			ChoiceNumber			= @choicenumber,
			ChoiceTable			= @choicetable,
			ChoiceProgress			= @choiceprogress,
			DontSetDone			= @dontsetdone,
			PauseSyncListWaitingNewAbonent	= @pausesynclistwaitingnewabonent,
			IncomingCallsMaxCount		= @incomingcallsmaxcount,
			ServiceScript			= @servicescript,
			DialCallerId			= @dialcallerid,
			IncomingUpdateIfExists		= @incomingupdateifexists,
			ActiveOnHolidays		= @activeonholidays,
			Code				= @code,
			PauseSyncListForcedSec		= @pausesynclistforcedsec,
			QueueMaxTimeMinutes		= @queuemaxtimeminutes,
			PredictCallErrProbability	= @predictcallerrprobability,
			QueueRandomFromPath		= @queuerandomfrompath,
			SaveDB				= @savedb,
			DirectDbMethod			= @directdbmethod,
			DirectDbMethodProc		= @directdbmethodproc
 		Where
			Id = @id


		-- Обновляет свойства дополнительные
		Update A_TaskManager_TaskProperties
		Set Stream = @streamparams
		Where IdTask = @id
		
		if ( @@rowcount = 0 ) begin
			insert into A_TaskManager_TaskProperties ( IdTask, Stream )
			Values ( @id, @streamparams )
		end
		
	end else begin
	
		-- Добавляет новую задачу
		Insert into A_TaskManager_Tasks
		(
			Id, 
			IdProject,
			Name, 
			Active, 
			Done, 
			Type, 
			IvrName, 
			StopReason, 
			TalkScript, 
			PauseWhenBusy, 
			PauseWhenNotRespond, 
			MaxEffortCountBusy, 
			MaxEffortCountNotRespond, 
			MinTimeToCheckDone, 
			TryConnectTime, 
			ChoiceOperator, 
			DeleteAfterSuccess, 
			IsExtNumber, 
			IsOutputTask, 
			Resulted, 
			ResultType, 
			ResultInfo, 
			ResultDateTime, 
			CreationUserId, 
			CreationType, 
			CreationDateTime, 
			StartRunDateTime, 
			StopRunDateTime, 
			CanAddDynamically, 
			TagDouble, 
			ListId,
			OperatorShowQuery,
			OperatorShowNotify,
			ProgressCall,
			ProgressCallMaxCount,
			ProgressCallKeepQueue,
			PredictCall,
			PredictCallOffsetSec,
			QueueUse,
			QueueMaxCount,
			QueueMusic,
			QueueLoyality,
			ChoiceAbonent,
			ChoiceNumber,
			ChoiceTable,
			ChoiceProgress,
			DontSetDone,			
			PauseSyncListWaitingNewAbonent,
			IncomingCallsMaxCount,
			ServiceScript,
			DialCallerId,
			IncomingUpdateIfExists,
			ActiveOnHolidays,
			Code,
			PauseSyncListForcedSec,
			QueueMaxTimeMinutes,
			PredictCallErrProbability,
			QueueRandomFromPath,
			SaveDB,
			DirectDbMethod,
			DirectDbMethodProc
		) 
		Values  
		(
			@id,
			@idproject,
			@name,
			@active,
			@done,
			@type,
			@ivrname,
			@stopreason,
			@talkscript,
			@pausewhenbusy,
			@pausewhennotrespond,
			@maxeffortcountbusy,
			@maxeffortcountnotrespond,
			@mintimetocheckdone,
			@tryconnecttime,
			@choiceoperator,
			@deleteaftersuccess,
			@isextnumber,
			@isoutputtask,
			@resulted,
			@resulttype,
			@resultinfo,
			@resultdatetime,
			@creationuserid,
			@creationtype,
			@creationdatetime,
			@startrundatetime,
			@stoprundatetime,
			@canadddynamically,
			@tagdouble,
			@listid,
			@operatorshowquery,
			@operatorshownotify,
			@progresscall,
			@progresscallmaxcount,
			@progresscallkeepqueue,
			@predictcall,
			@predictcalloffsetsec,
			@queueuse,
			@queuemaxcount,
			@queuemusic,
			@queueloyality,
			@choiceabonents,
			@choicenumber,
			@choicetable,
			@choiceprogress,
			@dontsetdone,
			@pausesynclistwaitingnewabonent,
			@incomingcallsmaxcount,
			@servicescript,
			@dialcallerid,
			@incomingupdateifexists,
			@activeonholidays,
			@code,
			@pausesynclistforcedsec,
			@queuemaxtimeminutes,
			@predictcallerrprobability,
			@queuerandomfrompath,
			@savedb,
			@directdbmethod,
			@directdbmethodproc
		)		

		/* Добавляет свойства для нее
		*/
		Insert into A_TaskManager_TaskProperties ( IdTask, Stream )
		Values ( @id, @streamparams )
	end


	-- Политика прав доступа
	if ( @creationtype != 4 ) begin --AUTO не надо
	
		/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии 
		*/
		exec A_Policy_Objects_Save
			@idObject = @id,
			@type = 102,
			@name = @name,
			@idParentProject = @idproject,
			@actionUserId = @iduser
			
		/* Заказ на обновление кэша групп пользователей (операторы супервизоры новые появились) 
		*/
		exec A_Policy_SubjectGroupCache_SetUpdate
		
	end


	-- в базе кубов поправляем имя юзера
	If Exists ( Select * FROM Master..sysdatabases where name like 'oktell_cc_temp' )
	If Exists ( Select * FROM oktell_cc_temp..sysobjects where id = Object_Id ( 'oktell_cc_temp.dbo.A_Cube_CC_Cat_Task' ) )
		Update oktell_cc_temp.dbo.A_Cube_CC_Cat_Task
		Set Name = @name
		Where Id = @id
end
GO

-------------------------------------------------------------------------
if exists ( Select * from sysobjects where id = object_id ( 'dbo.A_SmsTaskManager_Task_Save'))
	Drop procedure dbo.[A_SmsTaskManager_Task_Save]
GO

Create Procedure [dbo].[A_SmsTaskManager_Task_Save]
	@id 				uniqueidentifier,
	@idproject			uniqueidentifier,
	@name				nvarchar(500),
	@code				nvarchar(30),
	@active				int,
	@done				int,
	@type				int,
	@svcscript			nvarchar(500),
	@stopreason			int,
	@choiceoperator		int,
	@deleteaftersuccess	int,
	@isoutputtask		int,
	@resulted			int,
	@resulttype			int,
	@resultinfo			nvarchar(500),
	@resultdatetime		datetime,
	@creationuserid		uniqueidentifier,
	@creationtype		int,
	@creationdatetime	datetime,
	@startrundatetime	datetime,
	@stoprundatetime	datetime,
	@tagdouble			float,
	@listid				uniqueidentifier,
	@dontsetdone		bit,
	@activeonholidays	bit,
	@pausesynclistwaitingnewabonent int,
	@pausesynclistforcedsec		int,
	@ismultithread		bit,
	@threadcount		int,
	@incomingupdateifexists		bit,
	@iduser				uniqueidentifier = NULL
	
as
begin

	If object_id('dbo.H_Authorizeduserid_SPID_Save') is not NULL
		EXEC dbo.H_Authorizeduserid_SPID_Save @iduser
		

	if ( exists ( Select Id from A_SmsTaskManager_Tasks where Id = @id )) begin
	
		/* Обновляет задачу */
		Update A_SmsTaskManager_Tasks
		Set 
			IdProject			= @idproject,
			Name 				= @name,
			Code				= @code,
			Active 				= @active, 
			Done 				= @done, 
			Type 				= @type, 
			SvcScript 			= @svcscript, 
			StopReason 			= @stopreason, 
			ChoiceOperator		= @choiceoperator,
			DeleteAfterSuccess 	= @deleteaftersuccess, 
			IsOutputTask		= @isoutputtask,
			Resulted 			= @resulted, 
			ResultType 			= @resulttype, 
			ResultInfo 			= @resultinfo, 
			ResultDateTime 		= @resultdatetime, 
			CreationUserId 		= @creationuserid, 
			CreationType 		= @creationtype, 
			CreationDateTime 	= @creationdatetime,
			StartRunDateTime	= @startrundatetime,
			StopRunDateTime		= @stoprundatetime,
			TagDouble			= @tagdouble,	
			ListId				= @listid,
			DontSetDone			= @dontsetdone,
			ActiveOnHolidays	= @activeonholidays,
			PauseSyncListWaitingNewAbonent	= @pausesynclistwaitingnewabonent,
			PauseSyncListForcedSec		= @pausesynclistforcedsec,
			IsMultiThread		= @ismultithread,
			ThreadCount			= @threadcount,
			IncomingUpdateIfExists		= @incomingupdateifexists
 		Where
			Id = @id
	
	end	else begin
		
		Insert into A_SmsTaskManager_Tasks
		(
			Id, 
			IdProject,
			Name, 
			Code,
			Active, 
			Done, 
			Type, 
			SvcScript, 
			StopReason, 
			ChoiceOperator, 
			DeleteAfterSuccess, 
			IsOutputTask, 
			Resulted, 
			ResultType, 
			ResultInfo, 
			ResultDateTime, 
			CreationUserId, 
			CreationType, 
			CreationDateTime, 
			StartRunDateTime, 
			StopRunDateTime, 
			TagDouble, 
			ListId,
			DontSetDone,
			ActiveOnHolidays,
			PauseSyncListWaitingNewAbonent,
			PauseSyncListForcedSec,
			IsMultiThread,
			ThreadCount,
			IncomingUpdateIfExists
		) 
		Values  
		(
			@id,
			@idproject,
			@name,
			@code,
			@active,
			@done,
			@type,
			@svcscript,
			@stopreason,
			@choiceoperator,
			@deleteaftersuccess,
			@isoutputtask,
			@resulted,
			@resulttype,
			@resultinfo,
			@resultdatetime,
			@creationuserid,
			@creationtype,
			@creationdatetime,
			@startrundatetime,
			@stoprundatetime,
			@tagdouble,
			@listid,
			@dontsetdone,
			@activeonholidays,
			@pausesynclistwaitingnewabonent,
			@pausesynclistforcedsec,
			@ismultithread,
			@threadcount,
			@incomingupdateifexists
		)		
	end
	
	/* В таблицах политики прав доступа изменяем имя и проставляем дефолтные привилегии 
	*/
	exec A_Policy_Objects_Save
		@idObject = @id,
		@type = 102,
		@name = @name,
		@idParentProject = @idproject,
		@actionUserId = @iduser
			
	--if ( @type != 0 ) begin --Auto не нужны	
	--	-- Заказ на обновление кэша групп пользователей (операторы супервизоры новые появились)
	--	exec A_Policy_SubjectGroupCache_SetUpdate		
	--end
		
end
GO



-------------------------------------------------------------------------
-----------------------------------------------------------
-----------------------------------------------------------
-----------------------------------------------------------
Use oktell_settings

if (not object_id ( 'tempdb..#tmpT' ) is null)
	Drop table #tmpT

CREATE TABLE #tmpT
(
	[idx] int identity (1,1),
	[TaskId] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[FinishTime] datetime  not null
)

---------

Insert into #tmpT ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select [TaskId], [DayOfWeek], 
		DateAdd ( yyyy, 2000-DatePart(yyyy, [StartTime]), [StartTime]), 
		DateAdd ( yyyy, 2000-DatePart(yyyy, [FinishTime]), [FinishTime])
	From [A_TaskManager_RunPeriods]

Truncate table [dbo].[A_TaskManager_RunPeriods]
	
Insert into [dbo].[A_TaskManager_RunPeriods] ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select x.[TaskId], x.[DayOfWeek], x.[StartTime], x.[FinishTime]
	From #tmpT x
		Inner join ( Select [TaskId], [DayOfWeek], [StartTime], [FinishTime], Min([idx]) as idx
					 From #tmpT
					 Group by [TaskId], [DayOfWeek], [StartTime], [FinishTime] ) a 
			on (a.idx = x.idx)
				and (a.[TaskId] = x.[TaskId]) 
				and (a.[DayOfWeek] = x.[DayOfWeek]) 
				and (a.[StartTime] = x.[StartTime]) 
				and (a.[FinishTime] = x.[FinishTime]) 	
		
Truncate table #tmpT
				
---------

Insert into #tmpT ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select [TaskId], [DayOfWeek], 
		DateAdd ( yyyy, 2000-DatePart(yyyy, [StartTime]), [StartTime]), 
		DateAdd ( yyyy, 2000-DatePart(yyyy, [FinishTime]), [FinishTime])
	From [A_SmsTaskManager_RunPeriods]

Truncate table [dbo].[A_SmsTaskManager_RunPeriods]
	
Insert into [dbo].[A_SmsTaskManager_RunPeriods] ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select x.[TaskId], x.[DayOfWeek], x.[StartTime], x.[FinishTime]
	From #tmpT x
		Inner join ( Select [TaskId], [DayOfWeek], [StartTime], [FinishTime], Min([idx]) as idx
					 From #tmpT
					 Group by [TaskId], [DayOfWeek], [StartTime], [FinishTime] ) a 
			on (a.idx = x.idx)
				and (a.[TaskId] = x.[TaskId]) 
				and (a.[DayOfWeek] = x.[DayOfWeek]) 
				and (a.[StartTime] = x.[StartTime]) 
				and (a.[FinishTime] = x.[FinishTime]) 	
		
Truncate table #tmpT
GO
				
---------				

Insert into #tmpT ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select [TaskId], [DayOfWeek], 
		DateAdd ( yyyy, 2000-DatePart(yyyy, [StartTime]), [StartTime]), 
		DateAdd ( yyyy, 2000-DatePart(yyyy, [FinishTime]), [FinishTime])
	From [A_SvcTaskManager_RunPeriods]

Truncate table [dbo].[A_SvcTaskManager_RunPeriods]
	
Insert into [dbo].[A_SvcTaskManager_RunPeriods] ([TaskId], [DayOfWeek], [StartTime], [FinishTime])
	Select x.[TaskId], x.[DayOfWeek], x.[StartTime], x.[FinishTime]
	From #tmpT x
		Inner join ( Select [TaskId], [DayOfWeek], [StartTime], [FinishTime], Min([idx]) as idx
					 From #tmpT
					 Group by [TaskId], [DayOfWeek], [StartTime], [FinishTime] ) a 
			on (a.idx = x.idx)
				and (a.[TaskId] = x.[TaskId]) 
				and (a.[DayOfWeek] = x.[DayOfWeek]) 
				and (a.[StartTime] = x.[StartTime]) 
				and (a.[FinishTime] = x.[FinishTime]) 	
		
Truncate table #tmpT

---------		

Drop table #tmpT

---------		

if (not object_id ( 'tempdb..#tmpR' ) is null)
	Drop table #tmpR

CREATE TABLE #tmpR
(
	[idx] int identity (1,1),
	[IdRedir] uniqueidentifier not null,
	[DayOfWeek] int not null,
	[StartTime] datetime not null,
	[StopTime] datetime  not null
)

Insert into #tmpR ([IdRedir], [DayOfWeek], [StartTime], [StopTime])
	Select [IdRedir], [DayOfWeek], 
		DateAdd ( yyyy, 2000-DatePart(yyyy, [StartTime]), [StartTime]), 
		DateAdd ( yyyy, 2000-DatePart(yyyy, [StopTime]), [StopTime])
	From [A_Redirect_TimeTable]

Truncate table [dbo].[A_Redirect_TimeTable]
	
Insert into [dbo].[A_Redirect_TimeTable] ([IdRedir], [DayOfWeek], [StartTime], [StopTime])
	Select x.[IdRedir], x.[DayOfWeek], x.[StartTime], x.[StopTime]
	From #tmpR x
		Inner join ( Select [IdRedir], [DayOfWeek], [StartTime], [StopTime], Min([idx]) as idx
					 From #tmpR
					 Group by [IdRedir], [DayOfWeek], [StartTime], [StopTime] ) a 
			on (a.idx = x.idx)
				and (a.[IdRedir] = x.[IdRedir]) 
				and (a.[DayOfWeek] = x.[DayOfWeek]) 
				and (a.[StartTime] = x.[StartTime]) 
				and (a.[StopTime] = x.[StopTime]) 	
		
Truncate table #tmpR
				
---------	

Drop table #tmpR

---------
GO

-------------------------------------------------------------------------
use oktell
GO
	
Truncate table dbo.H_History
Truncate table dbo.H_Authorizeduserid_SPID
GO

-------------------------------------------------------------------------
--if exists ( Select * from sysobjects where id = object_id ( 'dbo.A'))
--	Drop procedure dbo.A
--GO
