﻿
If Object_id('dbo.A_CallCenter_GetJournal') Is not Null  
    Drop Procedure dbo.A_CallCenter_GetJournal
GO

Create Procedure dbo.A_CallCenter_GetJournal
    @authorizeduserid uniqueidentifier,
    @datestart datetime,
    @datestop datetime,
    @timestart float,
    @timestop float,
    @lentime int = NULL,
    @xmltasks ntext = NULL,
    @xmlopers ntext = NULL,
    @xmllines ntext = NULL,
    @abonentnumber nvarchar(100) = NULL,
    @comment nvarchar(2000) = NULL,
    @conntype int = NULL
as
begin

	-- Права доступа пользователя к проектам
    if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#privilegesp' ))
         Drop table #privilegesp
    Create table #privilegesp ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200))	
    Insert into #privilegesp ( [idObject], [idPrivilege], [code] )
        exec A_Policy_SubjectPrivileges_GetForUserByObjectType
            @iduser = @authorizeduserid,
            @objecttype = 101
    If object_id('tempdb..#tmp_ps') is not NULL
		Drop table #tmp_ps
	Create table #tmp_ps (id uniqueidentifier primary key)
	Insert into #tmp_ps
		Select Distinct [idObject]
		From #privilegesp

	-- Права доступа пользователя к задачам
    if exists ( Select * from tempdb..sysobjects where id = object_id ( 'tempdb..#privilegest' ))
        Drop table #privilegest
    Create table #privilegest ( [idObject] uniqueidentifier, [idPrivilege] uniqueidentifier, [code] nvarchar(200))	
    Insert into #privilegest ( [idObject], [idPrivilege], [code] )
        exec A_Policy_SubjectPrivileges_GetForUserByObjectType
            @iduser = @authorizeduserid,
            @objecttype = 102 
    If object_id('tempdb..#tmp_ts') is not NULL
		Drop table #tmp_ts
	Create table #tmp_ts (id uniqueidentifier primary key)
	Insert into #tmp_ts
		Select Distinct [idObject]
		From #privilegest
             
             
             
    -- Задачи фильтр (через XML)
	If object_id('tempdb..#tmp_tasks') is not NULL
		Drop table #tmp_tasks
	Create table #tmp_tasks (id uniqueidentifier primary key)
		
	if ( not @xmltasks is NULL ) begin
		Declare @idoc1 int
		Exec sp_xml_preparedocument @idoc1 OUTPUT, @xmltasks
		
		Insert into #tmp_tasks
			Select id
			From OpenXml (@idoc1, '/items/t', 2)	
			With ( id varchar(50) '@id' )
	end
		
		
	-- Операторы фильтр (через XML)
	If object_id('tempdb..#tmp_opers') is not NULL
		Drop table #tmp_opers
	Create table #tmp_opers (id uniqueidentifier primary key)
		
	if ( not @xmlopers is NULL ) begin
		Declare @idoc2 int
		Exec sp_xml_preparedocument @idoc2 OUTPUT, @xmlopers
		
		Insert into #tmp_opers
			Select id
			From OpenXml (@idoc2, '/items/o', 2)	
			With ( id varchar(50) '@id' )
	end
		
		
	-- Внешние линии фильтр (через XML)
	If object_id('tempdb..#tmp_lines') is not NULL
		Drop table #tmp_lines
	Create table #tmp_lines (id uniqueidentifier primary key)
		
	if ( not @xmllines is NULL ) begin
		Declare @idoc3 int
		Exec sp_xml_preparedocument @idoc3 OUTPUT, @xmllines
		
		Insert into #tmp_lines
			Select id
			From OpenXml (@idoc3, '/items/l', 2)	
			With ( id varchar(50) '@id' )
	end
             
    
    
    -- Журнал звонков
    Select * 
	From (
		Select Top 1000 idconn as Id, IdChain, DateTimeStart, AbonentNumber, 
			IsNull(p.Name + '. ' + t.Name, '') as Task, 
			Operator = case ec.IdOperator when 'AB000000-0000-0000-0000-000000000000' then 'IVR'
				else Coalesce(oi.Name, usr.Name, '') 
				end,
			LenTime, Cast ( IsRecorded as int ) as IsRecorded, IdRecDir, 
			ConnType = Case when ec.IsOutput=0 then 1 
					  when ec.IsOutput=1 then 2 
					 when ec.IsBetweenOp=1 then 3 
				else 0 
				end, 
			IsNull(ALineNum, '') as ALineNum, IsNull(BLineNum, '') as BLineNum, 
			Case when ( cmt.Id is NULL ) then 0 else 1 end as HasComment 
		From oktell_cc_temp..A_Cube_CC_EffortConnections ec 
			Left join oktell_cc_temp.dbo.A_Cube_CC_Cat_OperatorInfo oi on ec.IdOperator = oi.Id 
			Left join oktell_cc_temp.dbo.A_Cube_CC_Cat_Project p on p.Id = ec.IdProject 
			Left join oktell_cc_temp.dbo.A_Cube_CC_Cat_Task t on t.Id = ec.IdTask 
			Left join A_Stat_Comments cmt on cmt.Id = ec.IdConn			
			Left join A_Users usr On usr.Id = ec.IdOperator
			Inner join #tmp_ps ps on ec.IdProject = ps.Id 
			Inner join #tmp_ts ts on ec.IdTask = ts.Id 
			Left join #tmp_tasks ttasks on ttasks.id = ec.IdTask
			Left join #tmp_opers topers on topers.id = ec.IdOperator
			Left join #tmp_lines tlines on tlines.id = ec.IdExtLine
		Where (idconn is not NULL)
			and (DateStart between @datestart and @datestop ) 
			and (cast(TimeStart as float)-floor(cast(TimeStart as float)) between @timestart and @timestop)
			and ((@lentime is NULL) or (LenTime >= @lentime))
			and ((@xmltasks is NULL) or (not ttasks.id is null))
			and ((@xmlopers is NULL) or (not topers.id is null))
			and ((@xmllines is NULL) or (not tlines.id is null))
			and ((@abonentnumber is NULL) or (AbonentNumber like '%' + @abonentnumber + '%'))
			and ((@comment is NULL) or (Comment like '%' + @comment + '%'))
	) as xtab
	Where ((@conntype is NULL) or (@conntype = 0) or (ConnType = @conntype))

end
GO


--------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_Cube_CC_EffortConnections_Update_From_TmpCube' ))
	Drop procedure dbo.A_Cube_CC_EffortConnections_Update_From_TmpCube
GO

Create procedure [dbo].[A_Cube_CC_EffortConnections_Update_From_TmpCube]
as
begin

    Declare @last_id int, @curr_id int, @last_delete_dt datetime
    Select @last_id = curr_id,
           @last_delete_dt = last_delete_dt
    From [oktell_cc_temp]..tmp_cube_track
       
    -- Записи в [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections Держим только 1 день
    -- В полночь по серверному времени, произведется копирование очередной партии недостающих и чистка всех записей (в транзакции жесткой)
    If (DateDiff(dd, @last_delete_dt, GetDate()) > 0) begin
		
		-- В тразакции с удалением
        Begin Tran UpdAndTruncate
        Set Transaction Isolation Level Serializable
            
            Select @curr_id = max(id) 
            From [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections 
            
            If (@curr_id > @last_id)
				Insert into [oktell_cc_temp]..A_Cube_CC_EffortConnections 
					Select  IdProject,IdTask, IsOutput, IdEffort, IdOperator, IdInList, AbonentNumber, CallResult, CallResultInfo, DateTimeStart, DateStart, TimeStart, 
						DateTimeStop, IdConn, IdChain, IdAUser, IdBUser, IdExtLine, LenTime, LenQueue, IsBetweenOp, OpponentInfo, IsHandled, IsSuccess, 
						IsRecorded, IdRecDir, ALineNum, BLineNum, UserResult  
					From [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections
					Where id BETWEEN @last_id + 1 AND @curr_id
            
            -- Можно было бы делать Delete с условием и без транзакции, но тогда чаще
            Truncate table [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections
        
        Commit Tran UpdAndTruncate
        
        Update [oktell_cc_temp]..tmp_cube_track 
        Set last_delete_dt = GetDate(),
			last_id = @last_id, 
			curr_id = @curr_id        
    
    end else begin
    
		-- Простое копирование
        Select @curr_id = Max(id) 
        From [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections with (NOLOCK)
        
        If (@curr_id > @last_id)
			Insert [oktell_cc_temp]..A_Cube_CC_EffortConnections 
				Select IdProject, IdTask, IsOutput, IdEffort, IdOperator, IdInList, AbonentNumber, CallResult, CallResultInfo, DateTimeStart, DateStart, TimeStart, 
					DateTimeStop, IdConn, IdChain, IdAUser, IdBUser, IdExtLine, LenTime, LenQueue, IsBetweenOp, OpponentInfo, IsHandled, IsSuccess,
					IsRecorded, IdRecDir, ALineNum, BLineNum, UserResult  
				From [oktell_cc_temp]..Tmp_A_Cube_CC_EffortConnections with (NOLOCK)
				Where id BETWEEN @last_id + 1 AND @curr_id

        Update [oktell_cc_temp]..tmp_cube_track 
        Set last_id = @last_id + 1, 
			curr_id = @curr_id
    
    end
    
end
GO


--------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_Temp_AddMissedIncomingCallInfo' ))
	Drop procedure dbo.A_CallCenter_Temp_AddMissedIncomingCallInfo
GO

Create procedure [dbo].[A_CallCenter_Temp_AddMissedIncomingCallInfo]
	@idtask uniqueidentifier,	
	@ideffort uniqueidentifier,
	@idinlist int,
	@abonentnumber nvarchar ( 20 ),
	@callresult int,
	@timestart datetime, 
	@timestop datetime,
	@idchain uniqueidentifier,
	@idextline uniqueidentifier
	
as 
begin
	Declare @idproject uniqueidentifier
	Set @idproject = '00000000-0000-0000-0000-000000000000'
	Select @idproject = IdProject 
	From A_TaskManager_Tasks
	Where Id = @idtask

	Insert Into [oktell_cc_temp].[dbo].[Tmp_A_Cube_CC_EffortConnections] With ( TabLock )
	(
		IdProject,
		IdTask,
		IsOutput,
		IdEffort,
		IdInList,
		AbonentNumber,
		CallResult,
		CallResultInfo,
		DateTimeStart,
		DateStart,
		TimeStart,
		DateTimeStop,
		IdChain,
		IdExtLine,
		LenTime,
		LenQueue,
		IsHandled,
		IsSuccess
	)
	Values 
	( 
		@idproject,
		@idtask,
		0,
		@ideffort,
		@idinlist,
		@abonentnumber,
		@callresult,
		'',		
		@timestart, 
		Cast ( Floor ( Cast ( @timestart as Float ) ) as DateTime ),
		Cast ( Cast ( @timestart as Float ) - FLOOR ( Cast ( @timestart as Float ) ) as DateTime ),
		@timestop,
		@idchain,
		@idextline,
		Cast ( @timestop - @timestart as float ) * 86400,
		Cast ( @timestop - @timestart as float ) * 86400,
		0,
		0
	)
end
GO


--------------------------------------------------------------------
if exists ( Select * From sysobjects where id = object_id ( 'dbo.A_CallCenter_Temp_CreateTables' ))
	Drop procedure dbo.[A_CallCenter_Temp_CreateTables]
GO

Create Procedure [dbo].[A_CallCenter_Temp_CreateTables]
as
begin

	If not exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( '[oktell_cc_temp].[dbo].[A_Cube_CC_EffortConnections]' ) )
	begin
	Create table [oktell_cc_temp].[dbo].[A_Cube_CC_EffortConnections] 
	( 
		[IdProject] uniqueidentifier NOT NULL,
		[IdTask] uniqueidentifier NOT NULL,
		[IsOutput] bit NOT NULL,
		[IdEffort] uniqueidentifier NOT NULL,
		[IdOperator] uniqueidentifier,
		[IdInList] int NOT NULL,
		[AbonentNumber] nvarchar ( 20 ) collate database_default NOT NULL,
		[CallResult] int NOT NULL,
		[CallResultInfo] nvarchar ( 100 )  collate database_default Not NULL,
		[DateTimeStart] DateTime NOT NULL, 
		[DateStart] DateTime NOT NULL, 
		[TimeStart] DateTime NOT NULL,
		[DateTimeStop] DateTime Not Null,
		[IdConn] uniqueidentifier,
		[IdChain] uniqueidentifier,	
		[IdAUser] uniqueidentifier,
		[IdBUser] uniqueidentifier,
		[IdExtLine] uniqueidentifier NOT NULL,
		[LenTime] float NOT NULL, 
		[LenQueue] float NOT NULL,
		[IsBetweenOp] bit,
		[OpponentInfo] nvarchar ( 200 ) collate database_default,
		[IsHandled] bit NOT NULL,
		[IsSuccess] bit Not NULL,
		[IsRecorded] bit,
		[IdRecDir] int,
		[ALineNum] nvarchar ( 12 ) collate database_default,
		[BLineNum] nvarchar ( 12 ) collate database_default,
		--[ARecFile] nvarchar ( 100 ) collate database_default,
		--[BRecFile] nvarchar ( 100 ) collate database_default,
		[UserResult] int
	)	
	end
	
	if not exists ( Select * FROM oktell_cc_temp..sysobjects Where Id = object_id ( '[oktell_cc_temp].[dbo].[Tmp_A_Cube_CC_EffortConnections]' ) )
	CREATE TABLE oktell_cc_temp.[dbo].[Tmp_A_Cube_CC_EffortConnections] (
		[IdProject] uniqueidentifier NOT NULL,
		[IdTask] uniqueidentifier NOT NULL,
		[IsOutput] bit NOT NULL,
		[IdEffort] uniqueidentifier NOT NULL,
		[IdOperator] uniqueidentifier NULL,
		[IdInList] int NOT NULL,
		[AbonentNumber] nvarchar(20) NOT NULL,
		[CallResult] int NOT NULL,
		[CallResultInfo] nvarchar(100) NOT NULL,
		[DateTimeStart] datetime NOT NULL,
		[DateStart] datetime NOT NULL,
		[TimeStart] datetime NOT NULL,
		[DateTimeStop] datetime NOT NULL,
		[IdConn] uniqueidentifier NULL,
		[IdChain] uniqueidentifier NULL,
		[IdAUser] uniqueidentifier NULL,
		[IdBUser] uniqueidentifier NULL,
		[IdExtLine] uniqueidentifier NOT NULL,
		[LenTime] float NOT NULL,
		[LenQueue] float NOT NULL,
		[IsBetweenOp] bit NULL,
		[OpponentInfo] nvarchar(100) NULL,
		[IsHandled] bit NOT NULL,
		[IsSuccess] bit NOT NULL,
		[IsRecorded] bit NULL,
		[IdRecDir] int NULL,
		[ALineNum] nvarchar(12) NULL,
		[BLineNum] nvarchar(12) NULL,
		[UserResult] int NULL,
		[id] bigint IDENTITY(1, 1) NOT NULL,
		CONSTRAINT [Tmp_A_Cube_CC_EffortConnections_id_PK]
		PRIMARY KEY CLUSTERED ([id] ASC)
	)
	
	if not exists ( Select * FROM oktell_cc_temp..sysobjects Where Id = object_id ( '[oktell_cc_temp].[dbo].[tmp_cube_track]' ) )
	CREATE TABLE [oktell_cc_temp].[dbo].[tmp_cube_track] 
	(
		[last_id] BIGINT NULL,
		[curr_id] BIGINT NULL,
		last_delete_dt DATETIME
	)


	If not exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( '[oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates]' ) )
	begin
	Create table [oktell_cc_temp].[dbo].[A_Cube_CC_OperatorStates] 
	( 		
		[Id] int identity ( 1, 1 ) not null primary key,
		[IdTask] uniqueidentifier,
		[IsOutput] bit,	
		[IdOperator] uniqueidentifier NOT NULL,
		[CallResult] int,
		[State] int Not Null,
		[DateTimeStart] DateTime NOT NULL, 
		[DateStart] DateTime NOT NULL, 
		[TimeStart] DateTime NOT NULL,
		[DateTimeStop] DateTime NOT NULL, 
		[LenTime] float NOT NULL,
		[ICode] int NULL,
		[IdEffort] uniqueidentifier NULL
	)
	end


	If not exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( '[oktell_cc_temp].[dbo].[A_Cube_CC_CallQueue]' ) )
	begin
	Create table [oktell_cc_temp].[dbo].[A_Cube_CC_CallQueue] 
	( 
		[IdTask] uniqueidentifier NOT NULL,
		[IdProject] uniqueidentifier Not NULL,
		[DateTimeStart] DateTime NOT NULL, 
		[DateStart] DateTime NOT NULL, 
		[TimeStart] DateTime NOT NULL,
		[CountQueue] int Not Null,
		[CountOperator] int Not Null,
		[CountTalking] int Not Null,
		[elt1WaitStart] DateTime NOT NULL,
		[elt1WaitSec] int Not null,
	)
	end



	If not exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( '[oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorStateTypes]' ) )
	Create table [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorStateTypes] 
	(
		[Id] int not null primary key,
		[Name] nvarchar ( 500 ) collate database_default not null
	)

	If not exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( '[oktell_cc_temp].[dbo].[A_Cube_CC_Cat_TaskResultTypes]' ) )
	Create table [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_TaskResultTypes] 
	(
		[Id] int not null primary key,
		[Name] nvarchar ( 500 ) collate database_default not null
	)

	If not exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( '[oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task]' ) )
	Create table [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Task] 
	(
		[Id] uniqueidentifier not null primary key,
		[Name] nvarchar ( 500 ) collate database_default not null,
		[Type] int,
		[Isoutput] bit
		
	)

	If not exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( '[oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project]' ) )
	Create table [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_Project] 
	(
		[Id] uniqueidentifier not null primary key,
		[Name] nvarchar ( 500 ) collate database_default not null
	)

	If not exists ( Select * From oktell_cc_temp..sysobjects where id = object_id ( '[oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo]' ) )
	Create table [oktell_cc_temp].[dbo].[A_Cube_CC_Cat_OperatorInfo] 
	(
		[Id] uniqueidentifier not null primary key,
		[Name] nvarchar ( 500 ) collate database_default,
		p0 int,	p1 int, p2 int, p3 int, p4 int, p5 int,
		s0 int,	s1 int, s2 int, s3 int, s4 int, s5 int,
		[LastDT] DateTime
	)

	if not exists ( Select * FROM oktell_cc_temp..sysobjects Where Id = object_id ( '[oktell_cc_temp].[dbo].[A_Sms]' ) )
	Create table oktell_cc_temp.dbo.A_Sms
	(	
		[Id] int identity ( 1, 1 ) not null primary key,	
		[Received] bit not null,
		[DT] DateTime not null,
		[Sender] nvarchar ( 100 ) collate database_default not null,
		[Receiver] nvarchar ( 100 ) collate database_default not null,
		[SmsText] nvarchar ( 2000 ) collate database_default not null,
		[PushId] int null,
		[Delivered] bit null,
		[DeliveredDT] DateTime null
	)	
	
end
GO