﻿-------------------------------------------------------------------------
if not (object_id( 'dbo.A_TaskManager_List_DirectDB_CacheCreate' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_CacheCreate
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_CacheCreate]
	@tableidx nvarchar(200),
	@tableacindex nvarchar(200),
	@idfield nvarchar(200),
	@query nvarchar(4000) = '',
	@table nvarchar(500) = '',
	@taskid uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin
	-- Внутренняя вспомогательная.
	-- Производит создание и/или наполнение БД-кэша таблицы абонентов 
	
	Declare @sql nvarchar(4000)
	Declare @iscreating bit 
	Set @iscreating = 0
	
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) ) begin
				
		-- cоздаем
		Set @iscreating = 1
		Set @sql = '
			Create table ' + @tableidx + ' ( Id int primary key, AC int, State int, NIdx int )
	
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableacindex + ']'' ) and name = ''' + @tableacindex + ''' )
				Create Nonclustered Index [' + @tableacindex + '] ON ' + @tableidx + '
				( AC, State ) With FillFactor = 80	
			'

		exec sp_sqlexec @sql
		
	end
	
	-- Если идет синхронизация, а не создание, то
	--   Добавить те записи, которых нет в кэше
	
	-- Вставляем в цикле по 100000 записей чтобы не переполнить transactionlog
	
	-- Запрос|таблица
	Declare @fromstring nvarchar(4000)
	if ( not @query like '' )
		Set @fromstring = ' ( ' + @query + ') t '
	else
		Set @fromstring = ' ' + @table + ' t '
		
	-- Базовая часть
	Set @sql = 'Declare @k int
				Set @k = 1
				
				Select @k = IsNull ( Min ( AC ), 0 ) 
				From ' + @tableidx + '
				Where State = 0
				
				if ( @k > 0 )
					Set @k = @k - 1
				'
				
	-- Вставка новых записей	
	Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( Id, AC, State, NIdx )				
			 			Select top 100000 [' + @idfield + '], @k, 0, 0
						From ' + @fromstring + '
						Where not exists ( Select Id From ' + @tableidx + ' Where Id = t.[' + @idfield + '] )
				end
				'
				
	-- Обновление статусов вновь появившихся абонентов
	-- В режиме синхронизации существующего кэша
	if ( @iscreating = 0 ) begin
		
		Set @sql = @sql + '
				Set rowcount 100000				
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Update c
					Set c.State = IsNull(st.NResult, 0), 
						c.AC = @k
					From ' + @tableidx + ' c
						Inner join ' + @fromstring + ' on (c.Id = t.[' + @idfield + '])
						Left join A_TaskManager_SubTasks st on (st.IdInList = c.Id) and (st.TaskId = cast(''' + cast ( @taskid as nvarchar(100)) + ''' as uniqueidentifier))
					Where (c.State = 3)
				end
				Set rowcount 0
				'
	end

	exec sp_sqlexec @sql

end
GO

-------------------------------------------------------------------------
if not (object_id( 'dbo.A_TaskManager_List_DirectDB_CacheCreateP' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_CacheCreateP
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_CacheCreateP]
	@tableidx nvarchar(200),
	@tableacindex nvarchar(200),
	@idfield nvarchar(200),
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = '',	
	@query nvarchar(4000) = '',
	@table nvarchar(500) = '',
	@taskid uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin
	-- Внутренняя вспомогательная.
	-- Производит создание и/или наполнение БД-кэша таблицы абонентов
	
	Declare @sql nvarchar(4000)
	Declare @iscreating bit 
	Set @iscreating = 0
	
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) ) begin
				
		-- cоздаем
		Set @iscreating = 1
		Set @sql = '
			Create table ' + @tableidx + ' ( Id int primary key, AC int, State int, NIdx int )
	
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableacindex + ']'' ) and name = ''' + @tableacindex + ''' )
				Create Nonclustered Index [' + @tableacindex + '] ON ' + @tableidx + '
				( AC, State ) With FillFactor = 80	
			'

		exec sp_sqlexec @sql
		
	end
	
	-- Если идет синхронизация, а не создание, то
	--   Добавить те записи, которых нет в кэше		
	
	-- Вставляем в цикле по 100000 записей чтобы не переполнить transactionlog
	
	-- Запрос|таблица для подстановки в OPENROWSET
	Declare @fromstring nvarchar(4000)
	if ( not @query like '' )
		Set @fromstring = '(' + @query + ')a'
	else
		Set @fromstring = @table
		
	-- Базовая часть, удаление временной таблицы
	Set @sql = 'Declare @k int
				Set @k = 1
				
				Select @k = IsNull ( Min ( AC ), 0 ) 
				From ' + @tableidx + '
				Where State = 0
				
				if ( @k > 0 )
					Set @k = @k - 1
					
				if exists ( select * from tempdb..sysobjects where id=object_id(''tempdb..#tcache''))
					Drop table #tcache
				'
					
					
	-- Времянка из OPENROWSET
	Set @sql = @sql + '
				Select t.[' + @idfield + '] as idc into #tcache
				From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
					''Select [' + @idfield + '] From ' + @fromstring + ''') t 
				'				
			
	-- Вставка новых записей	
	Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( [Id], [AC], [State], [NIdx] )
		 				Select top 100000 [idc], @k, 0, 0
						From #tcache t
						Where not exists ( Select [Id] From ' + @tableidx + ' Where [Id] = t.[idc] )
				end
				'	
	
	-- Обновление статусов вновь появившихся абонентов
	-- В режиме синхронизации существующего кэша
	if ( @iscreating = 0 ) begin
	
		Set @sql = @sql + '
				Set rowcount 100000
				Set @k = @k				
				While ( @@rowcount > 0 ) begin
					Update c
					Set c.State = IsNull(st.NResult, 0), 
						c.AC = @k
					From ' + @tableidx + ' c
						Inner join #tcache t on (c.Id = t.[idc])
						Left join A_TaskManager_SubTasks st on (st.IdInList = c.Id) and (st.TaskId = cast(''' + cast ( @taskid as nvarchar(100)) + ''' as uniqueidentifier))
					Where (c.State = 3)
				end
				Set rowcount 0
				'
	end
	
	-- Удаление временной таблицы
	Set @sql = @sql + '
				Drop table #tcache
				'

	exec sp_sqlexec @sql
					
end
GO

---------------------------------------------------------------------------
if not (object_id( 'dbo.A_TaskManager_List_DistrDirectDB_CacheCreate' ) is null )
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_CacheCreate
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_CacheCreate]
	@tableidx nvarchar(200),
	@tableacindex nvarchar(200),
	@tableopindex nvarchar(200),
	@idfield nvarchar(200),
	@operatorfield nvarchar(200),
	@query nvarchar(4000) = '',
	@table nvarchar(500) = '',
	@taskid uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin
	-- Внутренняя вспомогательная.
	-- Производит создание и/или наполнение БД-кэша таблицы абонентов для задачи с закреплением абонентов за оператором
	
	Declare @sql nvarchar(4000)
	Declare @iscreating bit 
	Set @iscreating = 0
	
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) ) begin
				
		-- cоздаем
		Set @iscreating = 1
		Set @sql = '
			Create table ' + @tableidx + ' ( Id int primary key, AC int, State int, NIdx int, IdOperator uniqueidentifier )
	
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableacindex + ']'' ) and name = ''' + @tableacindex + ''' )
				Create Nonclustered Index [' + @tableacindex + '] ON ' + @tableidx + '
				( AC, State ) With FillFactor = 80	

			
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableopindex + ']'' ) and name = ''' + @tableopindex + ''' )
				Create Nonclustered Index [' + @tableopindex + '] ON ' + @tableidx + '
				( IdOperator ) With FillFactor = 80	
			'

		exec sp_executesql @sql
	
	end
	
	-- Если идет синхронизация, а не создание, то
	--   Добавить те записи, которых нет в кэше
	--   Обновить в кэше IdOperator и State=0 у тех у кого =3
	
	-- Вставляем в цикле по 100000 записей чтобы не переполнить transactionlog
	
	-- Запрос|таблица
	Declare @fromstring nvarchar(4000)
	if ( not @query like '' )
		Set @fromstring = '(' + @query + ') t'
	else
		Set @fromstring = @table + ' t'
		
	-- Формируем запрос для разных условий
	-- Базовая часть
	Set @sql = 'Declare @k int
				Set @k = 0
				'
		
	-- Вставка новых записей
	-- With|WO operators
	if ( @operatorfield like '' ) begin
			
		-- Без операторов
		Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( Id, AC, State, NIdx )				
	 					Select top 100000 [' + @idfield + '], @k, 0, 0
						From ' + @fromstring + '
						Where not exists ( Select [Id] From ' + @tableidx + ' Where [Id] = t.[' + @idfield + '] )
				end
				'
				
	end else begin
		
		-- С операторами
		Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( Id, AC, State, NIdx, IdOperator )				
	 					Select top 100000 [' + @idfield + '], @k, 0, 0, cast ( [' + @operatorfield + '] as uniqueidentifier )
						From ' + @fromstring + '
						Where not exists ( Select [Id] From ' + @tableidx + ' Where [Id] = t.[' + @idfield + '] )
				end
				'
	end
	
	-- State надо обновить по оперативной статистике (20.12.2013)
	if ( @taskid != '00000000-0000-0000-0000-000000000000' ) begin
		Set @sql = @sql + '
				Update cache
				Set [State] = st.NResult
				From ' + @tableidx + ' cache, A_TaskManager_SubTasks st
				Where (st.TaskId = cast(''' + cast ( @taskid as nvarchar(100)) + ''' as uniqueidentifier))
					and (cache.[Id] = st.[IdInList])
				'
	end
	
	-- В режиме синхронизации существующего кэша
	if ( @iscreating = 0 ) begin
		
		-- Обновление привязки к операторам
		if ( not @operatorfield like '' ) begin
			Set @sql = @sql + '
				Set rowcount 100000
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Update ct
					Set c.IdOperator = t.[' + @operatorfield + '],
						c.AC = @k
					From ' + @tableidx + ' c, ' + @fromstring + '
					Where (c.Id = t.[' + @idfield + ']) 
						and (c.State != 1)
						and (c.IdOperator != t.[' + @operatorfield + '])
				end
				Set rowcount 0
				'
		end
	
		-- Обновление статусов вновь появившихся абонентов
		Set @sql = @sql + '
				Set rowcount 100000
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Update c
					Set c.State = IsNull(st.NResult, 0), 
						c.AC = @k
					From ' + @tableidx + ' c
						Inner join ' + @fromstring + ' on (c.Id = t.[' + @idfield + '])
						Left join A_TaskManager_SubTasks st on (st.IdInList = c.Id) and (st.TaskId = cast(''' + cast ( @taskid as nvarchar(100)) + ''' as uniqueidentifier))
					Where (c.State = 3)
				end
				Set rowcount 0
				'
	end
	
	exec sp_executesql @sql
				
end
GO

-------------------------------------------------------------------------
if not (object_id( 'dbo.A_TaskManager_List_DistrDirectDB_CacheCreateP' ) is null )
	Drop procedure dbo.A_TaskManager_List_DistrDirectDB_CacheCreateP
GO

Create procedure [dbo].[A_TaskManager_List_DistrDirectDB_CacheCreateP]
	@tableidx nvarchar(200),
	@tableacindex nvarchar(200),
	@tableopindex nvarchar(200),
	@idfield nvarchar(200),
	@operatorfield nvarchar(200),
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = '',
	@query nvarchar(4000) = '',
	@table nvarchar(500) = '',
	@taskid uniqueidentifier = '00000000-0000-0000-0000-000000000000'
as
begin
	-- Внутренняя вспомогательная.
	-- Производит создание и/или наполнение БД-кэша таблицы абонентов для задачи с закреплением абонентов за оператором

	Declare @sql nvarchar(4000)
	Declare @iscreating bit 
	Set @iscreating = 0

	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) ) begin
		
		-- cоздаем
		Set @iscreating = 1
		Set @sql = '
			Create table ' + @tableidx + ' ( Id int primary key, AC int, State int, NIdx int, IdOperator uniqueidentifier )
	
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableacindex + ']'' ) and name = ''' + @tableacindex + ''' )
				Create Nonclustered Index [' + @tableacindex + '] ON ' + @tableidx + '
				( AC, State ) With FillFactor = 80	

			
			if not exists ( select name from sysindexes where id=object_id ( ''dbo.[' + @tableopindex + ']'' ) and name = ''' + @tableopindex + ''' )
				Create Nonclustered Index [' + @tableopindex + '] ON ' + @tableidx + '
				( IdOperator ) With FillFactor = 80	
			'

		exec sp_executesql @sql
		
	end
	
	-- Если идет синхронизация, а не создание, то
	--   Добавить те записи, которых нет в кэше
	--   Обновить в кэше IdOperator и State=0 у тех у кого =3
	
	-- Вставляем в цикле по 100000 записей чтобы не переполнить transactionlog
	
	-- Запрос|таблица для подстановки в OPENROWSET
	Declare @fromstring nvarchar(4000)
	if ( not @query like '' )
		Set @fromstring = '(' + @query + ')a'
	else
		Set @fromstring = @table
		
	-- Формируем запрос для разных условий
	-- Базовая часть, удаление временной таблицы
	Set @sql = '			
			Declare @k int
			Set @k = 0
				
			if exists ( select * from tempdb..sysobjects where id=object_id(''tempdb..#tcache''))
				Drop table #tcache
			'
	
	-- Времянка из OPENROWSET
	-- With|WO operators
	if ( @operatorfield like '' ) begin		
			
		-- Без операторов
		Set @sql = @sql + '
			Select t.[' + @idfield + '] as idc into #tcache
			From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
				''Select [' + @idfield + '] From ' + @fromstring + ''') t 
			'
	end else begin
			
		-- С операторами
		Set @sql = @sql + '
			Select t.[' + @idfield + '] as idc, t.[' + @operatorfield + '] as idoperator into #tcache
			From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
				''Select [' + @idfield + '], [' + @operatorfield + '] From ' + @fromstring + ''') t
			'
	end				
	
							
	-- Вставка новых записей
	-- With|WO operators
	if ( @operatorfield like '' ) begin			
		
		-- Без операторов
		Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( [Id], [AC], [State], [NIdx] )
		 				Select top 100000 [idc], @k, 0, 0
						From #tcache t
						Where not exists ( Select [Id] From ' + @tableidx + ' Where [Id] = t.[idc] )
				end
				'
				
	end else begin		
		
		-- С операторами
		Set @sql = @sql + '
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Insert into ' + @tableidx + ' ( [Id], [AC], [State], [NIdx], [IdOperator] )				
		 				Select top 100000 [idc], @k, 0, 0, cast ( [idoperator] as uniqueidentifier )
						From #tcache t
						Where not exists ( Select [Id] From ' + @tableidx + ' Where [Id] = t.[idc] )
				end
				'		
	end
	
	-- State надо обновить по оперативной статистике (20.12.2013)
	if ( @taskid != '00000000-0000-0000-0000-000000000000' ) begin
		Set @sql = @sql + '
				Update cache
				Set [State] = st.NResult
				From ' + @tableidx + ' cache, A_TaskManager_SubTasks st
				Where (st.TaskId = cast(''' + cast ( @taskid as nvarchar(100)) + ''' as uniqueidentifier))
					and (cache.[Id] = st.[IdInList])
				'
	end
	
	-- В режиме синхронизации существующего кэша
	if ( @iscreating = 0 ) begin
	
		-- Обновление привязки к операторам
		if ( not @operatorfield like '' )
			Set @sql = @sql + '
				Set rowcount 100000
				Set @k = @k				
				While ( @@rowcount > 0 ) begin
					Update c
					Set c.IdOperator = t.idoperator,
						c.AC = @k
					From ' + @tableidx + ' c, #tcache t
					Where (c.Id = t.[idc]) 
						and (c.State != 1)
						and (c.IdOperator != t.idoperator)
				end
				Set rowcount 0
				'
				
		-- Обновление статусов вновь появившихся абонентов
		Set @sql = @sql + '
				Set rowcount 100000
				Set @k = @k
				While ( @@rowcount > 0 ) begin
					Update c
					Set c.State = IsNull(st.NResult, 0), 
						c.AC = @k
					From ' + @tableidx + ' c
						Inner join #tcache t on (c.Id = t.[idc])
						Left join A_TaskManager_SubTasks st on (st.IdInList = c.Id) and (st.TaskId = cast(''' + cast ( @taskid as nvarchar(100)) + ''' as uniqueidentifier))
					Where (c.State = 3)
				end
				Set rowcount 0
				'
	end
				
	-- Удаление временной таблицы
	Set @sql = @sql + '
				Drop table #tcache
				'				
	
	exec sp_executesql @sql

end
GO

-------------------------------------------------------------------------
-------------------------------------------------------------------------
-------------------------------------------------------------------------
if not (object_id ( 'dbo.A_TaskManager_List_DirectDB_GetQuery' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_GetQuery
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_GetQuery]
	@taskid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 100 ) = 'Id'
as
begin
	-- Производит запрос следующего абонента для совершения звонка из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через запрос на локальном сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_Idx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @sql nvarchar ( 4000 )

	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DirectDB_CacheCreate
			@tableidx = @tableidx,
			@tableacindex = @tableacindex,
			@idfield = @idfield,
			@query = @query,
			@table = '',
			@taskid = @taskid
	
		

	-- берем значение минимального количества попыток по абонентам
	set @sql = ' 
		Declare @min int
		Select @min = Min ( AC ) 
		From ' + @tableidx + '  
		Where State = 0
		'
	-- берем первый индекс с таким количеством попыток (по списку)
	-- и достаем из таблицы строки для этого абонента
	set @sql = @sql + '
		--ТРАНЗАКЦИЯ В ОБЩЕМ ТО НЕ НУЖНА, ТАК КАК ИЗ-ПОД ЛОКА В ПУЛЕ ОДНОЙ ЗАДАЧИ
		begin tran search_update_q

		Declare @id int, @efc int, @st int, @ln int
		Select top 1 @Id = Id, @efc = IsNull( AC, 0 ), @st = IsNull ( State, 0 ), @ln = IsNull ( NIdx, 0 )
		From ' + @tableidx + ' with ( Index ( [' + @tableacindex + '] ) )
		Where AC = @min
			and State = 0

		Update ' + @tableidx + ' 
		Set AC = AC + 1, NIdx = NIdx + 1
		Where Id = @id

		commit tran search_update_q
		
		Select @efc, @st, @ln, ' + @fields + '
		From ( ' + @query + ' )a
		Where [' + @idfield + '] = @id
		'

	exec sp_sqlexec @sql
end
GO

-------------------------------------------------------------------------
if not (object_id ( 'dbo.A_TaskManager_List_DirectDB_GetQueryP' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_GetQueryP
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_GetQueryP]
	@taskid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 100 ) = 'Id',
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = ''
as
begin
	-- Производит запрос следующего абонента для совершения звонка из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через запрос на стороннем сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_Idx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @sql nvarchar ( 4000 )

	--Set @sql = 'Drop table ' + @tableidx
	--exec sp_executesql @sql

	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DirectDB_CacheCreateP
			@tableidx = @tableidx,
			@tableacindex = @tableacindex,
			@idfield = @idfield,
			@providername = @providername,
			@connectstring = @connectstring,
			@query = @query,
			@table = '',
			@taskid = @taskid
	

	-- берем значение минимального количества попыток по абонентам
	set @sql = ' 
		Declare @min int
		Select @min = Min ( AC ) 
		From ' + @tableidx + '  
		Where State = 0
		'
	-- берем первый индекс с таким количеством попыток (по списку)
	-- и возвращаем его для организации запроса к исходной удаленной таблице
	set @sql = @sql + '
		--ТРАНЗАКЦИЯ В ОБЩЕМ ТО НЕ НУЖНА, ТАК КАК ИЗ-ПОД ЛОКА В ПУЛЕ ОДНОЙ ЗАДАЧИ
		begin tran search_update_q

		Declare @id int, @efc int, @st int, @ln int
		Select top 1 @Id = Id, @efc = IsNull( AC, 0 ), @st = IsNull ( State, 0 ), @ln = IsNull ( NIdx, 0 )
		From ' + @tableidx + ' with ( Index ( [' + @tableacindex + '] ) )
		Where AC = @min
			and State = 0

		Update ' + @tableidx + ' 
		Set AC = AC + 1, NIdx = NIdx + 1
		Where Id = @id

		commit tran search_update_q

		Select @id, @efc, @st, @ln
		'

	-- Через времяночку индекс абонента получаем и дополнительные показатели
	if exists ( select * from tempdb..sysobjects where id=object_id('tempdb..#tx'))
	    Drop table #tx
	Create table #tx ( id int, efc int, st int, ln int )

	Insert into #tx ( id, efc, st, ln )
	    exec sp_executesql @sql

	Declare @id int, @efc int, @st int, @ln int
	Select @id = id, @efc = efc, @st = st, @ln = ln 
	From #tx
	
	Drop table #tx
	
	-- достаем из исходной таблицы строчку с указанным идентификатором
	set @sql = '
		Select ' + Cast ( @efc as nvarchar(10)) + ' as efc, ' + Cast ( @st as nvarchar(10)) + ' as st, ' + Cast ( @ln as nvarchar(10)) + ' as ln, ' + @fields + '
		From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
			''Select ' + @fields + ' From (' + @query + ')a Where [' + @idfield + '] = ' + cast ( @id as nvarchar(10))+ ''') t 
		'

	exec sp_executesql @sql

end
GO

-------------------------------------------------------------------------
if not (object_id ( 'dbo.A_TaskManager_List_DirectDB_GetTableP' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_GetTableP
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_GetTableP]
	@taskid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 1000 ),
	@idfield nvarchar ( 100 ) = 'Id',
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = ''
as
begin
	-- Производит запрос следующего абонента для совершения звонка из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через таблицу на стороннем сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_Idx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @sql nvarchar ( 4000 )

	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DirectDB_CacheCreateP
			@tableidx = @tableidx,
			@tableacindex = @tableacindex,
			@idfield = @idfield,
			@providername = @providername,
			@connectstring = @connectstring,
			@query = '',
			@table = @table,
			@taskid = @taskid
			
		

	-- берем значение минимального количества попыток по абонентам
	set @sql = '	
		Declare @min int
		Select @min = Min ( AC ) 
		From ' + @tableidx + '  
		Where State = 0
		'
		
	-- берем первый индекс с таким количеством попыток (по списку)
	-- и возвращаем его для организации запроса к исходной удаленной таблице
	set @sql = @sql + '
		--ТРАНЗАКЦИЯ В ОБЩЕМ ТО НЕ НУЖНА, ТАК КАК ИЗ-ПОД ЛОКА В ПУЛЕ ОДНОЙ ЗАДАЧИ
		begin tran search_update 

		Declare @id int, @efc int, @st int, @ln int
		Select top 1  @Id = Id, @efc = IsNull( AC, 0 ), @st = IsNull ( State, 0 ), @ln = IsNull ( NIdx, 0 )
		From ' + @tableidx + ' with ( Index ( [' + @tableacindex + '] ) )
		Where AC = @min
			and State = 0		

		Update ' + @tableidx + ' 
		Set AC = AC + 1, NIdx = NIdx + 1
		Where Id = @id

		commit tran search_update
		
		Select @id, @efc, @st, @ln
		'

	-- Через времяночку индекс абонента получаем и дополнительные показатели
	if exists ( select * from tempdb..sysobjects where id=object_id('tempdb..#tx'))
	    Drop table #tx
	Create table #tx ( id int, efc int, st int, ln int )

	Insert into #tx ( id, efc, st, ln )
	    exec sp_executesql @sql

	Declare @id int, @efc int, @st int, @ln int
	Select @id = id, @efc = efc, @st = st, @ln = ln 
	From #tx
	
	Drop table #tx
	
	-- достаем из исходной таблицы строчку с указанным идентификатором
	set @sql = '
		Select ' + Cast ( @efc as nvarchar(10)) + ' as efc, ' + Cast ( @st as nvarchar(10)) + ' as st, ' + Cast ( @ln as nvarchar(10)) + ' as ln, ' + @fields + '
		From OPENROWSET(''' + @providername + ''',''' + @connectstring + ''', 
			''Select ' + @fields + ' From ' + @table + ' Where [' + @idfield + '] = ' + cast ( @id as nvarchar(10))+ ''') t 
		'

	exec sp_executesql @sql
	
end
GO

-------------------------------------------------------------------------
if not (object_id ( 'dbo.A_TaskManager_List_DirectDB_GetTable' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_GetTable
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_GetTable]
	@taskid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 1000 ),
	@idfield nvarchar ( 100 ) = 'Id'
as
begin

	-- Производит запрос следующего абонента для совершения звонка из кэша в БД 
	-- При отсутствии кэша производит создание и начальную синхронизацию с таблицей абонентов 
	--   заданной через таблицу на локальном сервере
	
	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_Idx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @sql nvarchar ( 4000 )

	-- создаем таблицу для задачи, если ее нет еще
	if not exists ( select * from sysobjects where id = object_id ( @tableidx ) )
		exec A_TaskManager_List_DirectDB_CacheCreate
			@tableidx = @tableidx,
			@tableacindex = @tableacindex,
			@idfield = @idfield,
			@query = '',
			@table = @table,
			@taskid = @taskid
		

	-- берем значение минимального количества попыток по абонентам
	set @sql = '
		Declare @min int
		Select @min = Min ( AC ) 
		From ' + @tableidx + '  
		Where State = 0
		'
		
	-- берем первый индекс с таким количеством попыток (по списку)
	-- и достаем из таблицы строки для этого абонента
	set @sql = @sql + '
		--ТРАНЗАКЦИЯ В ОБЩЕМ ТО НЕ НУЖНА, ТАК КАК ИЗ-ПОД ЛОКА В ПУЛЕ ОДНОЙ ЗАДАЧИ
		begin tran search_update 

		Declare @id int, @efc int, @st int, @ln int
		Select top 1  @Id = Id, @efc = IsNull( AC, 0 ), @st = IsNull ( State, 0 ), @ln = IsNull ( NIdx, 0 )
		From ' + @tableidx + ' with ( Index ( [' + @tableacindex + '] ) )
		Where AC = @min
			and State = 0		

		Update ' + @tableidx + ' 
		Set AC = AC + 1, NIdx = NIdx + 1
		Where Id = @id

		commit tran search_update
		
		Select @efc, @st, @ln, ' + @fields + ' 
		From ' + @table + '
		Where [' + @idfield + '] = @id
		'
	
	exec sp_sqlexec @sql
end
GO

-------------------------------------------------------------------------
if not (object_id ( 'dbo.A_TaskManager_List_DirectDB_SyncQuery' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_SyncQuery
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_SyncQuery]
	@taskid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 100 ) = 'Id'
as
begin
	-- Производит синхронизацию кэша в БД с таблицей абонентов, заданной через запрос на локальном сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_Idx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	Declare @tablestindex nvarchar ( 200 )
	set @tableacindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'
	--set @tablestindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_ST'

	Declare @sql nvarchar ( 4000 )

	-- создаем таблицу для задачи, если ее нет еще, и наполняем значениями кэш, которых там нет
	exec A_TaskManager_List_DirectDB_CacheCreate
		@tableidx = @tableidx,
		@tableacindex = @tableacindex,
		@idfield = @idfield,
		@query = @query,
		@table = '',
		@taskid = @taskid
	
	
	-- обновляем на STATE = 3 удаленные из основной таблицы. (нужно только если количества не совпадают)
	Set @sql = '
		Declare @cnta int
		Select @cnta = Count ( [' + @idfield + '] )
		From (' + @query + ')a
		
		Declare @cntb int
		Select @cntb = Count ( Id )
		From ' + @tableidx + '
	
		if ( @cntb != @cnta )
			Update t
			Set State = 3
			From ' + @tableidx + ' t
			Where (State != 3)
				and not exists ( Select [' + @idfield + '] From ( ' + @query + ' ) a1 Where [' + @idfield + '] = t.[Id] )
		'

	exec sp_sqlexec @sql
end
GO


-------------------------------------------------------------------------
if not (object_id ( 'dbo.A_TaskManager_List_DirectDB_SyncQueryP' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_SyncQueryP
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_SyncQueryP]
	@taskid uniqueidentifier,
	@query nvarchar ( 4000 ),
	@fields nvarchar ( 2000 ),
	@idfield nvarchar ( 100 ) = 'Id',
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = ''
as
begin
	-- Производит синхронизацию кэша в БД с таблицей абонентов, заданной через запрос на стороннем сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_Idx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'


	Declare @sql nvarchar ( 4000 )

	-- создаем таблицу для задачи, если ее нет еще и наполняем значениями кэш которых в там нет
	exec A_TaskManager_List_DirectDB_CacheCreateP
		@tableidx = @tableidx,
		@tableacindex = @tableacindex,
		@idfield = @idfield,
		@providername = @providername,
		@connectstring = @connectstring,
		@query = @query,
		@table = '',
		@taskid = @taskid
	

	-- обновляем на STATE = 3 удаленные из основной таблицы. (нужно только если количества не совпадают)
	
	Set @sql = @sql + '
			Declare @cnta int
			Select @cnta = Count ( * ) 
			From #tcache
		
			Declare @cntb int
			Select @cntb = Count ( Id )
			From ' + @tableidx + '
	
			if ( @cntb != @cnta ) begin
				Update t
				Set State = 3
				From ' + @tableidx + ' t
				Where (State != 3)
					and not exists ( Select idc From #tcache Where idc = t.[Id] ) 
			end

			Drop table #tcache
			'

	exec sp_executesql @sql

end
GO


-------------------------------------------------------------------------
if not (object_id ( 'dbo.A_TaskManager_List_DirectDB_SyncTableP' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_SyncTableP
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_SyncTableP]
	@taskid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 500 ),
	@idfield nvarchar ( 50 ) = 'Id',
	@providername nvarchar(500) = 'SQLOLEDB',
	@connectstring nvarchar(4000) = ''
as
begin
	-- Производит синхронизацию кэша в БД с таблицей абонентов, заданной через таблицу на стороннем сервере

	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_Idx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	set @tableacindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'

	Declare @sql nvarchar ( 4000 )

	-- создаем таблицу для задачи, если ее нет еще
	exec A_TaskManager_List_DirectDB_CacheCreateP
		@tableidx = @tableidx,
		@tableacindex = @tableacindex,
		@idfield = @idfield,
		@providername = @providername,
		@connectstring = @connectstring,
		@query = '',
		@table = @table,
		@taskid = @taskid
			

	-- обновляем на STATE = 3 удаленные из основной таблицы. (нужно только если количества не совпадают)
	Set @sql = @sql + '

			Declare @cnta int
			Select @cnta = Count ( * ) 
			From #tcache
		
			Declare @cntb int
			Select @cntb = Count ( Id )
			From ' + @tableidx + '

			if ( @cnta < @cntb ) begin
				Update t
				Set State = 3
				From ' + @tableidx + ' t 
				Where (State != 3)
					and not exists ( Select idc From #tcache Where [idc] = t.[Id] )
			end

			Drop table #tcache
		'

	exec sp_executesql @sql

end
GO

-------------------------------------------------------------------------
if not (object_id ( 'dbo.A_TaskManager_List_DirectDB_SyncTable' ) is null )
	Drop procedure dbo.A_TaskManager_List_DirectDB_SyncTable
GO

Create procedure [dbo].[A_TaskManager_List_DirectDB_SyncTable]
	@taskid uniqueidentifier,
	@table nvarchar ( 200 ),
	@fields nvarchar ( 500 ),
	@idfield nvarchar ( 50 ) = 'Id'
as
begin
	-- Производит синхронизацию кэша в БД с таблицей абонентов, заданной через таблицу на локальном сервере
	
	Declare @tableidx nvarchar ( 200 )
	set @tableidx = 'dbo.[A_TaskManager_Idx_' + cast ( @taskid as nvarchar ( 50 ) ) + ']'
	Declare @tableacindex nvarchar ( 200 )
	Declare @tablestindex nvarchar ( 200 )
	set @tableacindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_AC_ST'
	--set @tablestindex = 'idxTASK_' + cast ( @taskid as nvarchar ( 50 ) ) + '_ST'

	Declare @sql nvarchar ( 4000 )

	-- создаем таблицу для задачи, если ее нет еще, и наполняем значениями кэш, которых в нем нет
	exec A_TaskManager_List_DirectDB_CacheCreate
		@tableidx = @tableidx,
		@tableacindex = @tableacindex,
		@idfield = @idfield,
		@query = '',
		@table = @table,
		@taskid = @taskid
	


	-- обновляем на STATE = 3 удаленные из основной таблицы. (нужно только если количества не совпадают)
	Set @sql = '
		Declare @cnta int
		Select @cnta = Count ( [' + @idfield + '] )
		From ' + @table + '
		
		Declare @cntb int
		Select @cntb = Count ( Id )
		From ' + @tableidx + '

		if ( @cnta < @cntb )
			Update t
			Set State = 3
			From ' + @tableidx + ' t
			Where (State != 3)
				and not exists ( Select [' + @idfield + '] From ' + @table + ' Where [' + @idfield + '] = t.[Id] )'

	exec sp_sqlexec @sql
end
GO
