	

if not (object_id('dbo.A_SmsTaskManager_UserInTask') is null)
	Drop table dbo.A_SmsTaskManager_UserInTask
GO

CREATE TABLE [dbo].[A_SmsTaskManager_UserInTask](
	[Id] [uniqueidentifier] NOT NULL Primary Key CLUSTERED,
	[UserId] [uniqueidentifier] NOT NULL,
	[TaskId] [uniqueidentifier] NOT NULL,
	[EffortId] [uniqueidentifier] NOT NULL,
	[TimeStart] [datetime] NOT NULL,
	[TimeStopBackRequest] [datetime] NULL,
	[TimeStopDialog] [datetime] NULL	
) ON [PRIMARY]
GO

	
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[A_SmsTaskManager_UserInTask]') AND name = N'idx_TTM_UIT_User_Start_Stop')
	DROP INDEX [idx_TTM_UIT_User_Start_Stop] ON [dbo].[A_SmsTaskManager_UserInTask] WITH ( ONLINE = OFF )
GO

CREATE NONCLUSTERED INDEX [idx_TTM_UIT_User_Start_Stop] ON [dbo].[A_SmsTaskManager_UserInTask] 
(
	[UserId],
	[TimeStart] ASC,
	[TimeStopDialog] ASC
)
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[A_SmsTaskManager_UserInTask]') AND name = N'idx_TTMUIT_complex')
	DROP INDEX [idx_TTMUIT_complex] ON [dbo].[A_SmsTaskManager_UserInTask] WITH ( ONLINE = OFF )
GO

CREATE NONCLUSTERED INDEX [idx_TTMUIT_complex] ON [dbo].[A_SmsTaskManager_UserInTask] 
(
	[EffortId] ASC,
	[TimeStopDialog] ASC,
	[TimeStopBackRequest] ASC	
)
GO

---------------------------------------------------

if not (object_id('dbo.A_SmsTaskManager_User_StartHandling') is null)
	Drop procedure dbo.A_SmsTaskManager_User_StartHandling
GO

Create procedure [dbo].[A_SmsTaskManager_User_StartHandling]
	@idreserve		uniqueidentifier,
	@userid			uniqueidentifier,
	@taskid			uniqueidentifier,
	@effortid		uniqueidentifier,
	@start			datetime	
as
begin	
	
	Insert Into A_SmsTaskManager_UserInTask
		( Id, UserId, TaskId, EffortId, TimeStart )
	Values 
		( @idreserve, @userid, @taskid, @effortid, @start )

end
GO

---------------------------------------------------

if not (object_id('dbo.A_SmsTaskManager_User_StopHandling') is null)
	Drop procedure dbo.A_SmsTaskManager_User_StopHandling
GO

Create procedure [dbo].[A_SmsTaskManager_User_StopHandling]
	@idreserve uniqueidentifier,
	@stop datetime
as
begin

	Update A_SmsTaskManager_UserInTask
	Set	TimeStopDialog = IsNull ( TimeStopDialog, @stop ),
		TimeStopBackRequest = IsNull ( TimeStopBackRequest, @stop )
	Where Id = @idreserve
	
end
GO

---------------------------------------------------

if not (object_id('dbo.A_SmsTaskManager_User_UpdateHandling') is null)
	Drop procedure dbo.[A_SmsTaskManager_User_UpdateHandling]
GO

Create procedure [dbo].[A_SmsTaskManager_User_UpdateHandling]
	@idreserve uniqueidentifier,
	@stopbackrequest datetime = NULL,
	@stopdialog datetime = NULL
as
begin

	Update A_SmsTaskManager_UserInTask
	Set TimeStopBackRequest = IsNull ( TimeStopBackRequest, @stopbackrequest ),
		TimeStopDialog = IsNull ( TimeStopDialog, @stopdialog )		
	Where Id = @idreserve
	
end
GO


if not (object_id('dbo.A_DB_InitOnStart') is null)
	Drop procedure dbo.A_DB_InitOnStart
GO

Create procedure [dbo].[A_DB_InitOnStart]
	@currentserver nvarchar(200) = NULL
As
begin
		-- On PBXStatusServer start (2)

		-----------------------------------------------------
		--    -----------
		-----------------------------------------------------
		
		Declare @lastpingtime datetime
		Set @lastpingtime = GetDate()
		select @lastpingtime = valueDateTime From A_Settings where [name] like 'PBXStatusServerPingTime'
		
		Declare @t1 table ( uid  uniqueidentifier ,  tm datetime )
		Insert into @t1
			Select UserId, Max ( TimeChange ) as TS FROM A_UserStateHistory Group By UserId
	

		Insert into A_UserStateHistory --with ( UpdLock )

		Select 	newid(), ush.UserId, 0, @lastpingtime, '', 0, 10104 --''
			From A_UserStateHistory ush inner join @t1 t on ush.UserId = t.uid and ush.TimeChange = t.tm
			Where ush.State != 0

		-----------------------------------------------------
		-- User in Task -    
		-----------------------------------------------------

		Update A_TaskManager_UserInTask with ( TabLock )
		Set	TimeStopPrior = IsNull ( TimeStopPrior, @lastpingtime ),
			TimeStopWaitCall = IsNull ( TimeStopWaitCall, @lastpingtime ),
			TimeStopBackCall = IsNull ( TimeStopBackCall, @lastpingtime ),
			TimeStopDialog = IsNull ( TimeStopDialog, @lastpingtime ),
			TimeStopPost = IsNull ( TimeStopPost, @lastpingtime )
		Where	( TimeStopPost is null ) or
			( TimeStopDialog is null ) or
			( TimeStopPrior is null ) or
			( TimeStopWaitCall is null ) or
			( TimeStopBackCall is null )
			
			
		Update A_SmsTaskManager_UserInTask with ( TabLock )
		Set	TimeStopBackRequest = IsNull ( TimeStopBackRequest, @lastpingtime ),
			TimeStopDialog = IsNull ( TimeStopDialog, @lastpingtime )
		Where
			( TimeStopDialog is null ) or			
			( TimeStopBackRequest is null )

		-----------------------------------------------------
		--        ,   
		-----------------------------------------------------

		Update A_CallCenter_UserStateHistory with ( TabLock )
		Set TimeLeave = @lastpingtime,
			LengthSec = Cast ( ( @lastpingtime - TimeEnter ) as float ) * 86400
		Where TimeLeave is NULL
		
		-----------------------------------------------------
		--  
		-----------------------------------------------------
		
		Delete From A_Monopoly
		Delete From A_MonopolyForTaskChanging
		Delete From C_MonopolyForTaskChanging /*obsolete*/

		-----------------------------------------------------
		--   ,  -    
		-----------------------------------------------------
		Delete 
		From A_TaskManager_Task_IncomingCalls
		Where ConnectionChainId = '00000000-0000-0000-0000-000000000000'

		-----------------------------------------------------
		--    
		-----------------------------------------------------

		Declare @llid int
		Declare @llname nvarchar ( 1000 )
		Declare @ltid uniqueidentifier
		Declare @ltids nvarchar ( 50 )
		Declare @sql nvarchar ( 500 )
		
		Declare tcursor Cursor For
			Select name, Id  
			From sysobjects 
			Where ([type] = 'U') 
				and (name like 'A_TaskManager_LocalList%')
		
		Open tcursor
		Fetch Next From tcursor 
					Into @llname, @llid

		While (@@FETCH_STATUS = 0) begin
		
			Set @llname = 'dbo.[' + @llname + ']'

			--   (id    Id  ) (      -       . N-1  )	
			--set @ltids = SUBSTRING ( @llname, 30, 36 )
			--set @ltid = cast ( @ltids as uniqueidentifier )
			--If not exists ( Select Id from A_Taskmanager_Lists Where Id = @ltid )
	
			--      JoinedTable  
			If not exists ( Select Id from A_TaskManager_Lists Where JoinedTable = @llname ) begin
				set @sql = ' DROP TABLE ' + @llname
				exec sp_executesql @stmt = @sql
			end

			Fetch Next From tcursor 
						Into @llname, @llid
		end

		Close tcursor
		Deallocate tcursor

		-----------------------------------------------------
		-----------------------------------------------------

		--   ??
		--         ?
		--     ?
		--   ..   ,     ..
		Exec dbo.A_CallCenter_Temp_OnLoad

		-----------------------------------------------------
		-----------------------------------------------------
		
		Declare @xsrv nvarchar(200)
		Set @xsrv = @currentserver
		
		Delete 
		From dbo.A_Registrar_BanList
		Where not (dtutccancel is null )
			and (DateDiff ( s, dtutc, GetUtcDate() ) > 180)
			and ((@xsrv is NULL) or (@xsrv = '') or (@xsrv = serverid))
			
		-----------------------------------------------------
		exec dbo.A_DB_UpdateOwnerUser
		
end
GO